@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -331,11 +331,9 @@ public:
 
     /// Return the size of this grid's voxels.
     Vec3d voxelSize() const { return transform().voxelSize(); }
-    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return transform().voxelSize(); }
     /// @brief Return the size of this grid's voxel at position (x, y, z).
     /// @note Frustum and perspective transforms have position-dependent voxel size.
     Vec3d voxelSize(const Vec3d& xyz) const { return transform().voxelSize(xyz); }
-    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d& xyz) const { return transform().voxelSize(xyz); }
     /// Return true if the voxels in world space are uniformly sized cubes
     bool hasUniformVoxels() const { return mTransform->hasUniformScale(); }
     //@{
@@ -488,7 +486,7 @@ public:
     struct ValueConverter {
         typedef Grid<typename TreeType::template ValueConverter<OtherValueType>::Type> Type;
     };
-    
+
     /// Return a new grid with the given background value.
     static Ptr create(const ValueType& background);
     /// Return a new grid with background value zero.
@@ -548,7 +546,7 @@ public:
     virtual Name valueType() const { return tree().valueType(); }
 
     /// Return this grid's background value.
-    ValueType background() const { return tree().getBackground(); }
+    const ValueType& background() const { return mTree->background(); }
     /// Replace this grid's background value.
     void setBackground(const ValueType& val) { tree().setBackground(val); }
 
@@ -597,12 +595,22 @@ public:
     /// operation for optimal sparseness.
     void fill(const CoordBBox& bbox, const ValueType& value, bool active = true);
 
-    /// @brief Set the values of all inactive voxels of a narrow-band level set
-    /// from the signs of the active values, setting outside values to +background
-    /// and inside values to -background.
-    /// @note This method should only be used on closed, narrow-band level sets!
+    /// @brief Set the values of all inactive voxels and tiles of a narrow-band
+    /// level set from the signs of the active voxels, setting outside values to
+    /// +background and inside values to -background.
+    ///
+    /// @note This operation should only be used on closed, narrow-band level sets!
     void signedFloodFill() { tree().signedFloodFill(); }
 
+    /// @brief Set the values of all inactive voxels and tiles of a narrow-band
+    /// level set from the signs of the active voxels, setting outside values to
+    /// @a outside and inside values to @a inside.
+    /// @details Also, set this grid's background value to @a outside.
+    ///
+    /// @note This operation should only be used on closed, narrow-band level sets!
+    /// Also, @a inside should be negative, and @a outside should be larger than @a inside.
+    void signedFloodFill(const ValueType& outside, const ValueType& inside);
+
     /// @brief Reduce the memory footprint of this grid by increasing its sparseness
     /// either losslessly (@a tolerance = 0) or lossily (@a tolerance > 0).
     /// @details With @a tolerance > 0, sparsify regions where voxels have the same
@@ -618,16 +626,22 @@ public:
 
     /// @brief Union this grid's set of active values with the active values
     /// of the other grid, whose value type may be different.
+    /// @details The resulting state of a value is active if the corresponding value
+    /// was already active OR if it is active in the other grid. Also, a resulting
+    /// value maps to a voxel if the corresponding value already mapped to a voxel
+    /// OR if it is a voxel in the other grid. Thus, a resulting value can only
+    /// map to a tile if the corresponding value already mapped to a tile
+    /// AND if it is a tile value in other grid.
     ///
-    /// @note Values in this grid are not modified, only their state.
-    /// That is, active values in this grid are untouched, and values that
-    /// were inactive in this grid but active in the other grid are marked
-    /// as active and set to their original value in this grid.
+    /// @note This operation modifies only active states, not values.
+    /// Specifically, active tiles and voxels in this grid are not changed, and
+    /// tiles or voxels that were inactive in this grid but active in the other grid
+    /// are marked as active in this grid but left with their original values.
     template<typename OtherTreeType>
     void topologyUnion(const Grid<OtherTreeType>& other) { tree().topologyUnion(other.tree()); }
     /// @todo topologyIntersection
     /// @todo topologyDifference
-    
+
     //
     // Statistics
     //
@@ -1042,6 +1056,14 @@ Grid<TreeT>::fill(const CoordBBox& bbox, const ValueType& value, bool active)
 }
 
 
+template<typename TreeT>
+inline void
+Grid<TreeT>::signedFloodFill(const ValueType& outside, const ValueType& inside)
+{
+    tree().signedFloodFill(outside, inside);
+}
+
+
 template<typename TreeT>
 inline void
 Grid<TreeT>::pruneGrid(float tolerance)
@@ -1189,6 +1211,6 @@ createLevelSet(double voxelSize, double halfWidth)
 
 #endif // OPENVDB_GRID_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
