@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -33,6 +33,8 @@
 #ifndef OPENVDB_PLATFORM_HAS_BEEN_INCLUDED
 #define OPENVDB_PLATFORM_HAS_BEEN_INCLUDED
 
+#include "PlatformConfig.h"
+
 /// Use OPENVDB_DEPRECATED to mark functions as deprecated.
 /// It should be placed right before the signature of the function,
 /// e.g., "OPENVDB_DEPRECATED void functionName();".
@@ -105,7 +107,7 @@ using boost::uint64_t;
     #define OPENVDB_IMPORT __attribute__((visibility("default")))
 #endif
 #ifdef _WIN32
-    #ifdef _DLL
+    #ifdef OPENVDB_DLL
         #define OPENVDB_EXPORT __declspec(dllexport)
         #define OPENVDB_IMPORT __declspec(dllimport)
     #else
@@ -134,13 +136,8 @@ using boost::uint64_t;
     #define OPENVDB_HOUDINI_API OPENVDB_IMPORT
 #endif
 
-// Windows build currently links to the shared Half.dll
-#ifdef _WIN32
-    #define OPENEXR_DLL 1
-#endif
-
 #endif // OPENVDB_PLATFORM_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
