@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -160,54 +160,6 @@ enum VecType {
 enum { NUM_VEC_TYPES = VEC_CONTRAVARIANT_ABSOLUTE + 1 };
 
 
-////////////////////////////////////////
-
-//Deprecated
-/*
-class ValueProbe
-{
-public:
-    void setState(bool)   {;}//noop
-    bool getState() const {return false;}//dummy
-    void setLevel(int)    {;}//noop
-    int  getLevel() const {return 0;}//dummy
-};
-
-class StateProbe
-{
-private:
-    bool mState;
-public:
-    void setState(bool state) {mState = state;}
-    bool getState() const {return mState;}
-    void setLevel(int)    {;}//noop
-    int  getLevel() const {return 0;}//dummy
-};
-
-class LevelProbe
-{
-private:
-    int mLevel;
-public:
-    void setLevel(int level) {mLevel=level;}
-    int  getLevel() const {return mLevel;}
-    void setState(bool) {;}//noop
-    bool getState() const {return false;}//dummy
-};
-
-class StateLevelProbe
-{
-private:
-    bool mState;
-    int  mLevel;
-public:
-    void setLevel(int level) {mLevel=level;}
-    int  getLevel() const {return mLevel;}
-    void setState(bool state) {mState = state;}
-    bool getState() const {return mState;}
-};
-*/
-
 ////////////////////////////////////////
 
 
@@ -228,7 +180,7 @@ template<> inline const char* typeNameAsString<Vec3d>()       { return "vec3d";
 template<> inline const char* typeNameAsString<std::string>() { return "string"; }
 template<> inline const char* typeNameAsString<Mat4s>()       { return "mat4s"; }
 template<> inline const char* typeNameAsString<Mat4d>()       { return "mat4d"; }
-    
+
 
 ////////////////////////////////////////
 
@@ -417,6 +369,6 @@ class TopologyCopy {};
 
 #endif // OPENVDB_TYPES_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
