@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -28,6 +28,7 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
+
 #include "Viewer.h"
 #include <iostream>
 #include <string>
@@ -44,6 +45,8 @@ usage(const char* progName, int status)
 "Options:\n" <<
 "    -i            print grid info\n" <<
 "Controls:\n" <<
+"    -> (Right)    show next grid\n" <<
+"    <- (Left)     show previous grid\n" <<
 "    1             show tree topology\n" <<
 "    2             show surface\n" <<
 "    3             show both topology and surface\n" <<
@@ -70,7 +73,7 @@ main(int argc, char * const argv[])
 
     try {
         openvdb::initialize();
-        Viewer::init();
+        Viewer::init(progName);
 
         bool printInfo = false;
 
@@ -131,6 +134,6 @@ main(int argc, char * const argv[])
     return status;
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
