@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -108,6 +108,18 @@ template<> struct RealToHalf<Vec3s>  { enum { isReal = true }; typedef Vec3H Hal
 template<> struct RealToHalf<Vec3d>  { enum { isReal = true }; typedef Vec3H HalfT; };
 
 
+/// Return the given value truncated to 16-bit float precision.
+template<typename T>
+inline T
+truncateRealToHalf(const T& val)
+{
+    return T(typename RealToHalf<T>::HalfT(val));
+}
+
+
+////////////////////////////////////////
+
+
 OPENVDB_API void zipToStream(std::ostream&, const char* data, size_t numBytes);
 OPENVDB_API void unzipFromStream(std::istream&, char* data, size_t numBytes);
 
@@ -335,7 +347,7 @@ readCompressedValues(std::istream& is, ValueT* destBuf, Index destCount,
         // Restore inactive values, using the background value and, if available,
         // the inside/outside mask.  (For fog volumes, the destination buffer is assumed
         // to be initialized to background value zero, so inactive values can be ignored.)
-        for (Index destIdx = 0, tempIdx = 0; destIdx < MaskT::BIT_SIZE; ++destIdx) {
+        for (Index destIdx = 0, tempIdx = 0; destIdx < MaskT::SIZE; ++destIdx) {
             if (valueMask.isOn(destIdx)) {
                 // Copy a saved active value into this node's buffer.
                 destBuf[destIdx] = tempBuf[tempIdx];
@@ -474,11 +486,22 @@ writeCompressedValues(std::ostream& os, ValueT* srcBuf, Index srcCount,
             metadata != NO_MASK_AND_MINUS_BG &&
             metadata != MASK_AND_NO_INACTIVE_VALS)
         {
-            // Write one of at most two distinct inactive values.
-            os.write(reinterpret_cast<const char*>(&inactiveVal[0]), sizeof(ValueT));
-            if (metadata == MASK_AND_TWO_INACTIVE_VALS) {
-                // Write the second of two distinct inactive values.
-                os.write(reinterpret_cast<const char*>(&inactiveVal[1]), sizeof(ValueT));
+            if (!toHalf) {
+                // Write one of at most two distinct inactive values.
+                os.write(reinterpret_cast<const char*>(&inactiveVal[0]), sizeof(ValueT));
+                if (metadata == MASK_AND_TWO_INACTIVE_VALS) {
+                    // Write the second of two distinct inactive values.
+                    os.write(reinterpret_cast<const char*>(&inactiveVal[1]), sizeof(ValueT));
+                }
+            } else {
+                // Write one of at most two distinct inactive values.
+                ValueT truncatedVal = truncateRealToHalf(inactiveVal[0]);
+                os.write(reinterpret_cast<const char*>(&truncatedVal), sizeof(ValueT));
+                if (metadata == MASK_AND_TWO_INACTIVE_VALS) {
+                    // Write the second of two distinct inactive values.
+                    truncatedVal = truncateRealToHalf(inactiveVal[1]);
+                    os.write(reinterpret_cast<const char*>(&truncatedVal), sizeof(ValueT));
+                }
             }
         }
 
@@ -538,6 +561,6 @@ writeCompressedValues(std::ostream& os, ValueT* srcBuf, Index srcCount,
 
 #endif // OPENVDB_IO_COMPRESSION_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
