@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -88,6 +88,15 @@ public:
                      static_cast<int>(Floor(xyz[2])));
     }
 
+    /// @brief Return the largest integer coordinates that are not greater
+    /// than @a xyz+1 (node centered conversion).
+    template<typename T> static Coord ceil(const Vec3<T>& xyz)
+    {
+        return Coord(static_cast<int>(Ceil(xyz[0])),
+                     static_cast<int>(Ceil(xyz[1])),
+                     static_cast<int>(Ceil(xyz[2])));
+    }
+
     Coord& reset(Int32 x, Int32 y, Int32 z)
     {
         mVec[0] = x; mVec[1] = y; mVec[2] = z;
@@ -162,13 +171,6 @@ public:
     }
     void asXYZ(Int32& x, Int32& y, Int32& z) const { x = mVec[0]; y = mVec[1]; z = mVec[2]; }
 
-    OPENVDB_DEPRECATED void asXYZ(Index32& x, Index32& y, Index32& z) const
-    {
-        x = static_cast<Index32>(mVec[0]);
-        y = static_cast<Index32>(mVec[1]);
-        z = static_cast<Index32>(mVec[2]);
-    }
-
     bool operator==(const Coord& rhs) const
     {
         return (mVec[0] == rhs.mVec[0] && mVec[1] == rhs.mVec[1] && mVec[2] == rhs.mVec[2]);
@@ -237,25 +239,6 @@ public:
         os.write(reinterpret_cast<const char*>(mVec), sizeof(mVec));
     }
 
-    /*
-    /// Return byte size needed for serialization
-    static size_t serialSize() { return 3*sizeof(Int32); }
-
-    inline void serialSave(char*& buffer) const {
-        Index32*& tmp = reinterpret_cast<Int32*&>(buffer);
-        *tmp++ = mVec[0];
-        *tmp++ = mVec[1];
-        *tmp++ = mVec[2];
-    }
-
-    inline void serialLoad(char*& buffer) {
-        Index32*& tmp = reinterpret_cast<Int32*&>(buffer);
-        mVec[0] = *tmp++;
-        mVec[1] = *tmp++;
-        mVec[2] = *tmp++;
-    }
-    */
-
 private:
     //no-op for now
     void dirty() { /*mHash.clear();*/ }
@@ -281,10 +264,6 @@ public:
     CoordBBox(): mMin(Coord::max()), mMax(Coord::min()) {}
     // Construct bbox from min and max
     CoordBBox(const Coord& min, const Coord& max): mMin(min), mMax(max) {}
-    // Construct cube from min and dim
-    /// @deprecated Use createCube() instead.
-    OPENVDB_DEPRECATED CoordBBox(const Coord& min, ValueType dim):
-        mMin(min), mMax(min.offsetBy(dim-1)) {}
 
     static CoordBBox createCube(const Coord& min, ValueType dim)
     {
@@ -306,7 +285,6 @@ public:
 
     void reset(const Coord& min, const Coord& max) { mMin = min; mMax = max; }
     void resetToCube(const Coord& min, ValueType dim) { mMin = min; mMax = min.offsetBy(dim - 1); }
-    OPENVDB_DEPRECATED void reset(const Coord& min, ValueType dim) { this->resetToCube(min, dim); }
 
     /// @note The start coordinate is inclusive.
     Coord getStart() const { return mMin; }
@@ -470,6 +448,6 @@ operator<<(std::ostream& os, const CoordBBox& b)
 
 #endif // OPENVDB_MATH_COORD_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
