@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -188,27 +188,20 @@ public:
     virtual double determinant() const = 0;
     virtual double determinant(const Vec3d&) const = 0;
 
-    
+
     //@{
     /// @brief Method to return the local size of a voxel.
     virtual Vec3d voxelSize() const = 0;
     virtual Vec3d voxelSize(const Vec3d&) const = 0;
     //@}
 
-    //@{
-    /// @brief Deprecated API to return local size of a voxel,
-    /// use voxelSize() instead.
-    virtual Vec3d voxelDimensions() const = 0;
-    virtual Vec3d voxelDimensions(const Vec3d&) const = 0;
-    //@}
-    
     virtual void read(std::istream&) = 0;
     virtual void write(std::ostream&) const = 0;
 
     virtual std::string str() const = 0;
 
     virtual MapBase::Ptr copy() const = 0;
-    
+
     //@{
     /// @brief Methods to update the map
     virtual MapBase::Ptr preRotate(double radians, Axis axis = X_AXIS) const = 0;
@@ -222,16 +215,6 @@ public:
     virtual MapBase::Ptr postShear(double shear, Axis axis0, Axis axis1) const = 0;
     //@}
 
-   
-    /// @brief Deprecated API for updating the map; use preRotate() instead.
-    virtual MapBase::Ptr rotate(double radians, Axis axis = X_AXIS) const = 0;
-    /// @brief Deprecated API for updating the map; use postTranslate() instead.
-    virtual MapBase::Ptr translate(const Vec3d&) const = 0;
-    /// @brief Deprecated API for updating the map; use preScale() instead.
-    virtual MapBase::Ptr scale(const Vec3d&) const = 0;
-    /// @brief Deprecated API for updating the map; use preShear() instead.
-    virtual MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const = 0;
-
 protected:
     MapBase() {}
 
@@ -410,11 +393,6 @@ public:
     /// Return the determinant of the Jacobian
     double determinant() const { return mDeterminant; }
 
-    /// @deprecated Use voxelSize() instead.
-OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return voxelSize(); }
-    /// @deprecated Use voxelSize() instead.
-OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(); }
- 
     //@{
     /// @brief Return the lengths of the images of the segments
     /// (0,0,0)-(1,0,0), (0,0,0)-(0,1,0) and (0,0,0)-(0,0,1).
@@ -435,7 +413,7 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     // Methods that modify the existing affine map
 
     //@{
-    /// @brief Methods that modify the existing 
+    /// @brief Methods that modify the existing
     /// affine map by pre-applying the given operation.
     void accumPreRotation(Axis axis, double radians)
     {
@@ -461,7 +439,7 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
 
 
     //@{
-    /// @brief Methods that modify the existing 
+    /// @brief Methods that modify the existing
     /// affine map by post-applying the given operation.
     void accumPostRotation(Axis axis, double radians)
     {
@@ -522,8 +500,8 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
 
 
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of prepending the appropraite operation. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of prepending the appropraite operation.
     MapBase::Ptr preRotate(double radians, Axis axis = X_AXIS) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -548,12 +526,12 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
         affineMap->accumPreShear(axis0, axis1, shear);
         return simplify(affineMap);
     }
-    //@} 
-    
+    //@}
+
 
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of postfixing the appropraite operation. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of postfixing the appropraite operation.
     MapBase::Ptr postRotate(double radians, Axis axis = X_AXIS) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -580,37 +558,11 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     }
     //@}
 
-
-    /// @deprecated Use accumPreRotation() instead.
-OPENVDB_DEPRECATED void accumRotation(Axis axis, double radians)
-    {   accumPreRotation(axis, radians); }
-    /// @deprecated Use accumPreScale() instead.
-OPENVDB_DEPRECATED void accumScale(const Vec3d& v)
-    {   accumPreScale(v); }
-    /// @deprecated Use accumPostTranaslation() instead
-OPENVDB_DEPRECATED void accumTranslation(const Vec3d& v)
-    {   accumPostTranslation(v); }
-    /// @deprecated Use accumPreShear() instead
-OPENVDB_DEPRECATED void accumShear(Axis axis0, Axis axis1, double shear)
-    {   accumPreShear(axis0, axis1, shear); }
-    
-    /// @deprecated Use preRotate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr rotate(double radians, Axis axis = X_AXIS) const
-   {    return preRotate(radians, axis); }
-    /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& t) const
-    {   return postTranslate(t); }
-    /// @deprecated Use preScale() instead.
-OPENVDB_DEPRECATED MapBase::Ptr scale(const Vec3d& s) const
-    {   return preScale(s); }
-    /// @deprecated Use preShear() instead.
-OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const
-    {   return preShear(shear, axis0, axis1); }
-    
     /// Return the matrix representation of this AffineMap
     Mat4d getMat4() const { return mMatrix;}
     const Mat4d& getConstMat4() const {return mMatrix;}
     const Mat3d& getConstJacobianInv() const {return mJacobianInv;}
+
 private:
     void updateAcceleration() {
         mDeterminant = mMatrix.getMat3().det();
@@ -757,12 +709,6 @@ public:
     /// Return 1/(scale)
     const Vec3d& getInvScale() const {return mScaleValuesInverse; }
 
-
-    /// @deprecated Use voxelSize() instead.
-    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return mVoxelSize; }
-    /// @deprecated Use voxelSize(const Vec3d&) instead.
-    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize();}
- 
     //@{
     /// @brief Returns the lengths of the images
     /// of the segments
@@ -815,11 +761,11 @@ public:
     {
         return AffineMap::Ptr(new AffineMap(math::scale<Mat4d>(mScaleValues)));
     }
-    
-    
-    
+
+
+
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
     /// of prepending the appropraite operation to the existing map
     MapBase::Ptr preRotate(double radians, Axis axis) const
     {
@@ -827,11 +773,11 @@ public:
         affineMap->accumPreRotation(axis, radians);
         return simplify(affineMap);
     }
-    
+
     MapBase::Ptr preTranslate(const Vec3d& tr) const;
-    
+
     MapBase::Ptr preScale(const Vec3d& v) const;
-    
+
     MapBase::Ptr preShear(double shear, Axis axis0, Axis axis1) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -840,21 +786,21 @@ public:
     }
     //@}
 
-    
+
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of prepending the appropraite operation to the existing map. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of prepending the appropraite operation to the existing map.
     MapBase::Ptr postRotate(double radians, Axis axis) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
         affineMap->accumPostRotation(axis, radians);
         return simplify(affineMap);
     }
-    
+
     MapBase::Ptr postTranslate(const Vec3d& tr) const;
-    
+
     MapBase::Ptr postScale(const Vec3d& v) const;
-    
+
     MapBase::Ptr postShear(double shear, Axis axis0, Axis axis1) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -863,17 +809,6 @@ public:
     }
     //@}
 
-    /// @deprecated Use preRotate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr rotate(double radians, Axis axis) const
-    {  return preRotate(radians, axis); }
-    /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& tr) const;
-    /// @deprecated Use preScale() instead.
-OPENVDB_DEPRECATED MapBase::Ptr scale(const Vec3d& v) const;
-    /// @deprecated Use preShear() instead.
-OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const
-    {   return preShear(shear, axis0, axis1); }
-    
 private:
     Vec3d mScaleValues, mVoxelSize, mScaleValuesInverse, mInvScaleSqr, mInvTwiceScale;
 }; // class ScaleMap
@@ -921,11 +856,9 @@ public:
     /// post-translation on this map
     MapBase::Ptr postTranslate(const Vec3d& tr) const;
 
-    /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& tr) const;
-
 }; // class UniformScaleMap
 
+
 ////////////////////////////////////////
 
 
@@ -948,10 +881,6 @@ ScaleMap::postScale(const Vec3d& v) const
 }
 
 
-OPENVDB_DEPRECATED  inline MapBase::Ptr
-ScaleMap::scale(const Vec3d& v) const
-{   return preScale(v); }
-
 /// @brief A specialized linear transform that performs a translation
 class OPENVDB_API TranslationMap: public MapBase
 {
@@ -960,15 +889,9 @@ public:
     typedef boost::shared_ptr<const TranslationMap> ConstPtr;
 
     // default constructor is a translation by zero.
-    TranslationMap(): MapBase(),mTranslation(Vec3d(0,0,0)) {}
-
-    TranslationMap(const Vec3d& t): MapBase(),mTranslation(t)
-    {
-    }
-
-    TranslationMap(const TranslationMap& other): MapBase(),mTranslation(other.mTranslation)
-    {
-    }
+    TranslationMap(): MapBase(), mTranslation(Vec3d(0,0,0)) {}
+    TranslationMap(const Vec3d& t): MapBase(), mTranslation(t) {}
+    TranslationMap(const TranslationMap& other): MapBase(), mTranslation(other.mTranslation) {}
 
     ~TranslationMap() {}
 
@@ -1019,12 +942,8 @@ public:
 
     /// Return \f$ (1,1,1) \f$
     Vec3d voxelSize() const { return Vec3d(1,1,1);}
-    /// @deprecated Use voxelSize() instead.
-OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return Vec3d(1,1,1);}
     /// Return \f$ (1,1,1) \f$
     Vec3d voxelSize(const Vec3d&) const { return voxelSize();}
-    /// @deprecated Use voxelSize() instead.
-OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize();}
 
     /// Return the translation vector
     const Vec3d& getTranslation() const { return mTranslation; }
@@ -1062,10 +981,9 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
         return affineMap;
     }
 
-    
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of prepending the appropraite operation. 
+    /// @brief Return a MapBase::Ptr to a new map that is the result
+    /// of prepending the appropriate operation.
     MapBase::Ptr preRotate(double radians, Axis axis) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -1089,8 +1007,8 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     //@}
 
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of postfixing the appropraite operation. 
+    /// @brief Return a MapBase::Ptr to a new map that is the result
+    /// of postfixing the appropriate operation.
     MapBase::Ptr postRotate(double radians, Axis axis) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -1113,20 +1031,6 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     }
     //@}
 
-    /// ---- Deprecated --- ////
-
-    /// @deprecated use preRotate() instead
-OPENVDB_DEPRECATED MapBase::Ptr rotate(double radians, Axis axis) const
-    {   return preRotate(radians, axis); }
-    /// @deprecated use postTranslate() instead
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& t) const
-    {   return preTranslate(t); }
-    /// @deprecated use preScale() instead
-OPENVDB_DEPRECATED MapBase::Ptr scale(const Vec3d& v) const;
-    /// @deprecated use preShear() instead
-OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const
-    {   return preShear(shear, axis0, axis1); }
-
 private:
     Vec3d mTranslation;
 }; // class TranslationMap
@@ -1207,7 +1111,7 @@ public:
             ScaleTranslateMap::mapType(),
             ScaleTranslateMap::create);
     }
-    
+
     Name type() const { return mapType(); }
     static Name mapType() { return Name("ScaleTranslateMap"); }
 
@@ -1271,13 +1175,9 @@ public:
     double determinant() const { return mScaleValues.x()*mScaleValues.y()*mScaleValues.z(); }
     /// Return the absolute values of the scale values
     Vec3d voxelSize() const { return mVoxelSize;}
-    /// @deprecated use voxelSize() instead
-OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return voxelSize();}
     /// Return the absolute values of the scale values, ignores
     ///argument
     Vec3d voxelSize(const Vec3d&) const { return voxelSize();}
-    /// @deprecated use voxelSize() instead
-OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize();}
 
     /// Returns the scale values
     const Vec3d& getScale() const { return mScaleValues; }
@@ -1340,10 +1240,10 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
         affineMap->accumPostTranslation(mTranslation);
         return affineMap;
     }
-    
+
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of prepending the appropraite operation. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of prepending the appropraite operation.
     MapBase::Ptr preRotate(double radians, Axis axis) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -1353,12 +1253,12 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     MapBase::Ptr preTranslate(const Vec3d& t) const
     {
         const Vec3d& s = mScaleValues;
-        const Vec3d scaled_trans( t.x() * s.x(), 
-                                  t.y() * s.y(), 
+        const Vec3d scaled_trans( t.x() * s.x(),
+                                  t.y() * s.y(),
                                   t.z() * s.z() );
         return MapBase::Ptr( new ScaleTranslateMap(mScaleValues, mTranslation + scaled_trans));
     }
-    
+
     MapBase::Ptr preScale(const Vec3d& v) const;
 
     MapBase::Ptr preShear(double shear, Axis axis0, Axis axis1) const
@@ -1368,10 +1268,10 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
         return simplify(affineMap);
     }
     //@}
-        
+
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of postfixing the appropraite operation. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of postfixing the appropraite operation.
     MapBase::Ptr postRotate(double radians, Axis axis) const
     {
         AffineMap::Ptr affineMap = getAffineMap();
@@ -1382,7 +1282,7 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     {
         return MapBase::Ptr( new ScaleTranslateMap(mScaleValues, mTranslation + t));
     }
-    
+
     MapBase::Ptr postScale(const Vec3d& v) const;
 
     MapBase::Ptr postShear(double shear, Axis axis0, Axis axis1) const
@@ -1393,18 +1293,6 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     }
     //@}
 
-    /// @deprecated Use preRotate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr rotate(double radians, Axis axis) const
-    {   return preRotate(radians, axis);  }
-    /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& t) const
-    {   return postTranslate(t); }
-    /// @deprecated Use preScale() instead.
-OPENVDB_DEPRECATED MapBase::Ptr scale(const Vec3d& v) const;
-    /// @deprecated Use preShear() instead.
-OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const
-    {   return preShear(shear, axis0, axis1); }
-
 private:
     Vec3d mTranslation, mScaleValues, mVoxelSize, mScaleValuesInverse,
         mInvScaleSqr, mInvTwiceScale;
@@ -1421,18 +1309,14 @@ ScaleMap::postTranslate(const Vec3d& t) const
 inline MapBase::Ptr
 ScaleMap::preTranslate(const Vec3d& t) const
 {
-    
+
     const Vec3d& s = mScaleValues;
-    const Vec3d scaled_trans( t.x() * s.x(), 
-                              t.y() * s.y(), 
+    const Vec3d scaled_trans( t.x() * s.x(),
+                              t.y() * s.y(),
                               t.z() * s.z() );
     return MapBase::Ptr(new ScaleTranslateMap(mScaleValues, scaled_trans));
 }
 
-OPENVDB_DEPRECATED inline MapBase::Ptr
-ScaleMap::translate(const Vec3d& t) const
-{  return postTranslate(t); }
-
 
 /// @brief A specialized Affine transform that uniformaly scales along the principal axis
 /// and then translates the result.
@@ -1441,7 +1325,7 @@ class OPENVDB_API UniformScaleTranslateMap: public ScaleTranslateMap
 public:
     typedef boost::shared_ptr<UniformScaleTranslateMap>       Ptr;
     typedef boost::shared_ptr<const UniformScaleTranslateMap> ConstPtr;
-    
+
     UniformScaleTranslateMap():ScaleTranslateMap(Vec3d(1,1,1), Vec3d(0,0,0)) {}
     UniformScaleTranslateMap(double scale, const Vec3d& translate):
         ScaleTranslateMap(Vec3d(scale,scale,scale), translate) {}
@@ -1479,7 +1363,6 @@ public:
     }
     bool operator!=(const UniformScaleTranslateMap& other) const { return !(*this == other); }
 
-    
     /// @brief Return a MapBase::Ptr to a UniformScaleTranslateMap that is
     /// the result of prepending translation on this map.
     MapBase::Ptr preTranslate(const Vec3d& t) const
@@ -1496,15 +1379,9 @@ public:
         const double scale = this->getScale().x();
         return MapBase::Ptr( new UniformScaleTranslateMap(scale, this->getTranslation() + t));
     }
-    /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& t) const
-    {   return postTranslate(t); }
-
 }; // class UniformScaleTanslateMap
 
 
-
-
 inline MapBase::Ptr
 UniformScaleMap::postTranslate(const Vec3d& t) const
 {
@@ -1536,8 +1413,8 @@ TranslationMap::postScale(const Vec3d& v) const
     if (isApproxEqual(v[0],v[1]) && isApproxEqual(v[0],v[2])) {
         return MapBase::Ptr(new UniformScaleTranslateMap(v[0], v[0]*mTranslation));
     } else {
-        const Vec3d trans(mTranslation.x()*v.x(), 
-                          mTranslation.y()*v.y(), 
+        const Vec3d trans(mTranslation.x()*v.x(),
+                          mTranslation.y()*v.y(),
                           mTranslation.z()*v.z());
         return MapBase::Ptr(new ScaleTranslateMap(v, trans));
     }
@@ -1558,8 +1435,8 @@ inline MapBase::Ptr
 ScaleTranslateMap::postScale(const Vec3d& v) const
 {
     const Vec3d new_scale( v * mScaleValues );
-    const Vec3d new_trans( mTranslation.x()*v.x(), 
-                           mTranslation.y()*v.y(), 
+    const Vec3d new_trans( mTranslation.x()*v.x(),
+                           mTranslation.y()*v.y(),
                            mTranslation.z()*v.z() );
 
     if (isApproxEqual(new_scale[0],new_scale[1]) && isApproxEqual(new_scale[0],new_scale[2])) {
@@ -1570,21 +1447,9 @@ ScaleTranslateMap::postScale(const Vec3d& v) const
 }
 
 
-OPENVDB_DEPRECATED inline MapBase::Ptr
-UniformScaleMap::translate(const Vec3d& t) const
-{     return postTranslate(t); }
-
-OPENVDB_DEPRECATED inline MapBase::Ptr
-TranslationMap::scale(const Vec3d& v) const
-{    return preScale(v);  }
-
-OPENVDB_DEPRECATED inline MapBase::Ptr
-ScaleTranslateMap::scale(const Vec3d& v) const
-{   return preScale(v); }
-
-
 ////////////////////////////////////////
 
+
 /// @brief A specialized linear transform that performs a unitary maping
 /// i.e. rotation  and or reflection.
 class OPENVDB_API UnitaryMap: public MapBase
@@ -1724,10 +1589,6 @@ public:
     /// \f$(0,0,0)-(0,0,1)\f$
     Vec3d voxelSize() const { return mAffineMap.voxelSize();}
     Vec3d voxelSize(const Vec3d&) const { return voxelSize();}
-    /// @deprecated Use voxelSize() instead.
-OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return voxelSize();}
-    /// @deprecated Use voxelSize() instead.
-OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize();}
 
     /// read serialization
     void read(std::istream& is)
@@ -1750,22 +1611,9 @@ OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&) const { return voxelSize(
     /// Return AffineMap::Ptr to an AffineMap equivalent to *this
     AffineMap::Ptr getAffineMap() const { return AffineMap::Ptr(new AffineMap(mAffineMap)); }
 
-    /// @deprecated Use preRotate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr rotate(double radians, Axis axis) const
-     {   return postRotate(radians, axis); }
-    /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& t) const
-    {   return postTranslate(t); }
-    /// @deprecated Use preScale() instead.
-OPENVDB_DEPRECATED MapBase::Ptr scale(const Vec3d& v) const
-    {   return preScale(v);  }
-    /// @deprecated Use preShear() instead.
-OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const
-    {   return preShear(shear, axis0, axis1); }
-
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of prepending the appropraite operation. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of prepending the appropraite operation.
     MapBase::Ptr preRotate(double radians, Axis axis) const
     {
         UnitaryMap first(axis, radians);
@@ -1792,10 +1640,10 @@ OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) cons
     }
     //@}
 
-  
+
     //@{
-    /// @brief  Return a MapBase::Ptr to a new map that is the result 
-    /// of postfixing the appropraite operation. 
+    /// @brief  Return a MapBase::Ptr to a new map that is the result
+    /// of postfixing the appropraite operation.
     MapBase::Ptr postRotate(double radians, Axis axis) const
     {
         UnitaryMap second(axis, radians);
@@ -1854,36 +1702,17 @@ public:
     /// @brief Constructor that takes an index-space bounding box
     /// to be mapped into a frustum with a given @a depth and @a taper
     /// (defined as ratio of nearplane/farplane).
-OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double depth):
-       MapBase(),mBBox(bb.min().asVec3d(), bb.max().asVec3d()), mTaper(taper), mDepth(depth)
-    {
-        init();
-    }
-
     NonlinearFrustumMap(const BBoxd& bb, double taper, double depth):
         MapBase(),mBBox(bb), mTaper(taper), mDepth(depth)
     {
         init();
     }
 
-
     /// @brief Constructor that takes an index-space bounding box
     /// to be mapped into a frustum with a given @a depth and @a taper
     /// (defined as ratio of nearplane/farplane).
     /// @details This frustum is further modifed by the @a secondMap,
     /// intended to be a simple translation and rotation and uniform scale
-OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double depth,
-        const MapBase::Ptr& secondMap):
-        mBBox(bb.min().asVec3d(),bb.max().asVec3d()), mTaper(taper), mDepth(depth)
-    {
-        if (!secondMap->isLinear() ) {
-              OPENVDB_THROW(ArithmeticError,
-                "The second map in the Frustum transfrom must be linear");
-        }
-        mSecondMap = *( secondMap->getAffineMap() );
-        init();
-    }
-
    NonlinearFrustumMap(const BBoxd& bb, double taper, double depth,
         const MapBase::Ptr& secondMap):
         mBBox(bb), mTaper(taper), mDepth(depth)
@@ -1907,7 +1736,6 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
         init();
     }
 
-
     /// @brief Constructor from a camera frustum
     ///
     /// @param position the tip of the frustum (i.e., the camera's position).
@@ -1975,7 +1803,7 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
         // T_inv * R1_inv * R2_inv * scale_inv
         Mat4d mat = scale * r2 * r1;
         mat.setTranslation(position + z_near*direction_unit);
-        
+
         mSecondMap = AffineMap(mat);
 
         init();
@@ -2005,6 +1833,26 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
 
     /// Return @c false (by convention false)
     bool hasUniformScale() const { return false; }
+    
+    /// Return @c true if the map is equivalent to an identity 
+    bool isIdentity() const {
+        
+        // The frustum can only be consistent with a linear map if the taper value is 1
+        if (!isApproxEqual(mTaper, double(1)) ) return false;
+        
+        // There are various ways an identity can decomposed between the two parts of the
+        // map.  Best to just check that the principle vectors are stationary.
+        const Vec3d e1(1,0,0);
+        if (!applyMap(e1).eq(e1)) return false;
+     
+        const Vec3d e2(0,1,0);
+        if (!applyMap(e2).eq(e2)) return false;
+       
+        const Vec3d e3(0,0,1);
+        if (!applyMap(e3).eq(e3)) return false;
+    
+        return true;
+    }
 
     virtual bool isEqual(const MapBase& other) const { return isEqualBase(*this, other); }
 
@@ -2013,7 +1861,7 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
         if (mBBox!=other.mBBox) return false;
         if (!isApproxEqual(mTaper, other.mTaper)) return false;
         if (!isApproxEqual(mDepth, other.mDepth)) return false;
-       
+
         // Two linear transforms are equivalent iff they have the same translation
         // and have the same affects on orthongal spanning basis check translation
         Vec3d e(0,0,0);
@@ -2057,23 +1905,22 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
     /// @c ijk is the location in the pre-image space (e.g. index space)
     Vec3d applyIJT(const Vec3d& d1_is, const Vec3d& ijk) const
     {
-        
         const Vec3d loc = applyFrustumMap(ijk);
         const double s = mGamma * loc.z() + 1.;
 
         // verify that we aren't at the singularity
         if (isApproxEqual(s, 0.)) {
-            OPENVDB_THROW(ArithmeticError,
-                          "Tried to evaluate the frustum transform at the singular focal point (e.g. camera)");
+            OPENVDB_THROW(ArithmeticError, "Tried to evaluate the frustum transform"
+                " at the singular focal point (e.g. camera)");
         }
-        
+
         const double sinv = 1.0/s;        // 1/(z*gamma + 1)
         const double pt0 = mLx * sinv;    // Lx / (z*gamma +1)
-        const double pt1 = mGamma * pt0;  // gamma * Lx / ( z*gamma +1) 
+        const double pt1 = mGamma * pt0;  // gamma * Lx / ( z*gamma +1)
         const double pt2 = pt1 * sinv;    // gamma * Lx / ( z*gamma +1)**2
-        
+
         const Mat3d& jacinv = mSecondMap.getConstJacobianInv();
-        
+
         // compute \frac{\partial E_i}{\partial x_j}
         Mat3d gradE(Mat3d::zero());
         for (int j = 0; j < 3; ++j ) {
@@ -2081,12 +1928,12 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
             gradE(1,j) =  pt0 * jacinv(1,j) -  pt2 * loc.y()*jacinv(2,j);
             gradE(2,j) = (1./mDepthOnLz) * jacinv(2,j);
         }
-        
+
         Vec3d result;
         for (int i = 0; i < 3; ++i) {
             result(0) = d1_is(0) * gradE(0,i) + d1_is(1) * gradE(1,i) + d1_is(2) * gradE(2,i);
         }
-        
+
         return result;
 
     }
@@ -2100,37 +1947,37 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
     Mat3d applyIJC(const Mat3d& d2_is, const Vec3d& d1_is, const Vec3d& ijk) const
     {
         const Vec3d loc = applyFrustumMap(ijk);
-        
+
         const double s =  mGamma * loc.z()  + 1.;
 
         // verify that we aren't at the singularity
         if (isApproxEqual(s, 0.)) {
-            OPENVDB_THROW(ArithmeticError,
-                          "Tried to evaluate the frustum transform at the singular focal point (e.g. camera)");
+            OPENVDB_THROW(ArithmeticError, "Tried to evaluate the frustum transform"
+                " at the singular focal point (e.g. camera)");
         }
-        
-        // precompute 
+
+        // precompute
         const double sinv = 1.0/s;     // 1/(z*gamma + 1)
         const double pt0 = mLx * sinv;   // Lx / (z*gamma +1)
-        const double pt1 = mGamma * pt0;   // gamma * Lx / ( z*gamma +1) 
+        const double pt1 = mGamma * pt0;   // gamma * Lx / ( z*gamma +1)
         const double pt2 = pt1 * sinv;   // gamma * Lx / ( z*gamma +1)**2
         const double pt3 = pt2 * sinv;   // gamma * Lx / ( z*gamma +1)**3
-        
+
 
         const Mat3d& jacinv = mSecondMap.getConstJacobianInv();
-      
+
         // compute \frac{\partial^2 E_i}{\partial x_j \partial x_k}
-      
+
         Mat3d matE0(Mat3d::zero());
         Mat3d matE1(Mat3d::zero()); // matE2 = 0
         for(int j = 0; j < 3; j++) {
             for (int k = 0; k < 3; k++) {
-                
+
                 const double pt4 =  2. * jacinv(2,j) * jacinv(2,k) * pt3;
 
                 matE0(j,k) = -(jacinv(0,j) * jacinv(2,k) + jacinv(2,j) * jacinv(0,k)) * pt2 +
                     pt4 * loc.x();
-                
+
                 matE1(j,k) = -(jacinv(1,j) * jacinv(2,k) + jacinv(2,j) * jacinv(1,k)) * pt2 +
                     pt4 * loc.y();
             }
@@ -2143,7 +1990,7 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
             gradE(1,j) =  pt0 * jacinv(1,j) -  pt2 * loc.y()*jacinv(2,j);
             gradE(2,j) = (1./mDepthOnLz) * jacinv(2,j);
         }
-        
+
         Mat3d result(Mat3d::zero());
         // compute \fac{\partial E_j}{\partial x_m} \fac{\partial E_i}{\partial x_n}
         // \frac{\partial^2 input}{\partial E_i \partial E_j}
@@ -2159,7 +2006,8 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
 
          for (int m = 0; m < 3; ++m ) {
             for ( int n = 0; n < 3; ++n) {
-                result(m, n) += matE0(m, n) * d1_is(0) + matE1(m, n) * d1_is(1);// + matE2(m, n) * d1_is(2);
+                result(m, n) +=
+                    matE0(m, n) * d1_is(0) + matE1(m, n) * d1_is(1);// + matE2(m, n) * d1_is(2);
             }
         }
 
@@ -2180,14 +2028,14 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
 
     /// Return the size of a voxel at the center of the near plane
     Vec3d voxelSize() const {
-        const Vec3d loc( 0.5*(mBBox.min().x() + mBBox.max().x()), 
+        const Vec3d loc( 0.5*(mBBox.min().x() + mBBox.max().x()),
                          0.5*(mBBox.min().y() + mBBox.max().y()),
                          mBBox.min().z());
-        
+
         return voxelSize(loc);
-    
+
     }
-  
+
     /// @brief Returns the lengths of the images of the three segments
     /// from @a loc to @a loc + (1,0,0), from @a loc to @a loc + (0,1,0)
     /// and from @a loc to @a loc + (0,0,1)
@@ -2200,11 +2048,6 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
         return out;
     }
 
-    /// @deprecated Use voxelSize() instead.
-    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return voxelSize(); }
-    /// @deprecated Use voxelSiz(loc) instead.
-    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d& loc) const { return voxelSize(loc); }
-
     AffineMap::Ptr getAffineMap()  const
     {
         return mSecondMap.getAffineMap();
@@ -2221,14 +2064,10 @@ OPENVDB_DEPRECATED NonlinearFrustumMap(const CoordBBox& bb, double taper, double
     // gamma a non-dimensional  number:  nearplane x-width / camera to near plane distance
     double getGamma() const { return mGamma; }
 
-    /// Set a bounding box in pre-image space (e.g. index space) that is converted to the frustm
-    /// in image space
-OPENVDB_DEPRECATED void setBBox(const CoordBBox& bb) { mBBox = BBoxd(bb.min().asVec3d(), bb.max().asVec3d()); }
     /// Return the bounding box that defines the frustum in pre-image space
     const BBoxd& getBBox() const { return mBBox; }
 
     /// Return MapBase::Ptr& to the second map
-OPENVDB_DEPRECATED MapBase::Ptr linearMap() const { return getAffineMap(); }
     const AffineMap& secondMap() const { return mSecondMap; }
     /// Return @c true if the  the bounding box in index space that defines the region that
     /// is maped into the frustum is non-zero, otherwise @c false
@@ -2290,7 +2129,7 @@ OPENVDB_DEPRECATED MapBase::Ptr linearMap() const { return getAffineMap(); }
     }
 
     //@{
-    /// @brief Return a MapBase::Ptr to a new map that is the result 
+    /// @brief Return a MapBase::Ptr to a new map that is the result
     /// of prepending the appropriate operation to the linear part of this map
     MapBase::Ptr preRotate(double radians, Axis axis = X_AXIS) const
     {
@@ -2339,19 +2178,6 @@ OPENVDB_DEPRECATED MapBase::Ptr linearMap() const { return getAffineMap(); }
     }
     //@}
 
-    /// @deprecated Use preRotate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr rotate(double radians, Axis axis = X_AXIS) const
-    {    return preRotate(radians, axis); }
-     /// @deprecated Use postTranslate() instead.
-OPENVDB_DEPRECATED MapBase::Ptr translate(const Vec3d& t) const
-    {    return postTranslate(t); }
-    /// @deprecated Use preScale() instead.
-OPENVDB_DEPRECATED MapBase::Ptr scale(const Vec3d& s) const
-    {    return preScale(s); }
-    /// @deprecated Use preShear() instead.
-OPENVDB_DEPRECATED MapBase::Ptr shear(double shear, Axis axis0, Axis axis1) const
-    {    return preShear(shear, axis0, axis1); }
-
 
 private:
     void init() {
@@ -2359,19 +2185,19 @@ private:
         mLx = mBBox.extents().x();
         mLy = mBBox.extents().y();
         mLz = mBBox.extents().z();
-        
+
         if (isApproxEqual(mLx,0.) || isApproxEqual(mLy,0.) || isApproxEqual(mLz,0.) ) {
-            OPENVDB_THROW(ArithmeticError,
-                          "The index space bounding box must have at least two index points in each direction.");
+            OPENVDB_THROW(ArithmeticError, "The index space bounding box"
+                " must have at least two index points in each direction.");
         }
 
-        mXo = 0.5* mLx; 
-        mYo = 0.5* mLy; 
+        mXo = 0.5* mLx;
+        mYo = 0.5* mLy;
 
         // mDepth is non-dimensionalized on near
         mGamma = (1./mTaper - 1) / mDepth;
-     
-        mDepthOnLz = mDepth/mLz; 
+
+        mDepthOnLz = mDepth/mLz;
         mDepthOnLzLxLx = mDepthOnLz/(mLx * mLx);
 
         /// test for shear and non-uniform scale
@@ -2436,18 +2262,18 @@ private:
 
     // bounding box in index space used in Frustum transforms.
     BBoxd   mBBox;
-    
+
     // taper value used in constructing Frustums.
     double      mTaper;
     double      mDepth;
 
     // defines the second map
     AffineMap mSecondMap;
-    
+
     // these are derived from the above.
     double mLx, mLy, mLz;
     double mXo, mYo, mGamma, mDepthOnLz, mDepthOnLzLxLx;
-    
+
     // true: if the mSecondMap is linear and has no shear, and has no non-uniform scale
     bool mHasSimpleAffine;
 }; // class NonlinearFrustumMap
@@ -2569,13 +2395,12 @@ private:
     AffineMap      mAffineMap;
 }; // class CompoundMap
 
-
 } // namespace math
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_MATH_MAPS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
