@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -853,10 +853,10 @@ inline bool isSymmetric(const MatType& m) {
 template <typename MatType>
 inline bool isUnitary(const MatType& m) {
     typedef typename MatType::ValueType  value_type;
-    if (!isApproxEqual(fabs(m.det()), (value_type)1)) return false;
+    if (!isApproxEqual(std::abs(m.det()), value_type(1.0))) return false;
     // check that the matrix transpose is the inverse
     MatType temp = m * m.transpose();
-    return temp.eq(MatType::identity());
+    return temp.eq(MatType::identity(), toleranceValue<value_type>());
 }
 
 /// Determine if a matrix is diagonal
@@ -871,7 +871,7 @@ inline bool isDiagonal(const MatType& mat) {
             }
         }
     }
-    return isApproxEqual(temp, 0.);
+    return isApproxEqual(temp, typename MatType::ValueType(0.0));
 }
 
 /// takes a n by n matrix and returns the L_Infinty norm
@@ -970,6 +970,6 @@ bool polarDecomposition(const MatType& input, MatType& unitary, MatType& positiv
 
 #endif // OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
