@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -46,7 +46,7 @@ namespace math {
 
 template<typename T> class Vec3;
 template<typename T> class Mat4;
-template<typename T> class Quat; 
+template<typename T> class Quat;
 
 /// @class Mat3 Mat3.h
 /// @brief 3x3 matrix class.
@@ -65,7 +65,7 @@ public:
     /// The quaternion is normalized and used to construct the matrix
     Mat3(const Quat<T> &q)
     { setToRotation(q); }
-    
+
 
     /// Constructor given array of elements, the ordering is in row major form:
     /** @verbatim
@@ -201,7 +201,6 @@ public:
     T* asPointer() {return MyBase::mm;}
     const T* asPointer() const {return MyBase::mm;}
 
-
     /// Alternative indexed reference to the elements
     /// Note that the indices are row first and column second.
     /// e.g.    m(0,0) = 1;
@@ -265,19 +264,18 @@ public:
     void setSkew(const Vec3<T> &v)
     {*this = skew(v);}
 
-    /// Set "this" matrix to the rotation matrix specified by the quaternion
-    /// The quaternion is normalized and used to construct the matrix
-    /// Note that the matrix is transposed to match post-multiplication
-    /// symantics 
+    /// @brief Set this matrix to the rotation matrix specified by the quaternion
+    /// @details The quaternion is normalized and used to construct the matrix.
+    /// Note that the matrix is transposed to match post-multiplication semantics.
     void setToRotation(const Quat<T> &q)
     {*this = rotation<Mat3<T> >(q);}
-    
-    /// Set "this" matrix to the rotation specified by axis and angle
-    /// The axis must be unit vector
+
+    /// @brief Set this matrix to the rotation specified by @a axis and @a angle
+    /// @details The axis must be unit vector
     void setToRotation(const Vec3<T> &axis, T angle)
     {*this = rotation<Mat3<T> >(axis, angle);}
 
-    // Set "this" matrix to zero
+    /// Set this matrix to zero
     void setZero()
     {
         MyBase::mm[0] = 0;
@@ -414,7 +412,7 @@ public:
         MyBase::mm[2] = static_cast<T>(s0[0] * s1[2] +
                                        s0[1] * s1[5] +
                                        s0[2] * s1[8]);
-        
+
         MyBase::mm[3] = static_cast<T>(s0[3] * s1[0] +
                                        s0[4] * s1[3] +
                                        s0[5] * s1[6]);
@@ -424,7 +422,7 @@ public:
         MyBase::mm[5] = static_cast<T>(s0[3] * s1[2] +
                                        s0[4] * s1[5] +
                                        s0[5] * s1[8]);
-        
+
         MyBase::mm[6] = static_cast<T>(s0[6] * s1[0] +
                                        s0[7] * s1[3] +
                                        s0[8] * s1[6]);
@@ -434,7 +432,7 @@ public:
         MyBase::mm[8] = static_cast<T>(s0[6] * s1[2] +
                                        s0[7] * s1[5] +
                                        s0[8] * s1[8]);
-        
+
         return *this;
     }
 
@@ -490,11 +488,11 @@ public:
     } // determinantTest
 
     /// Trace of matrix
-    T trace() const 
+    T trace() const
     {
         return MyBase::mm[0]+MyBase::mm[4]+MyBase::mm[8];
     }
-    
+
     /// This function snaps a specific axis to a specific direction,
     /// preserving scaling. It does this using minimum energy, thus
     /// posing a unique solution if basis & direction arent parralel.
@@ -530,130 +528,6 @@ public:
         return snapBasis(*this, axis, direction);
     }
 
-
-    /// For certain legacy APIs, we need to treat the contents as a pointer.
-    /// @remark This was originally designed as an "operator T*()", but that
-    /// automatic pointer cast would be applied in unexpected cases, such as
-    /// std::min/max.
-#ifdef  ALLOW_CAST_TO_POINTER
-    /// Cast to T*
-OPENVDB_DEPRECATED operator T*()
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
-            __attribute__ ((deprecated))
-#endif
-            { return MyBase::mm; } // trivial
-OPENVDB_DEPRECATED operator const T*() const
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
-            __attribute__ ((deprecated))
-#endif
-            { return MyBase::mm; } // trivial
-#endif
-
-
-    /// this = m1 + m2
-    /// m1 and m2 need not be distinct objects than "this", e.g. m1.add(m1,m2);
-    template<typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat3 &add(const Mat3<T0> &m1, const Mat3<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-
-        MyBase::mm[0] = src1[0] + src2[0];
-        MyBase::mm[1] = src1[1] + src2[1];
-        MyBase::mm[2] = src1[2] + src2[2];
-        MyBase::mm[3] = src1[3] + src2[3];
-        MyBase::mm[4] = src1[4] + src2[4];
-        MyBase::mm[5] = src1[5] + src2[5];
-        MyBase::mm[6] = src1[6] + src2[6];
-        MyBase::mm[7] = src1[7] + src2[7];
-        MyBase::mm[8] = src1[8] + src2[8];
-
-        return *this;
-    } // opPlusTest
-
-    /// this = m1 - m2
-    /// m1 and m2 need not be distinct objects than "this", e.g. m1.sub(m1,m2);
-    template<typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat3 &sub(const Mat3<T0> &m1, const Mat3<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-
-        MyBase::mm[0] = src1[0] - src2[0];
-        MyBase::mm[1] = src1[1] - src2[1];
-        MyBase::mm[2] = src1[2] - src2[2];
-        MyBase::mm[3] = src1[3] - src2[3];
-        MyBase::mm[4] = src1[4] - src2[4];
-        MyBase::mm[5] = src1[5] - src2[5];
-        MyBase::mm[6] = src1[6] - src2[6];
-        MyBase::mm[7] = src1[7] - src2[7];
-        MyBase::mm[8] = src1[8] - src2[8];
-
-        return *this;
-    } // opMinusTest
-
-    /// this = m * scalar
-    template<typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat3 &scale(T0 scalar, const Mat3<T1> &m)
-    {
-        const T1 *src = m.asPointer();
-
-        MyBase::mm[0] = scalar * src[0];
-        MyBase::mm[1] = scalar * src[1];
-        MyBase::mm[2] = scalar * src[2];
-        MyBase::mm[3] = scalar * src[3];
-        MyBase::mm[4] = scalar * src[4];
-        MyBase::mm[5] = scalar * src[5];
-        MyBase::mm[6] = scalar * src[6];
-        MyBase::mm[7] = scalar * src[7];
-        MyBase::mm[8] = scalar * src[8];
-
-        return *this;
-    }
-
-    /// this = m1 * m2
-    /// @note m1 and m2 must not be the same object as this.
-    template<typename T0, typename T1>
-OPENVDB_DEPRECATED const Mat3 &mult(const Mat3<T0> &m1, const Mat3<T1> &m2)
-    {
-        const T0 *src1 = m1.asPointer();
-        const T1 *src2 = m2.asPointer();
-        // assert(this!=&m1);
-        // assert(this!=&m2);
-
-        MyBase::mm[0] = static_cast<T>(src1[0] * src2[0] +
-                                       src1[1] * src2[3] +
-                                       src1[2] * src2[6]);
-        MyBase::mm[1] = static_cast<T>(src1[0] * src2[1] +
-                                       src1[1] * src2[4] +
-                                       src1[2] * src2[7]);
-        MyBase::mm[2] = static_cast<T>(src1[0] * src2[2] +
-                                       src1[1] * src2[5] +
-                                       src1[2] * src2[8]);
-        
-        MyBase::mm[3] = static_cast<T>(src1[3] * src2[0] +
-                                       src1[4] * src2[3] +
-                                       src1[5] * src2[6]);
-        MyBase::mm[4] = static_cast<T>(src1[3] * src2[1] +
-                                       src1[4] * src2[4] +
-                                       src1[5] * src2[7]);
-        MyBase::mm[5] = static_cast<T>(src1[3] * src2[2] +
-                                       src1[4] * src2[5] +
-                                       src1[5] * src2[8]);
-        
-        MyBase::mm[6] = static_cast<T>(src1[6] * src2[0] +
-                                       src1[7] * src2[3] +
-                                       src1[8] * src2[6]);
-        MyBase::mm[7] = static_cast<T>(src1[6] * src2[1] +
-                                       src1[7] * src2[4] +
-                                       src1[8] * src2[7]);
-        MyBase::mm[8] = static_cast<T>(src1[6] * src2[2] +
-                                       src1[7] * src2[5] +
-                                       src1[8] * src2[8]);
-        return *this;
-    } // multTest
-
-
 private:
     static const Mat3<T> sIdentity;
     static const Mat3<T> sZero;
@@ -780,7 +654,7 @@ template <typename T>
 Mat3<T> outerProduct(const Vec3<T>& v1, const Vec3<T>& v2)
 {
     Mat3<T> m;
-    
+
     m.setBasis(Vec3<T>(v1[0]*v2[0], v1[1]*v2[0], v1[2]*v2[0]),
                Vec3<T>(v1[0]*v2[1], v1[1]*v2[1], v1[2]*v2[1]),
                Vec3<T>(v1[0]*v2[2], v1[1]*v2[2], v1[2]*v2[2]));
@@ -813,7 +687,7 @@ Mat3<T> powLerp(const Mat3<T0> &m1, const Mat3<T0> &m2, T t)
 
 namespace {
     template<typename T>
-    void pivot(int i, int j, Mat3<T>& S, Vec3<T>& D, Mat3<T>& Q) 
+    void pivot(int i, int j, Mat3<T>& S, Vec3<T>& D, Mat3<T>& Q)
     {
         const int& n = Mat3<T>::size;  // should be 3
         T temp;
@@ -823,24 +697,26 @@ namespace {
         double cosin_of_theta;
         double sin_of_theta;
         double z;
-        
+
         double Sij = S(i,j);
-        
+
         double Sjj_minus_Sii = D[j] - D[i];
-        
+
         if (fabs(Sjj_minus_Sii) * (10*tolerance<T>::value()) > fabs(Sij)) {
             tan_of_theta = Sij / Sjj_minus_Sii;
         } else {
             /// pivot on Sij
             cotan_of_2_theta = 0.5*Sjj_minus_Sii / Sij ;
-            
+
             if (cotan_of_2_theta < 0.) {
-                        tan_of_theta = -1./(sqrt(1. + cotan_of_2_theta*cotan_of_2_theta) - cotan_of_2_theta);
+                tan_of_theta =
+                    -1./(sqrt(1. + cotan_of_2_theta*cotan_of_2_theta) - cotan_of_2_theta);
             } else {
-                tan_of_theta = 1./(sqrt(1. + cotan_of_2_theta*cotan_of_2_theta) + cotan_of_2_theta);
+                tan_of_theta =
+                    1./(sqrt(1. + cotan_of_2_theta*cotan_of_2_theta) + cotan_of_2_theta);
             }
         }
-        
+
         cosin_of_theta = 1./sqrt( 1. + tan_of_theta * tan_of_theta);
         sin_of_theta = cosin_of_theta * tan_of_theta;
         z = tan_of_theta * Sij;
@@ -872,26 +748,27 @@ namespace {
 }
 
 
-/// @brief Use jacobi iterations to decompose a symmetric 3x3 matrix (diagonalize and compute eigenvectors)
-/// This is based on the "Efficient numerical diagonalization of Hermitian 3x3 matrices" 
+/// @brief Use Jacobi iterations to decompose a symmetric 3x3 matrix
+/// (diagonalize and compute eigenvectors)
+/// @details This is based on the "Efficient numerical diagonalization of Hermitian 3x3 matrices"
 /// Joachim Kopp.  arXiv.org preprint: physics/0610206
 /// with the addition of largest pivot
 template<typename T>
-bool diagonalizeSymmetricMatrix(const Mat3<T>& input, Mat3<T>& Q, Vec3<T>& D, unsigned int MAX_ITERATIONS=250) 
+bool diagonalizeSymmetricMatrix(const Mat3<T>& input, Mat3<T>& Q, Vec3<T>& D,
+    unsigned int MAX_ITERATIONS=250)
 {
     /// use Givens rotation matrix to eliminate off-diagonal entries.
     /// initialize the rotation matrix as idenity
     Q  = Mat3<T>::identity();
     int n = Mat3<T>::size;  // should be 3
-    
+
     /// temp matrix.  Assumed to be symmetric
     Mat3<T> S(input);
-    
+
     for (int i = 0; i < n; ++i) {
         D[i] = S(i,i);
     }
 
-   
     unsigned int iterations(0);
     /// Just iterate over all the non-diagonal enteries
     /// using the largest as a pivot.
@@ -915,12 +792,12 @@ bool diagonalizeSymmetricMatrix(const Mat3<T>& input, Mat3<T>& Q, Vec3<T>& D, un
         /// loop over all the off-diagonals above the diagonal
         for (int i = 0; i < n; ++i) {
             for (int j = i+1; j < n; ++j){
-                               
+
                 if ( fabs(D[i]) * (10*tolerance<T>::value()) > fabs(S(i,j))) {
                     /// value too small to pivot on
                     S(i,j) = 0;
                 }
-                if (fabs(S(i,j)) > max_element) { 
+                if (fabs(S(i,j)) > max_element) {
                     max_element = fabs(S(i,j));
                     ip = i;
                     jp = j;
@@ -928,19 +805,17 @@ bool diagonalizeSymmetricMatrix(const Mat3<T>& input, Mat3<T>& Q, Vec3<T>& D, un
             }
         }
         pivot(ip, jp, S, D, Q);
-    } while (iterations < MAX_ITERATIONS); 
-        
+    } while (iterations < MAX_ITERATIONS);
+
     return false;
 }
 
-
 } // namespace math
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
-
 #endif // OPENVDB_MATH_MAT3_H_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
