@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -82,6 +82,9 @@ template<> inline std::string zeroVal<std::string>() { return ""; }
 /// Return the bool value that corresponds to zero.
 template<> inline bool zeroVal<bool>() { return false; }
 
+template<typename T> inline T toleranceValue() { return T(1e-8); }
+template<> inline float toleranceValue<float>() { return float(1e-6); }
+
 
 /// @todo These won't be needed if we eliminate StringGrids.
 //@{
@@ -221,13 +224,19 @@ isZero(const Type& x)
     return x == zeroVal<Type>();
     OPENVDB_NO_FP_EQUALITY_WARNING_END
 }
-
+    
+template<typename Type>
+inline bool
+isNegative(const Type& x)
+{
+    return x < zeroVal<Type>();
+}    
 
 template<typename Type>
 inline bool
 isApproxEqual(const Type& a, const Type& b)
 {
-    const Type tolerance = Type(zeroVal<Type>() + 1.0e-8);
+    const Type tolerance = Type(zeroVal<Type>() + toleranceValue<Type>());
     return !(Abs(a - b) > tolerance);
 }
 
@@ -638,6 +647,6 @@ template<> struct tolerance<double>   { static double value() { return 1e-15; }
 
 #endif // OPENVDB_MATH_MATH_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
