@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -2032,9 +2032,8 @@ public:
 
     Vec3d voxelSize() const { return mMap->voxelSize(); }
     Vec3d voxelSize(const Vec3d&v) const { return mMap->voxelSize(v); }
-    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return mMap->voxelDimensions(); }
-    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d&v) const { return mMap->voxelDimensions(v); }
- private:
+
+private:
     MapBase::ConstPtr mMap;
 };
 
@@ -2044,6 +2043,6 @@ public:
 
 #endif // OPENVDB_MATH_OPERATORS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
