@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -90,7 +90,7 @@ public:
         assert(pos < mStencil.size());
         return mStencil[pos];
     }
-    
+
     /// Return the value at the specified location relative to the center of the stencil
     template<int i, int j, int k>
     const ValueType& getValue() const
@@ -98,12 +98,6 @@ public:
         return mStencil[static_cast<const StencilType&>(*this).template pos<i,j,k>()];
     }
 
-    /// Return value at specified the linear off pos
-    OPENVDB_DEPRECATED const ValueType& operator()(int pos) const { return this->getValue(pos); }
-
-    template<int i, int j, int k>
-    OPENVDB_DEPRECATED const ValueType& get() const { return this->template getValue<i,j,k>(); }
-    
     /// Set the value at the specified location relative to the center of the stencil
     template<int i, int j, int k>
     void setValue(const ValueType& value)
@@ -147,12 +141,9 @@ public:
         return *iter;
     }
 
-    /// Return the coordinates of the center point of the stencil.
-    OPENVDB_DEPRECATED inline const Coord& getCenter() const { return mCenter; }
-
     /// Return the coordinates of the center point of the stencil.
     inline const Coord& getCenterCoord() const { return mCenter; }
-    
+
     /// Return the value at the center of the stencil
     inline const ValueType& getCenterValue() const
     {
@@ -216,7 +207,7 @@ public:
     static const int SIZE = 7;
 
     SevenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
-    
+
     /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
     unsigned int pos() const { return SevenPt<i,j,k>::idx; }
@@ -290,7 +281,7 @@ public:
     static const int SIZE = 19;
 
     SecondOrderDenseStencil(const GridType& grid): BaseType(grid, SIZE) {}
-    
+
     /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
     unsigned int pos() const { return DensePt<i,j,k>::idx; }
@@ -370,7 +361,7 @@ public:
     static const int SIZE = 13;
 
     ThirteenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
-    
+
     /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
     unsigned int pos() const { return ThirteenPt<i,j,k>::idx; }
@@ -636,7 +627,7 @@ public:
     typedef typename BaseType::BufferType   BufferType;
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
-    
+
     static const int SIZE = 19;
 
     NineteenPointStencil(const GridType& grid): BaseType(grid, SIZE) {}
@@ -851,11 +842,11 @@ public:
     typedef typename BaseType::BufferType   BufferType;
     typedef typename GridType::ValueType    ValueType;
     typedef math::Vec3<ValueType>           Vec3Type;
-    
+
     static const int SIZE = 127;
 
     SixthOrderDenseStencil(const GridType& grid): BaseType(grid, SIZE) {}
-    
+
     /// Return linear offset for the specified stencil point relative to its center
     template<int i, int j, int k>
     unsigned int pos() const { return SixthDensePt<i,j,k>::idx; }
@@ -1470,6 +1461,6 @@ private:
 
 #endif // OPENVDB_MATH_STENCILS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
