@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -254,9 +254,9 @@ public:
     enum { STOP = 3 };
 
     CsgVisitorBase(const TreeT& aTree, const TreeT& bTree):
-        mAOutside(aTree.getBackground()),
+        mAOutside(aTree.background()),
         mAInside(negative(mAOutside)),
-        mBOutside(bTree.getBackground()),
+        mBOutside(bTree.background()),
         mBInside(negative(mBOutside))
     {
         const ValueT zero = zeroVal<ValueT>();
@@ -550,6 +550,6 @@ csgDifference(GridOrTreeT& a, GridOrTreeT& b, bool prune)
 
 #endif // OPENVDB_TOOLS_COMPOSITE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
