@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -40,7 +40,6 @@
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
 #include <boost/scoped_ptr.hpp>
-#include <openvdb/openvdb.h>
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h>
 #include <openvdb/math/Stencils.h>
@@ -49,7 +48,6 @@
 #include <openvdb/util/NullInterrupter.h>
 #include <openvdb/Grid.h>
 
-
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
@@ -65,39 +63,28 @@ public:
     typedef typename TreeType::LeafNodeType        LeafType;
     typedef typename LeafType::ValueType           ValueType;
     typedef typename tree::LeafManager<TreeType>   LeafManagerType;
-    typedef typename LeafManagerType::RangeType    RangeType;
+    typedef typename LeafManagerType::LeafRange    RangeType;
     typedef typename LeafManagerType::BufferType   BufferType;
 
     /// Constructor
-    explicit Filter(GridT& grid, InterruptT* interrupt = NULL) :
-        mIsMaster(true), mGrid(grid), mTask(0),
-        mLeafs(new LeafManagerType(grid.tree())), mInterrupter(interrupt)
+    Filter(GridT& grid, InterruptT* interrupt = NULL) :
+        mGrid(grid), mTask(0), mInterrupter(interrupt)
     {
     }
-    /// Shallow copy constructor called by tbb::parallel_for() threads during filtering
-    Filter(const Filter& other) :
-        mIsMaster(false), mGrid(other.mGrid), mTask(other.mTask),
-        mLeafs(other.mLeafs), mInterrupter(other.mInterrupter)
-    {
-    }
-    virtual ~Filter()
-    {
-        if ( mIsMaster ) delete mLeafs;
-    }
 
     /// @brief One iteration of a fast separable mean-value (i.e. box) filter.
-    void mean(int width = 1, bool serial = false);
+    void mean(int width = 1, int iterations = 1, bool serial = false);
 
     /// @brief One iteration of a fast separable gaussian filter.
     ///
     /// @note This is approximated as 4 iterations of a separable mean filter
     /// which typically leads an approximation that's better than 95%!
-    void gaussian(int width = 1, bool serial = false);
+    void gaussian(int width = 1, int iterations = 1, bool serial = false);
 
     /// @brief One iteration of a median-value filter
     ///
     /// @note This filter is not separable and is hence relatively slow!
-    void median(int width = 1, bool serial = false);
+    void median(int width = 1, int iterations = 1, bool serial = false);
 
     /// Offsets (i.e. adds) a constant value to all active voxels.
     void offset(float offset, bool serial = false);
@@ -114,7 +101,7 @@ public:
 private:
     typedef typename boost::function<void (Filter*, const RangeType&)> FuncType;
 
-    void cook(bool serial, bool swapBuffers);
+    void cook(bool serial, LeafManagerType& leafs);
 
     // Private filter methods called by tbb::parallel_for threads
     void doBoxX(const RangeType&, Int32);
@@ -122,83 +109,93 @@ private:
     void doBoxZ(const RangeType&, Int32);
     void doMedian(const RangeType&, int);
     void doOffset(const RangeType&, float);
-    void startInterrupter(const char* msg);
-    void endInterrupter();
-    void checkInterrupter();
+    /// @return true if the process was interrupted
+    bool wasInterrupted();
 
-    const bool        mIsMaster;
     GridType&         mGrid;
     FuncType          mTask;
-    LeafManagerType*  mLeafs;
     InterruptT*       mInterrupter;
-
 }; // end of Filter class
 
 ////////////////////////////////////////
 
 template<typename GridT, typename InterruptT>
 inline void
-Filter<GridT, InterruptT>::mean(int width, bool serial)
+Filter<GridT, InterruptT>::mean(int width, int iterations, bool serial)
 {
-    this->startInterrupter("Applying mean filter");
-    width = std::max(1, width);
+    if (mInterrupter) mInterrupter->start("Applying mean filter");
 
-    mLeafs->rebuildAuxBuffers(1, serial);
+    const int w = std::max(1, width);
 
-    mTask = boost::bind(&Filter::doBoxX, _1, _2, width);
-    this->cook(serial, true);
+    LeafManagerType leafs(mGrid.tree(), 1, serial);
 
-    mTask = boost::bind(&Filter::doBoxY, _1, _2, width);
-    this->cook(serial, true);
+    for (int i=0; i<iterations && !this->wasInterrupted(); ++i) {
+        mTask = boost::bind(&Filter::doBoxX, _1, _2, w);
+        this->cook(serial, leafs);
 
-    mTask = boost::bind(&Filter::doBoxZ, _1, _2, width);
-    this->cook(serial, true);
-    this->endInterrupter();
-}
+        mTask = boost::bind(&Filter::doBoxY, _1, _2, w);
+        this->cook(serial, leafs);
 
-template<typename GridT, typename InterruptT>
-inline void
-Filter<GridT, InterruptT>::gaussian(int width, bool serial)
-{
-    this->startInterrupter("Applying gaussian filter");
-    width = std::max(1, width);
-
-    mLeafs->rebuildAuxBuffers(1, serial);
-
-    for (int n=0; n<4; ++n) {
-        mTask = boost::bind(&Filter::doBoxX, _1, _2, width);
-        this->cook(serial, true);
-
-        mTask = boost::bind(&Filter::doBoxY, _1, _2, width);
-        this->cook(serial, true);
-
-        mTask = boost::bind(&Filter::doBoxZ, _1, _2, width);
-        this->cook(serial, true);
+        mTask = boost::bind(&Filter::doBoxZ, _1, _2, w);
+        this->cook(serial, leafs);
     }
-    this->endInterrupter();
+
+    if (mInterrupter) mInterrupter->end();
+}
+
+template<typename GridT, typename InterruptT>
+inline void
+Filter<GridT, InterruptT>::gaussian(int width, int iterations, bool serial)
+{
+    if (mInterrupter) mInterrupter->start("Applying gaussian filter");
+
+    const int w = std::max(1, width);
+
+    LeafManagerType leafs(mGrid.tree(), 1, serial);
+
+    for (int i=0; i<iterations; ++i) {
+        for (int n=0; n<4 && !this->wasInterrupted(); ++n) {
+            mTask = boost::bind(&Filter::doBoxX, _1, _2, w);
+            this->cook(serial, leafs);
+
+            mTask = boost::bind(&Filter::doBoxY, _1, _2, w);
+            this->cook(serial, leafs);
+
+            mTask = boost::bind(&Filter::doBoxZ, _1, _2, w);
+            this->cook(serial, leafs);
+        }
+    }
+
+    if (mInterrupter) mInterrupter->end();
 }
 
 
 template<typename GridT, typename InterruptT>
 inline void
-Filter<GridT, InterruptT>::median(int width, bool serial)
+Filter<GridT, InterruptT>::median(int width, int iterations, bool serial)
 {
-    this->startInterrupter("Applying median filter");
-    mLeafs->rebuildAuxBuffers(1, serial);
+    if (mInterrupter) mInterrupter->start("Applying median filter");
+
+    LeafManagerType leafs(mGrid.tree(), 1, serial);
+
     mTask = boost::bind(&Filter::doMedian, _1, _2, std::max(1, width));
-    this->cook(serial, true);
-    this->endInterrupter();
+    for (int i=0; i<iterations && !this->wasInterrupted(); ++i) this->cook(serial, leafs);
+
+    if (mInterrupter) mInterrupter->end();
 }
 
 template<typename GridT, typename InterruptT>
 inline void
 Filter<GridT, InterruptT>::offset(float value, bool serial)
 {
-    this->startInterrupter("Applying offset");
-    mLeafs->removeAuxBuffers();
+    if (mInterrupter) mInterrupter->start("Applying offset");
+
+    LeafManagerType leafs(mGrid.tree(), 0, serial);
+
     mTask = boost::bind(&Filter::doOffset, _1, _2, value);
-    this->cook(serial, false);
-    this->endInterrupter();
+    this->cook(serial, leafs);
+
+    if (mInterrupter) mInterrupter->end();
 }
 
 ////////////////////////////////////////
@@ -208,14 +205,14 @@ Filter<GridT, InterruptT>::offset(float value, bool serial)
 /// subsequently swap the leaf buffers.
 template<typename GridT, typename InterruptT>
 inline void
-Filter<GridT, InterruptT>::cook(bool serial, bool swapBuffers)
+Filter<GridT, InterruptT>::cook(bool serial, LeafManagerType& leafs)
 {
     if (serial) {
-        (*this)(mLeafs->getRange());
+        (*this)(leafs.leafRange());
     } else {
-        tbb::parallel_for(mLeafs->getRange(), *this);
+        tbb::parallel_for(leafs.leafRange(), *this);
     }
-    if (swapBuffers) mLeafs->swapLeafBuffer(1, serial);
+    leafs.swapLeafBuffer(1, serial);
 }
 
 /// X convolution of a separable box filter
@@ -223,19 +220,18 @@ template<typename GridT, typename InterruptT>
 inline void
 Filter<GridT, InterruptT>::doBoxX(const RangeType& range, Int32 w)
 {
-    this->checkInterrupter();
+    this->wasInterrupted();
     const ValueType frac = ValueType(1)/ValueType(2*w+1);
     typename GridT::ConstAccessor acc = mGrid.getConstAccessor();
-    for (size_t n=range.begin(), nLast = range.end(); n != nLast; ++n) {
-        const LeafType& leaf = mLeafs->leaf(n);
-        BufferType& buffer   = mLeafs->getBuffer(n, 1);
-        for (typename LeafType::ValueOnCIter iter = leaf.cbeginValueOn(); iter; ++iter) {
+    for (typename RangeType::Iterator lIter=range.begin(); lIter; ++lIter) {
+        BufferType& buffer = lIter.buffer(1);
+        for (typename LeafType::ValueOnCIter vIter = lIter->cbeginValueOn(); vIter; ++vIter) {
             ValueType sum = zeroVal<ValueType>();
-            math::Coord xyz = iter.getCoord();
+            math::Coord xyz = vIter.getCoord();
             for (Int32 x = xyz.x()-w, xLast = xyz.x()+w; x <= xLast; ++x) {
                 sum += acc.getValue(xyz.setX(x));
             }
-            buffer.setValue(iter.pos(), sum*frac);
+            buffer.setValue(vIter.pos(), sum*frac);
         }
     }
 }
@@ -245,19 +241,18 @@ template<typename GridT, typename InterruptT>
 inline void
 Filter<GridT, InterruptT>::doBoxY(const RangeType& range, Int32 w)
 {
-    this->checkInterrupter();
+    this->wasInterrupted();
     const ValueType frac = ValueType(1)/ValueType(2*w+1);
     typename GridT::ConstAccessor acc = mGrid.getConstAccessor();
-    for (size_t n=range.begin(), nLast = range.end(); n != nLast; ++n) {
-        const LeafType& leaf = mLeafs->leaf(n);
-        BufferType& buffer   = mLeafs->getBuffer(n, 1);
-        for (typename LeafType::ValueOnCIter iter = leaf.cbeginValueOn(); iter; ++iter) {
+    for (typename RangeType::Iterator lIter=range.begin(); lIter; ++lIter) {
+        BufferType& buffer = lIter.buffer(1);
+        for (typename LeafType::ValueOnCIter vIter = lIter->cbeginValueOn(); vIter; ++vIter) {
             ValueType sum = zeroVal<ValueType>();
-            math::Coord xyz = iter.getCoord();
+            math::Coord xyz = vIter.getCoord();
             for (Int32 y = xyz.y()-w, yLast = xyz.y()+w; y <= yLast; ++y) {
                 sum += acc.getValue(xyz.setY(y));
             }
-            buffer.setValue(iter.pos(), sum*frac);
+            buffer.setValue(vIter.pos(), sum*frac);
         }
     }
 }
@@ -267,19 +262,18 @@ template<typename GridT, typename InterruptT>
 inline void
 Filter<GridT, InterruptT>::doBoxZ(const RangeType& range, Int32 w)
 {
-    this->checkInterrupter();
+    this->wasInterrupted();
     const ValueType frac = ValueType(1)/ValueType(2*w+1);
     typename GridT::ConstAccessor acc = mGrid.getConstAccessor();
-    for (size_t n=range.begin(), nLast = range.end(); n != nLast; ++n) {
-        const LeafType& leaf = mLeafs->leaf(n);
-        BufferType& buffer   = mLeafs->getBuffer(n, 1);
-        for (typename LeafType::ValueOnCIter iter = leaf.cbeginValueOn(); iter; ++iter) {
+    for (typename RangeType::Iterator lIter=range.begin(); lIter; ++lIter) {
+        BufferType& buffer = lIter.buffer(1);
+        for (typename LeafType::ValueOnCIter vIter = lIter->cbeginValueOn(); vIter; ++vIter) {
             ValueType sum = zeroVal<ValueType>();
-            math::Coord xyz = iter.getCoord();
+            math::Coord xyz = vIter.getCoord();
             for (Int32 z = xyz.z()-w, zLast = xyz.z()+w; z <= zLast; ++z) {
                 sum += acc.getValue(xyz.setZ(z));
             }
-            buffer.setValue(iter.pos(), sum*frac);
+            buffer.setValue(vIter.pos(), sum*frac);
         }
     }
 }
@@ -289,14 +283,13 @@ template<typename GridT, typename InterruptT>
 inline void
 Filter<GridT, InterruptT>::doMedian(const RangeType& range, int width)
 {
-    this->checkInterrupter();
+    this->wasInterrupted();
     typename math::DenseStencil<GridType> stencil(mGrid, width);//creates local cache!
-    for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
-        const LeafType& leaf = mLeafs->leaf(n);
-        BufferType& buffer   = mLeafs->getBuffer(n, 1);
-        for (typename LeafType::ValueOnCIter iter = leaf.cbeginValueOn(); iter; ++iter) {
-            stencil.moveTo(iter);
-            buffer.setValue(iter.pos(), stencil.median());
+    for (typename RangeType::Iterator lIter=range.begin(); lIter; ++lIter) {
+        BufferType& buffer = lIter.buffer(1);
+        for (typename LeafType::ValueOnCIter vIter = lIter->cbeginValueOn(); vIter; ++vIter) {
+            stencil.moveTo(vIter);
+            buffer.setValue(vIter.pos(), stencil.median());
         }
     }
 }
@@ -307,28 +300,18 @@ inline void
 Filter<GridT, InterruptT>::doOffset(const RangeType& range, float floatVal)
 {
     const ValueType value = static_cast<ValueType>(floatVal);
-    for (size_t n=range.begin(), e=range.end(); n != e; ++n) mLeafs->leaf(n).addValue(value);
+    for (typename RangeType::Iterator lIter=range.begin(); lIter; ++lIter) lIter->addValue(value);
 }
 
 template<typename GridT, typename InterruptT>
-inline void
-Filter<GridT, InterruptT>::startInterrupter(const char* msg)
+inline bool
+Filter<GridT, InterruptT>::wasInterrupted()
 {
-    if (mInterrupter) mInterrupter->start(msg);
-}
-
-template<typename GridT, typename InterruptT>
-inline void
-Filter<GridT, InterruptT>::endInterrupter()
-{
-    if (mInterrupter) mInterrupter->end();
-}
-
-template<typename GridT, typename InterruptT>
-inline void
-Filter<GridT, InterruptT>::checkInterrupter()
-{
-    if (util::wasInterrupted(mInterrupter)) tbb::task::self().cancel_group_execution();
+    if (util::wasInterrupted(mInterrupter)) {
+        tbb::task::self().cancel_group_execution();
+        return true;
+    }
+    return false;
 }
 
 } // namespace tools
@@ -337,6 +320,6 @@ Filter<GridT, InterruptT>::checkInterrupter()
 
 #endif // OPENVDB_TOOLS_FILTER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
