@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -29,17 +29,19 @@
 ///////////////////////////////////////////////////////////////////////////
 //
 /// @file GridOperators.h
+///
+/// @brief Applies an operator on an input grid to produce an output
+/// grid with the same topology but potentially different value type.
 
 #ifndef OPENVDB_TOOLS_GRID_OPERATORS_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_GRID_OPERATORS_HAS_BEEN_INCLUDED
 
 #include <openvdb/Grid.h>
 #include <openvdb/math/Operators.h>
-#include <openvdb/math/Stencils.h>
+#include <openvdb/util/NullInterrupter.h>
 #include <openvdb/tree/LeafManager.h>
 #include <openvdb/tree/ValueAccessor.h>
 #include <tbb/parallel_for.h>
-#include <tbb/blocked_range.h>
 
 
 namespace openvdb {
@@ -66,87 +68,154 @@ template<typename ScalarGridType> struct ScalarToVectorConverter {
 };
 
 
+/// @brief Compute the Closest-Point Transform (CPT) from a distance field.
+/// @return a new vector-valued grid with the same numerical precision as the input grid
+///     (for example, if the input grid is a DoubleGrid, the output grid will be a Vec3DGrid)
+///
+/// @note The current implementation assumes all the input distance values
+/// are represented by leaf voxels and not tiles.  This is true for all
+/// narrow-band level sets, which this class was originally developed for.
+/// In the future we will expand this class to also handle tile values.
+template<typename GridType, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+cpt(const GridType& grid, bool threaded, InterruptT* interrupt);
+    
+template<typename GridType> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+cpt(const GridType& grid, bool threaded = true)
+{
+    return cpt<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
+
+
 /// @brief Compute the curl of the given vector-valued grid.
 /// @return a new vector-valued grid
+template<typename GridType, typename InterruptT> inline
+typename GridType::Ptr
+curl(const GridType& grid, bool threaded, InterruptT* interrupt);
+    
 template<typename GridType> inline
 typename GridType::Ptr
-curl(const GridType& grid, bool threaded = true);
-
+curl(const GridType& grid, bool threaded = true)
+{
+    return curl<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
 
 /// @brief Compute the divergence of the given vector-valued grid.
 /// @return a new scalar-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a Vec3DGrid, the output grid will be a DoubleGrid)
+template<typename GridType, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+divergence(const GridType& grid, bool threaded, InterruptT* interrupt);
+    
 template<typename GridType> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
-divergence(const GridType& grid, bool threaded = true);
-
+divergence(const GridType& grid, bool threaded = true)
+{
+    return divergence<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
 
 /// @brief Compute the gradient of the given scalar grid.
 /// @return a new vector-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a DoubleGrid, the output grid will be a Vec3DGrid)
+template<typename GridType, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+gradient(const GridType& grid, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
-gradient(const GridType& grid, bool threaded = true);
+gradient(const GridType& grid, bool threaded = true)
+{
+    return gradient<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
 
 
 /// @brief Compute the Laplacian of the given scalar grid.
 /// @return a new scalar grid
+template<typename GridType, typename InterruptT> inline
+typename GridType::Ptr
+laplacian(const GridType& grid, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename GridType::Ptr
-laplacian(const GridType& grid, bool threaded = true);
-
-
+laplacian(const GridType& grid, bool threaded = true)
+{
+    return laplacian<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
+    
 /// @brief Compute the mean curvature of the given grid.
 /// @return a new grid
+template<typename GridType, typename InterruptT> inline
+typename GridType::Ptr
+meanCurvature(const GridType& grid, bool threaded, InterruptT* interrupt);
+
 template<typename GridType> inline
 typename GridType::Ptr
-meanCurvature(const GridType& grid, bool threaded = true);
+meanCurvature(const GridType& grid, bool threaded = true)    
+{
+    return meanCurvature<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
+
+/// @brief Compute the magnitudes of the vectors of the given vector-valued grid.
+/// @return a new scalar-valued grid with the same numerical precision as the input grid
+///     (for example, if the input grid is a Vec3DGrid, the output grid will be a DoubleGrid)
+template<typename GridType, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+magnitude(const GridType& grid, bool threaded, InterruptT* interrupt);
     
+template<typename GridType> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+magnitude(const GridType& grid, bool threaded = true) 
+{
+    return magnitude<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
+    
+/// @brief Normalize the vectors of the given vector-valued grid.
+/// @return a new vector-valued grid
+template<typename GridType, typename InterruptT> inline
+typename GridType::Ptr
+normalize(const GridType& grid, bool threaded, InterruptT* interrupt);
 
-/// @brief Compute the magnitude of the given grid.
-/// @return a new grid
 template<typename GridType> inline
 typename GridType::Ptr
-magnitude(const GridType& grid, bool threaded = true);
-
-
-/// @brief Normalizes the given vector grid.
-/// @return a new grid
-template<typename GridType> inline
-typename GridType::Ptr
-normalize(const GridType& grid, bool threaded = true);      
-
+normalize(const GridType& grid, bool threaded = true)
+{
+    return normalize<GridType, util::NullInterrupter>(grid, threaded, NULL);
+}
+    
 ////////////////////////////////////////
 
 
 namespace {
 
-/// @brief Applies an operator on an input grid to produce an output
-/// grid with the same topology but potentially different value type.
+/// @brief Apply an operator on an input grid to produce an output grid
+/// with the same topology but a possibly different value type.
+/// @details To facilitate inlining, this class is also templated on a Map type.
 ///
-/// @note This is a helper class and should never be used directly
+/// @note This is a helper class and should never be used directly.
 ///
 /// @note The current implementation assumes all the input
 /// values are represented by leaf voxels and not tiles. In the
 /// future we will expand this class to also handle tile values.
-template<typename InGridT, typename OutGridT, typename MapT, typename OperatorT>
+template<typename InGridT, typename OutGridT, typename MapT,
+         typename OperatorT, typename InterruptT = util::NullInterrupter>
 class GridOperator
 {
 public:
     typedef typename OutGridT::TreeType           OutTreeT;
     typedef typename OutTreeT::LeafNodeType       OutLeafT;
     typedef typename tree::LeafManager<OutTreeT>  LeafManagerT;
-    typedef typename LeafManagerT::RangeType      RangeT;
 
-    GridOperator(const InGridT& grid, const MapT& map):
-        mAcc(grid.getConstAccessor()), mMap(map), mLeafs(NULL)
+    GridOperator(const InGridT& grid, const MapT& map, InterruptT* interrupt = NULL):
+        mAcc(grid.getConstAccessor()), mMap(map), mInterrupt(interrupt)
     {
     }
     virtual ~GridOperator() {}
     typename OutGridT::Ptr process(bool threaded = true)
     {
+        if (mInterrupt) mInterrupt->start("Processing grid");
         // Derive background value of the output grid
-        typename InGridT::TreeType tmp(mAcc.getTree()->getBackground());
+        typename InGridT::TreeType tmp(mAcc.getTree()->background());
         typename OutGridT::ValueType backg = OperatorT::result(mMap, tmp, math::Coord(0));
         // output tree = topology copy of input tree!
         typename OutTreeT::Ptr tree(new OutTreeT(*mAcc.getTree(), backg, TopologyCopy()));
@@ -154,15 +223,16 @@ public:
         typename OutGridT::Ptr result(new OutGridT(tree));
         // transform of output grid = transform of input grid
         result->setTransform(math::Transform::Ptr(new math::Transform( mMap.copy() )));
-
-        mLeafs = new LeafManagerT(*tree);
+       
+        LeafManagerT leafManager(*tree);
+        
         if (threaded) {
-            tbb::parallel_for(mLeafs->getRange(), *this);
+            tbb::parallel_for(leafManager.leafRange(), *this);
         } else {
-            (*this)(mLeafs->getRange());
+            (*this)(leafManager.leafRange());
         }
-        delete mLeafs; mLeafs=NULL;
-
+        
+        if (mInterrupt) mInterrupt->end();
         return result;
     }
 
@@ -172,23 +242,23 @@ public:
     ///
     /// @note Never call this public method directly - it is called by
     /// TBB threads only!
-    void operator()(const RangeT& range) const
+    void operator()(const typename LeafManagerT::LeafRange& range) const
     {
-        assert(mLeafs);
-        for (size_t n=range.begin(), e=range.end(); n != e; ++n) {
-            OutLeafT& leaf = mLeafs->leaf(n);
-            for (typename OutLeafT::ValueOnIter iter = leaf.beginValueOn(); iter; ++iter) {
-                iter.setValue(OperatorT::result(mMap, mAcc, iter.getCoord()));
+        if (util::wasInterrupted(mInterrupt)) tbb::task::self().cancel_group_execution();
+        
+        for (typename LeafManagerT::LeafRange::Iterator leaf=range.begin(); leaf; ++leaf) {
+            for (typename OutLeafT::ValueOnIter value=leaf->beginValueOn(); value; ++value) {
+                value.setValue(OperatorT::result(mMap, mAcc, value.getCoord()));
             }
         }
     }
 
 protected:
+    
     typedef typename InGridT::ConstAccessor  AccessorT;
-
-    mutable AccessorT  mAcc;
-    const MapT&        mMap;
-    LeafManagerT*      mLeafs;
+    mutable AccessorT mAcc;
+    const MapT&       mMap;
+    InterruptT*       mInterrupt;
 }; // end of GridOperator class
 
 } //end of anonymous namespace
@@ -197,39 +267,110 @@ protected:
 ////////////////////////////////////////
 
 
-/// @brief Computes the Curl of a scalar grid
-template<typename GridT>
+/// @brief Compute the closest-point transform of a scalar grid.
+template<typename InGridT, typename InterruptT = util::NullInterrupter>
+class Cpt
+{
+public:
+    typedef InGridT                                         InGridType;
+    typedef typename ScalarToVectorConverter<InGridT>::Type OutGridType;
+
+    Cpt(const InGridType& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
+    typename OutGridType::Ptr process(bool threaded = true, bool useWorldTransform = true)
+    {
+        Functor functor(mInputGrid, threaded, useWorldTransform, mInterrupt);
+        processTypedMap(mInputGrid.transform(), functor);
+        return functor.mOutputGrid;
+    }
+private:
+    struct IsOpT
+    {
+        template<typename MapT, typename AccT>
+        static typename OutGridType::ValueType
+        result(const MapT& map, const AccT& acc, const Coord& xyz)
+        {
+            return math::CPT<MapT, math::CD_2ND>::result(map, acc, xyz);
+        }
+    };
+    struct WsOpT
+    {
+        template<typename MapT, typename AccT>
+        static typename OutGridType::ValueType
+        result(const MapT& map, const AccT& acc, const Coord& xyz)
+        {
+            return math::CPT_RANGE<MapT, math::CD_2ND>::result(map, acc, xyz);
+        }
+    };
+    struct Functor
+    {
+        Functor(const InGridType& grid, bool threaded, bool worldspace, InterruptT* interrupt):
+            mThreaded(threaded), mWorldSpace(worldspace), mInputGrid(grid), mInterrupt(interrupt){}
+        template<typename MapT>
+        void operator()(const MapT& map)
+        {
+            if (mWorldSpace) {
+                GridOperator<InGridType, OutGridType, MapT, WsOpT, InterruptT> op(mInputGrid, map, mInterrupt);
+                mOutputGrid = op.process(mThreaded); // cache the result
+            } else {
+                GridOperator<InGridType, OutGridType, MapT, IsOpT, InterruptT> op(mInputGrid, map, mInterrupt);
+                mOutputGrid = op.process(mThreaded); // cache the result
+            }
+        }
+        const bool                mThreaded;
+        const bool                mWorldSpace;
+        const InGridType&         mInputGrid;
+        typename OutGridType::Ptr mOutputGrid;
+        InterruptT*               mInterrupt;
+    };
+    const InGridType& mInputGrid;
+    InterruptT*       mInterrupt;
+}; // end of Cpt class
+
+
+////////////////////////////////////////
+
+
+/// @brief Compute the curl of a scalar grid.
+template<typename GridT, typename InterruptT = util::NullInterrupter>
 class Curl
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
-    Curl(const GridT& grid): mInputGrid(grid) {}
-    virtual ~Curl() {}
+    Curl(const GridT& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded);
+        Functor functor(mInputGrid, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
 
-protected:
+private:
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt){}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Curl<MapT, math::CD_2ND> OpT;
-            GridOperator<GridT, GridT, MapT, OpT> op(mInputGrid, map);
+            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
+        InterruptT*          mInterrupt;
     }; // Private Functor
 
     const GridT& mInputGrid;
+    InterruptT*  mInterrupt;
 }; // end of Curl class
 
 
@@ -237,24 +378,26 @@ protected:
 
 
 /// @brief Computes the Divergence of a scalar grid
-template<typename InGridT>
+template<typename InGridT, typename InterruptT = util::NullInterrupter>
 class Divergence
 {
 public:
     typedef InGridT                                         InGridType;
     typedef typename VectorToScalarConverter<InGridT>::Type OutGridType;
 
-    Divergence(const InGridT& grid): mInputGrid(grid) {}
-    virtual ~Divergence() {}
+    Divergence(const InGridT& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename OutGridType::Ptr process(bool threaded = true)
     {
         if( mInputGrid.getGridClass() == GRID_STAGGERED ) {
-            Functor<math::FD_1ST> functor(mInputGrid, threaded);
+            Functor<math::FD_1ST> functor(mInputGrid, threaded, mInterrupt);
             processTypedMap(mInputGrid.transform(), functor);
             return functor.mOutputGrid;
         }
         else {
-            Functor<math::CD_2ND> functor(mInputGrid, threaded);
+            Functor<math::CD_2ND> functor(mInputGrid, threaded, mInterrupt);
             processTypedMap(mInputGrid.transform(), functor);
             return functor.mOutputGrid;
         }
@@ -264,20 +407,23 @@ protected:
     template<math::DScheme DiffScheme>
     struct Functor
     {
-        Functor(const InGridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const InGridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Divergence<MapT, DiffScheme> OpT;
-            GridOperator<InGridType, OutGridType, MapT, OpT> op(mInputGrid, map);
+            GridOperator<InGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool                 mThreaded;
         const InGridType&          mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
+        InterruptT*                mInterrupt;
     }; // Private Functor
 
     const InGridType& mInputGrid;
+    InterruptT*       mInterrupt;
 }; // end of Divergence class
 
 
@@ -285,18 +431,20 @@ protected:
 
 
 /// @brief Computes the Gradient of a scalar grid
-template<typename InGridT>
+template<typename InGridT, typename InterruptT = util::NullInterrupter>
 class Gradient
 {
 public:
     typedef InGridT                                         InGridType;
     typedef typename ScalarToVectorConverter<InGridT>::Type OutGridType;
 
-    Gradient(const InGridT& grid): mInputGrid(grid) {}
-    virtual ~Gradient() {}
+    Gradient(const InGridT& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename OutGridType::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded);
+        Functor functor(mInputGrid, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -304,20 +452,23 @@ public:
 protected:
     struct Functor
     {
-        Functor(const InGridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const InGridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Gradient<MapT, math::CD_2ND> OpT;
-            GridOperator<InGridType, OutGridType, MapT, OpT> op(mInputGrid, map);
+            GridOperator<InGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool                 mThreaded;
         const InGridT&             mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
+        InterruptT*                mInterrupt;
     }; // Private Functor
 
     const InGridT& mInputGrid;
+    InterruptT*    mInterrupt;
 }; // end of Gradient class
 
 
@@ -325,17 +476,19 @@ protected:
 
 
 /// @brief Computes the Laplacian of a scalar grid
-template<typename GridT>
+template<typename GridT, typename InterruptT = util::NullInterrupter>
 class Laplacian
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
-    Laplacian(const GridT& grid): mInputGrid(grid) {}
-    virtual ~Laplacian() {}
+    Laplacian(const GridT& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded);
+        Functor functor(mInputGrid, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -343,37 +496,42 @@ public:
 protected:
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Laplacian<MapT, math::CD_SECOND> OpT;
-            GridOperator<GridT, GridT, MapT, OpT> op(mInputGrid, map);
+            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
+        InterruptT*          mInterrupt;
     }; // Private Functor
 
     const GridT& mInputGrid;
+    InterruptT*  mInterrupt;
 }; // end of Laplacian class
 
 
 ////////////////////////////////////////
 
 
-template<typename GridT>
+template<typename GridT, typename InterruptT = util::NullInterrupter>
 class MeanCurvature
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
-    MeanCurvature(const GridT& grid): mInputGrid(grid) {}
-    virtual ~MeanCurvature() {}
+    MeanCurvature(const GridT& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded);
+        Functor functor(mInputGrid, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -381,36 +539,42 @@ public:
 protected:
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::MeanCurvature<MapT, math::CD_SECOND, math::CD_2ND> OpT;
-            GridOperator<GridT, GridT, MapT, OpT> op(mInputGrid, map);
+            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
+        InterruptT*          mInterrupt;
     }; // Private Functor
 
     const GridT& mInputGrid;
+    InterruptT*  mInterrupt;
 }; // end of MeanCurvature class
 
 
 ////////////////////////////////////////
-    
-template<typename InGridT>
+
+
+template<typename InGridT, typename InterruptT = util::NullInterrupter>
 class Magnitude
 {
 public:
     typedef InGridT                                         InGridType;
     typedef typename VectorToScalarConverter<InGridT>::Type OutGridType;
-    Magnitude(const InGridType& grid): mInputGrid(grid) {}
-    virtual ~Magnitude() {}
+    Magnitude(const InGridType& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename OutGridType::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded);
+        Functor functor(mInputGrid, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -424,36 +588,41 @@ protected:
     };
     struct Functor
     {
-        Functor(const InGridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const InGridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
-            GridOperator<InGridType, OutGridType, MapT, OpT> op(mInputGrid, map);
+            GridOperator<InGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool                 mThreaded;
         const InGridType&          mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
+        InterruptT*                mInterrupt;
     }; // Private Functor
 
     const InGridType& mInputGrid;
+    InterruptT*       mInterrupt;
 }; // end of Magnitude class
 
 
 ////////////////////////////////////////
-    
-    
-template<typename GridT>
+
+
+template<typename GridT, typename InterruptT = util::NullInterrupter>
 class Normalize
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
-    Normalize(const GridT& grid): mInputGrid(grid) {}
-    virtual ~Normalize() {}
+    Normalize(const GridT& grid, InterruptT* interrupt = NULL):
+        mInputGrid(grid), mInterrupt(interrupt)
+    {
+    }
     typename GridT::Ptr process(bool threaded = true)
     {
-        Functor functor(mInputGrid, threaded);
+        Functor functor(mInputGrid, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
         return functor.mOutputGrid;
     }
@@ -472,79 +641,91 @@ protected:
     };
     struct Functor
     {
-        Functor(const GridT& grid, bool threaded): mThreaded(threaded), mInputGrid(grid) {}
+        Functor(const GridT& grid, bool threaded, InterruptT* interrupt):
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt) {}
         template<typename MapT>
         void operator()(const MapT& map)
         {
-            GridOperator<GridT, GridT, MapT, OpT> op(mInputGrid, map);
+            GridOperator<GridT, GridT, MapT, OpT, InterruptT> op(mInputGrid, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
+        InterruptT*          mInterrupt;
     }; // Private Functor
 
     const GridT& mInputGrid;
+    InterruptT*  mInterrupt;
 }; // end of Normalize class
 
+
 ////////////////////////////////////////
 
-    
-template<typename GridType> inline
-typename GridType::Ptr
-curl(const GridType& grid, bool threaded)
-{
-    Curl<GridType> op(grid);
-    return op.process(threaded);
-}
 
-template<typename GridType> inline
-typename VectorToScalarConverter<GridType>::Type::Ptr
-divergence(const GridType& grid, bool threaded)
-{
-    Divergence<GridType> op(grid);
-    return op.process(threaded);
-}
-
-template<typename GridType> inline
+template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
-gradient(const GridType& grid, bool threaded)
+cpt(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Gradient<GridType> op(grid);
+    Cpt<GridType, InterruptT> op(grid, interrupt);
     return op.process(threaded);
 }
 
-template<typename GridType> inline
+template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
-laplacian(const GridType& grid, bool threaded)
+curl(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Laplacian<GridType> op(grid);
+    Curl<GridType, InterruptT> op(grid, interrupt);
     return op.process(threaded);
 }
 
-template<typename GridType> inline
-typename GridType::Ptr
-meanCurvature(const GridType& grid, bool threaded)
+template<typename GridType, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+divergence(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    MeanCurvature<GridType> op(grid);
+    Divergence<GridType, InterruptT> op(grid, interrupt);
     return op.process(threaded);
 }
 
-template<typename GridType> inline
-typename GridType::Ptr
-magnitude(const GridType& grid, bool threaded)
+template<typename GridType, typename InterruptT> inline
+typename ScalarToVectorConverter<GridType>::Type::Ptr
+gradient(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Magnitude<GridType> op(grid);
+    Gradient<GridType, InterruptT> op(grid, interrupt);
     return op.process(threaded);
 }
 
-template<typename GridType> inline
+template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
-normalize(const GridType& grid, bool threaded)
+laplacian(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Normalize<GridType> op(grid);
+    Laplacian<GridType, InterruptT> op(grid, interrupt);
     return op.process(threaded);
-}  
+}
+
+template<typename GridType, typename InterruptT> inline
+typename GridType::Ptr
+meanCurvature(const GridType& grid, bool threaded, InterruptT* interrupt)
+{
+    MeanCurvature<GridType, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename InterruptT> inline
+typename VectorToScalarConverter<GridType>::Type::Ptr
+magnitude(const GridType& grid, bool threaded, InterruptT* interrupt)
+{
+    Magnitude<GridType, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
+
+template<typename GridType, typename InterruptT> inline
+typename GridType::Ptr
+normalize(const GridType& grid, bool threaded, InterruptT* interrupt)
+{
+    Normalize<GridType, InterruptT> op(grid, interrupt);
+    return op.process(threaded);
+}
 
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
@@ -552,6 +733,6 @@ normalize(const GridType& grid, bool threaded)
 
 #endif // OPENVDB_TOOLS_GRID_OPERATORS_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
