@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,7 +31,7 @@
 /// @file Interpolation.h
 ///
 /// Sampler classes such as PointSampler and BoxSampler that are intended for use
-/// with tools::GridTransformer, should operate in voxel space and must adhere to
+/// with tools::GridTransformer should operate in voxel space and must adhere to
 /// the interface described in the example below:
 /// @code
 /// struct MySampler
@@ -69,7 +69,9 @@
 
 #include <cmath>
 #include <boost/shared_ptr.hpp>
+#include <openvdb/version.h> // for OPENVDB_VERSION_NAME
 #include <openvdb/Platform.h> // for round()
+#include <openvdb/math/Transform.h> // for Transform
 
 
 namespace openvdb {
@@ -256,102 +258,6 @@ private:
 ////////////////////////////////////////
 
 
-/// @brief Base class that provides the interface for continuous sampling
-/// of values in a grid.
-/// @details Since grids support only discrete voxel sampling, GridSampling
-/// must be used to sample arbitrary continuous points in space.
-/// @deprecated Use GridSampler instead.
-template<typename TreeType>
-class GridSampling
-{
-public:
-    typedef boost::shared_ptr<GridSampling>                 Ptr;
-    typedef typename TreeType::ValueType                    ValueType;
-    typedef typename tree::ValueAccessor<const TreeType>    ConstAccessor;
-
-    /// Constructor
-    explicit GridSampling(const TreeType& tree): mTree(&tree) {}
-
-    /// Destructor
-    virtual ~GridSampling() {}
-
-    const TreeType& tree() const { return *mTree; }
-
-    /// @brief Sample a point in voxel space in the grid.
-    /// @param pt the grid point to be sampled, in voxel coordinates
-    ValueType sampleVoxel(const Vec3R& pt) const { return sampleVoxel(pt[0], pt[1], pt[2]); }
-
-    /// @brief Sample a point in voxel space in the grid.
-    /// @param x x-coordinate of point in voxel-coordinates of grid
-    /// @param y y-coordinate of point in voxel-coordinates of grid
-    /// @param z z-coordinate of point in voxel-coordinates of grid
-    virtual ValueType sampleVoxel(Real x, Real y, Real z) const = 0;
-
-private:
-    // The grid we are sampling.
-    const TreeType* mTree;
-};
-
-
-////////////////////////////////////////
-
-
-/// @brief Perform linear interpolation on a grid voxel to sample space.
-template<typename TreeType>
-class LinearInterp: public GridSampling<TreeType>
-{
-public:
-    typedef GridSampling<TreeType>          BaseType;
-    typedef boost::shared_ptr<LinearInterp> Ptr;
-    typedef typename TreeType::ValueType    ValueType;
-
-    /// @deprecated Use GridSampler<TreeType, openvdb::tools::BoxSampler> instead.
-    OPENVDB_DEPRECATED explicit LinearInterp(const TreeType& tree): BaseType(tree) {}
-    virtual ~LinearInterp() {}
-
-    /// @brief Sample space at a point defined by (x, y, z) in a grid by linear
-    /// interpolation of values in adjacent voxels.
-    /// @param x x-coordinate of point in voxel-coordinates of grid
-    /// @param y y-coordinate of point in voxel-coordinates of grid
-    /// @param z z-coordinate of point in voxel-coordinates of grid
-    virtual ValueType sampleVoxel(Real x, Real y, Real z) const
-    {
-        ValueType result = zeroVal<ValueType>();
-        BoxSampler::sample(this->tree(), Vec3R(x, y, z), result);
-        return result;
-    }
-};
-
-
-////////////////////////////////////////
-
-
-template<typename TreeType>
-class QuadraticInterp: public GridSampling<TreeType>
-{
-public:
-    typedef GridSampling<TreeType>              BaseType;
-    typedef boost::shared_ptr<QuadraticInterp>  Ptr;
-    typedef typename TreeType::Ptr              TreePtr;
-    typedef typename TreeType::ValueType        ValueType;
-
-    /// @deprecated Use GridSampler<TreeType, openvdb::tools::QaudraticSampler> instead.
-    OPENVDB_DEPRECATED explicit QuadraticInterp(const TreeType& tree): BaseType(tree) {}
-    virtual ~QuadraticInterp() {}
-
-    /// Triquadratically sample a grid at (x, y, z).
-    virtual ValueType sampleVoxel(Real x, Real y, Real z) const
-    {
-        ValueType result = zeroVal<ValueType>();
-        QuadraticSampler::sample(this->tree(), Vec3R(x, y, z), result);
-        return result;
-    }
-};
-
-
-////////////////////////////////////////
-
-
 namespace local_util {
 
 inline Vec3i
@@ -643,6 +549,6 @@ StaggeredQuadraticSampler::sample(const TreeT& inTree, const Vec3R& inCoord,
 
 #endif // OPENVDB_TOOLS_INTERPOLATION_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
