@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -44,7 +44,6 @@
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
 #include <boost/type_traits/is_floating_point.hpp>
-#include <openvdb/openvdb.h>
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h>
 #include <openvdb/math/FiniteDifference.h>
@@ -244,7 +243,7 @@ LevelSetTracker<GridT, InterruptT>::prune()
     mGrid.tree().pruneLevelSet();
 
     // The tree topology has changes so rebuild the list of leafs
-    mLeafs->rebuildLeafs();
+    mLeafs->rebuildLeafArray();
     this->endInterrupter();
 }
 
@@ -252,10 +251,8 @@ template<typename GridT, typename InterruptT>
 inline void
 LevelSetTracker<GridT, InterruptT>::track()
 {
-    // Dilate narrow-band
+    // Dilate narrow-band (this also rebuilds the leaf array!)
     tools::dilateVoxels(*mLeafs);
-    // The tree topology has changes so rebuild the list of leafs
-    mLeafs->rebuildLeafs();
 
     // Compute signed distances in dilated narrow-band
     this->normalize();
@@ -276,7 +273,7 @@ inline void
 LevelSetTracker<GridT, InterruptT>::endInterrupter()
 {
     if (mInterrupter) mInterrupter->end();
-}    
+}
 
 template<typename GridT,  typename InterruptT>
 inline bool
@@ -507,7 +504,6 @@ euler2(const RangeType& range, ValueType dt, ValueType alpha, Index phiBuffer, I
         }
     }
 }
-/// --------------------------------------------------------------
 
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
@@ -515,6 +511,6 @@ euler2(const RangeType& range, ValueType dt, ValueType alpha, Index phiBuffer, I
 
 #endif // OPENVDB_TOOLS_LEVEL_SET_TRACKER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
