@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -86,7 +86,9 @@ public:
     /// @param polygonList  List of triangles and/or quads.
     /// @param exBandWidth  The exterior narrow-band width in voxel units.
     /// @param inBandWidth  The interior narrow-band width in voxel units.
-    void convertToLevelSet(const std::vector<Vec3s>& pointList, const std::vector<Vec4I>& polygonList,
+    void convertToLevelSet(
+        const std::vector<Vec3s>& pointList,
+        const std::vector<Vec4I>& polygonList,
         DistValueT exBandWidth = DistValueT(LEVEL_SET_HALF_WIDTH),
         DistValueT inBandWidth = DistValueT(LEVEL_SET_HALF_WIDTH));
 
@@ -101,7 +103,6 @@ public:
     void convertToUnsignedDistanceField(const std::vector<Vec3s>& pointList,
         const std::vector<Vec4I>& polygonList, DistValueT exBandWidth);
 
-
     void clear();
 
     /// Returns a narrow-band (signed) distance field / level set grid.
@@ -111,23 +112,6 @@ public:
     /// voxel in the narrow-band.
     typename IndexGridT::Ptr indexGridPtr() const { return mIndexGrid; }
 
-
-    //////////
-
-    /// @brief DEPRICATED will be removed soon.
-    /// @note The new functions take the narrow band width in voxel units.
-    ///
-    /// @param pointList    List of points in grid index space, preferably unique
-    ///                     and shared by different polygons.
-    /// @param polygonList  List of triangles and/or quads.
-    /// @param exBandWidth  The exterior narrow-band width in world space units.
-    /// @param inBandWidth  The interior narrow-band width in world space units.
-    OPENVDB_DEPRECATED
-    void convert(const std::vector<Vec3s>& pointList, const std::vector<Vec4I>& polygonList,
-        DistValueT exBandWidth, DistValueT inBandWidth = DistValueT(0.0));
-
-    //////////
-
 private:
     // disallow copy by assignment
     void operator=(const MeshToVolume<DistGridT, InterruptT>&) {}
@@ -154,6 +138,15 @@ private:
 
 namespace internal {
 
+
+template<typename ValueType>
+struct Tolerance
+{
+    static ValueType epsilon() { return ValueType(1e-7); }
+    static ValueType minNarrowBandWidth() { return ValueType(1.0 + 1e-6); }
+};
+
+
 template<typename DistTreeT, typename IndexTreeT>
 inline void
 combine(DistTreeT& lhsDist, IndexTreeT& lhsIndex, DistTreeT& rhsDist, IndexTreeT& rhsIndex)
@@ -333,36 +326,36 @@ MeshVoxelizer<DistTreeT, InterruptT>::operator()(const tbb::blocked_range<size_t
 
         const Int32 primIdx = n;
         const Vec4I verts = (*mPolygonList)[n];
-        
+
         Vec3d p0((*mPointList)[verts[0]]);
         Vec3d p1((*mPointList)[verts[1]]);
         Vec3d p2((*mPointList)[verts[2]]);
 
         if (shortEdge(p0, p1, p2)) {
             coordList.clear();
-            
-            
+
+
             ijk = util::nearestCoord(p0);
             evalVoxel(ijk, primIdx);
             coordList.push_back(ijk);
-            
-            
+
+
             ijk = util::nearestCoord(p1);
             evalVoxel(ijk, primIdx);
             coordList.push_back(ijk);
 
-            
+
             ijk = util::nearestCoord(p2);
             evalVoxel(ijk, primIdx);
             coordList.push_back(ijk);
-            
+
             if (util::INVALID_IDX != verts[3]) {
                 Vec3d p3((*mPointList)[verts[3]]);
                 ijk = util::nearestCoord(p3);
                 evalVoxel(ijk, primIdx);
                 coordList.push_back(ijk);
             }
-            
+
             while (!coordList.empty()) {
                 if (mInterrupter && mInterrupter->wasInterrupted()) {
                     break;
@@ -382,12 +375,12 @@ MeshVoxelizer<DistTreeT, InterruptT>::operator()(const tbb::blocked_range<size_t
                     }
                 }
             }
-        
+
         } else {
 
             ijk = util::nearestCoord(p0);
             evalVoxel(ijk, primIdx);
-            
+
             mLastPrimAccessor.setValue(ijk, primIdx);
             auxAcc.setActiveState(ijk, true);
 
@@ -433,20 +426,25 @@ bool
 MeshVoxelizer<DistTreeT, InterruptT>::evalVoxel(const Coord& ijk, const Int32 polyIdx)
 {
     Vec3d voxelCenter(ijk[0], ijk[1], ijk[2]);
-    Vec4I verts = (*mPolygonList)[polyIdx];
-
-    // Grab the triangle's points
-    Vec3d p0((*mPointList)[verts[0]]);
-    Vec3d p1((*mPointList)[verts[1]]);
-    Vec3d p2((*mPointList)[verts[2]]);
-
-    double dist = math::triToPtnDistSqr(p0, p1, p2, voxelCenter);
+    const Vec4I& verts = (*mPolygonList)[polyIdx];
+    const std::vector<Vec3s>& points = *mPointList;
 
+    // Evaluate first triangle
+    const Vec3d a(points[verts[0]]);
+    const Vec3d b(points[verts[1]]);
+    const Vec3d c(points[verts[2]]);
+    
+    Vec3d uvw;
+    double dist = (voxelCenter -
+        closestPointOnTriangleToPoint(a, c, b, voxelCenter, uvw)).lengthSqr();
+    
     // Split-up quad into a second triangle and calac distance.
     if (util::INVALID_IDX != verts[3]) {
-        p1 = Vec3d((*mPointList)[verts[3]]);
+        const Vec3d d(points[verts[3]]);
 
-        double secondDist = math::triToPtnDistSqr(p0, p1, p2, voxelCenter);
+        double secondDist = (voxelCenter -
+            closestPointOnTriangleToPoint(a, d, c, voxelCenter, uvw)).lengthSqr();
+            
         if (secondDist < dist) dist = secondDist;
     }
 
@@ -684,8 +682,8 @@ ContourTracer<DistTreeT, InterruptT>::sparseScan(int slice) const
                         // backtrace
                         for (--ijk[2]; ijk[2] >= last_k; --ijk[2]) {
                             if (mIntersectionAccessor.isValueOn(ijk)) break;
-                            DistValueT& v = const_cast<DistValueT&>(mDistAccessor.getValue(ijk));
-                            if(v < DistValueT(0.0)) v = -v; // flip sign
+                            DistValueT& vb = const_cast<DistValueT&>(mDistAccessor.getValue(ijk));
+                            if(vb < DistValueT(0.0)) vb = -vb; // flip sign
                         }
 
                         last_k = tmp_k;
@@ -861,36 +859,32 @@ Vec3d
 IntersectingVoxelSign<DistTreeT>::getClosestPointDir(const Coord& ijk) const
 {
     Vec3d voxelCenter(ijk[0], ijk[1], ijk[2]);
-    Vec4I prim = (*mPolygonList)[mIndexAccessor.getValue(ijk)];
+    const Vec4I& prim = (*mPolygonList)[mIndexAccessor.getValue(ijk)];
+    const std::vector<Vec3s>& points = *mPointList;
 
-    // Grab the first triangle's points
-    Vec3d p0((*mPointList)[prim[0]]);
-    Vec3d p1((*mPointList)[prim[1]]);
-    Vec3d p2((*mPointList)[prim[2]]);
-
-    Vec2d uv;
-    double dist = math::sTri3ToPointDistSqr(p0, p1, p2, voxelCenter, uv);
-
-    // Check if quad.
+    // Evaluate first triangle
+    const Vec3d a(points[prim[0]]);
+    const Vec3d b(points[prim[1]]);
+    const Vec3d c(points[prim[2]]);
+    
+    Vec3d uvw;
+    Vec3d cpt = closestPointOnTriangleToPoint(a, c, b, voxelCenter, uvw);
+    Vec3d diff = voxelCenter - cpt;
+    
+    // Evaluate second triangle if quad.
     if (prim[3] != util::INVALID_IDX) {
-        Vec3d p3((*mPointList)[prim[3]]);
+        const Vec3d d(points[prim[3]]);
 
-        Vec2d uv2;
-        double dist2 = math::sTri3ToPointDistSqr(p0, p3, p2, voxelCenter, uv2);
-
-        if (dist2 < dist) {
-            p1 = p3;
-            uv = uv2;
+        cpt = closestPointOnTriangleToPoint(a, d, c, voxelCenter, uvw);
+        Vec3d tmpDiff = voxelCenter - cpt;    
+        
+        if (tmpDiff.lengthSqr() < diff.lengthSqr()) {
+            diff = tmpDiff;
         }
     }
 
-    Vec3d closestPoint = p0 * uv[0] +
-                         p1 * uv[1] +
-                         p2 * (1.0 - uv(0) - uv(1));
-
-    Vec3d dir = (voxelCenter-closestPoint);
-    dir.normalize();
-    return dir;
+    diff.normalize();
+    return diff;
 }
 
 
@@ -992,7 +986,7 @@ IntersectingVoxelCleaner<DistTreeT>::operator()(
 {
     Coord ijk, m_ijk;
     bool turnOff;
-    DistValueT value, bg = mDistTree.getBackground();
+    DistValueT value, bg = mDistTree.background();
 
     typename StencilTreeT::LeafNodeType::ValueOnCIter iter;
 
@@ -1126,8 +1120,8 @@ ShellVoxelCleaner<DistTreeT>::operator()(
     DistValueT value;
 
     const DistValueT distC = -0.86602540378443861;
-    const DistValueT distBG = mDistTree.getBackground();
-    const Int32 indexBG = mIntersectionTree.getBackground();
+    const DistValueT distBG = mDistTree.background();
+    const Int32 indexBG = mIntersectionTree.background();
 
     typename DistTreeT::LeafNodeType::ValueOnCIter iter;
     for (size_t n = range.begin(); n < range.end(); ++n) {
@@ -1307,15 +1301,15 @@ ExpandNB<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
                     continue;
                 }
 
-                DistValueT distance =
+                DistValueT distance = Tolerance<DistValueT>::epsilon() + 
                     getDist(iter.getCoord(), distAcc, indexAcc, closestPrimIndex);
 
-                bool inside = distLeaf.getValue(pos) < DistValueT(0.0);
+                const bool inside = distLeaf.getValue(pos) < DistValueT(0.0);
 
-                if (!inside && !(distance > mExteriorBandWidth)) {
+                if (!inside && distance < mExteriorBandWidth) {
                     distLeaf.setValueOn(pos, distance);
                     indexLeaf.setValueOn(pos, closestPrimIndex);
-                } else if (inside && !(distance > mInteriorBandWidth)) {
+                } else if (inside && distance < mInteriorBandWidth) {
                     distLeaf.setValueOn(pos, -distance);
                     indexLeaf.setValueOn(pos, closestPrimIndex);
                 } else {
@@ -1360,20 +1354,25 @@ double
 ExpandNB<DistTreeT>::getDistToPrim(const Coord& ijk, const Int32 polyIdx) const
 {
     Vec3d voxelCenter(ijk[0], ijk[1], ijk[2]);
-    Vec4I verts = (*mPolygonList)[polyIdx];
+    const Vec4I& verts = (*mPolygonList)[polyIdx];
+    const std::vector<Vec3s>& points = *mPointList;
 
-    // Grab the triangle's points
-    Vec3d p0((*mPointList)[verts[0]]);
-    Vec3d p1((*mPointList)[verts[1]]);
-    Vec3d p2((*mPointList)[verts[2]]);
-
-    double dist = math::triToPtnDistSqr(p0, p1, p2, voxelCenter);
+    // Evaluate first triangle
+    const Vec3d a(points[verts[0]]);
+    const Vec3d b(points[verts[1]]);
+    const Vec3d c(points[verts[2]]);
+    
+    Vec3d uvw;
+    double dist = (voxelCenter -
+        closestPointOnTriangleToPoint(a, c, b, voxelCenter, uvw)).lengthSqr();
 
     // Split-up quad into a second triangle and calac distance.
     if (util::INVALID_IDX != verts[3]) {
-        p1 = Vec3d((*mPointList)[verts[3]]);
+        const Vec3d d(points[verts[3]]);
+
+        double secondDist = (voxelCenter -
+            closestPointOnTriangleToPoint(a, d, c, voxelCenter, uvw)).lengthSqr();
 
-        double secondDist = math::triToPtnDistSqr(p0, p1, p2, voxelCenter);
         if (secondDist < dist) dist = secondDist;
     }
 
@@ -1559,11 +1558,13 @@ struct TrimOp
         typename LeafNodeType::ValueOnIter iter = leaf.beginValueOn();
 
         for (; iter; ++iter) {
-            ValueType val = iter.getValue();
-            if (val < -mInBandWidth) {
+            const ValueType& val = iter.getValue();
+            const bool inside = val < ValueType(0.0);
+
+            if (inside && !(val > -mInBandWidth)) {
                 iter.setValue(-mInBandWidth);
                 iter.setValueOff();
-            } else if (val > mExBandWidth) {
+            } else if (!inside && !(val < mInBandWidth)) {
                 iter.setValue(mExBandWidth);
                 iter.setValueOff();
             }
@@ -1736,8 +1737,9 @@ MeshToVolume<DistGridT, InterruptT>::convertToLevelSet(
     const std::vector<Vec3s>& pointList, const std::vector<Vec4I>& polygonList,
     DistValueT exBandWidth, DistValueT inBandWidth)
 {
-    exBandWidth = std::max(DistValueT(1.0), exBandWidth);
-    inBandWidth = std::max(DistValueT(1.0), inBandWidth);
+    // The narrow band width is exclusive, the shortest valid distance has to be > 1 voxel
+    exBandWidth = std::max(internal::Tolerance<DistValueT>::minNarrowBandWidth(), exBandWidth);
+    inBandWidth = std::max(internal::Tolerance<DistValueT>::minNarrowBandWidth(), inBandWidth);
     const DistValueT vs = mTransform->voxelSize()[0];
     doConvert(pointList, polygonList, vs * exBandWidth, vs * inBandWidth);
     mDistGrid->setGridClass(GRID_LEVEL_SET);
@@ -1750,22 +1752,13 @@ MeshToVolume<DistGridT, InterruptT>::convertToUnsignedDistanceField(
     const std::vector<Vec3s>& pointList, const std::vector<Vec4I>& polygonList,
     DistValueT exBandWidth)
 {
-    exBandWidth = std::max(DistValueT(1.0), exBandWidth);
+    // The narrow band width is exclusive, the shortest valid distance has to be > 1 voxel
+    exBandWidth = std::max(internal::Tolerance<DistValueT>::minNarrowBandWidth(), exBandWidth);
     const DistValueT vs = mTransform->voxelSize()[0];
     doConvert(pointList, polygonList, vs * exBandWidth, 0.0, true);
     mDistGrid->setGridClass(GRID_UNKNOWN);
 }
 
-template<typename DistGridT, typename InterruptT>
-inline void
-MeshToVolume<DistGridT, InterruptT>::convert(
-    const std::vector<Vec3s>& pointList, const std::vector<Vec4I>& polygonList,
-    DistValueT exBandWidth, DistValueT inBandWidth)
-{
-    doConvert(pointList, polygonList, exBandWidth, inBandWidth);
-    mDistGrid->setGridClass(GRID_LEVEL_SET);
-}
-
 template<typename DistGridT, typename InterruptT>
 void
 MeshToVolume<DistGridT, InterruptT>::doConvert(
@@ -1939,7 +1932,7 @@ MeshToVolume<DistGridT, InterruptT>::doConvert(
 
             if (mInterrupter && mInterrupter->wasInterrupted()) return;
 
-            offsetOp.resetOffset(offset);
+            offsetOp.resetOffset(offset - internal::Tolerance<DistValueT>::epsilon());
             offsetXform.runParallel();
         }
 
@@ -1949,7 +1942,8 @@ MeshToVolume<DistGridT, InterruptT>::doConvert(
     if (mInterrupter && mInterrupter->wasInterrupted()) return;
 
     // Narrow-band dilation
-    if (inBandWidth > voxelSize || exBandWidth > voxelSize) {
+    const DistValueT minWidth = voxelSize * 2.0;
+    if (inBandWidth > minWidth || exBandWidth > minWidth) {
 
         // Create the initial voxel mask.
         StencilTreeT maskTree(false);
@@ -1984,7 +1978,7 @@ MeshToVolume<DistGridT, InterruptT>::doConvert(
 
                 std::vector<DistLeafType*> newDistLeafs;
                 newDistLeafs.reserve(2 * distLeafs.size());
-                
+
                 for (size_t n = 0, N = distLeafs.size(); n < N; ++n) {
 
                     Coord ijk = distLeafs[n]->getOrigin();
@@ -2051,13 +2045,12 @@ MeshToVolume<DistGridT, InterruptT>::doConvert(
 
             expand.runParallel();
         }
-
     }
 
     if (!bool(GENERATE_PRIM_INDEX_GRID & mConversionFlags)) mIndexGrid->clear();
 
-
-    if ((inBandWidth < (voxelSize*3)) || (exBandWidth < (voxelSize*3))) {
+    const DistValueT minTrimWidth = voxelSize * 4.0;
+    if (inBandWidth < minTrimWidth || exBandWidth < minTrimWidth) {
 
         // If the narrow band was not expanded, we might need to trim off
         // some of the active voxels in order to respect the narrow band limits.
@@ -2067,15 +2060,14 @@ MeshToVolume<DistGridT, InterruptT>::doConvert(
 
         typedef internal::TrimOp<DistValueT> TrimOp;
 
-        TrimOp op(exBandWidth, inBandWidth);
+        TrimOp op(exBandWidth, unsignedDistField ? exBandWidth : inBandWidth);
         LeafTransformer<DistTreeT, TrimOp> transform(leafs, op);
         transform.runParallel();
     }
 
     if (mInterrupter && mInterrupter->wasInterrupted()) return;
 
-    tree::LevelSetPrune<DistValueT> prune;
-    mDistGrid->tree().pruneOp(prune);
+    mDistGrid->tree().pruneLevelSet();
 }
 
 } // namespace tools
@@ -2084,6 +2076,6 @@ MeshToVolume<DistGridT, InterruptT>::doConvert(
 
 #endif // OPENVDB_TOOLS_MESH_TO_VOLUME_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
