@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -81,41 +81,11 @@ namespace tools {
 /// @note This uniform scattering technique assumes that the number of
 /// points is generally smaller than the number of active voxels
 /// (including virtual active voxels in active tiles).
-template<
-    typename PointAccessorType,
-    typename RandomGenerator,
-    typename InterruptType = openvdb::util::NullInterrupter>
+template<typename PointAccessorType,
+         typename RandomGenerator,
+         typename InterruptType = openvdb::util::NullInterrupter>
 class UniformPointScatter
 {
-protected:
-    PointAccessorType&        mPoints;
-    InterruptType*            mInterrupter;
-    int                       mPointCount;
-    float                     mPointsPerVolume;
-    openvdb::Index64          mVoxelCount;
-    RandomGenerator&          mRandomGen;
-    boost::uniform_01<double> mRandom;
-
-    double getRand() { return mRandom(mRandomGen); }
-
-    template <typename GridT>
-    inline void addPoint(const GridT &grid, const openvdb::Vec3R &pos, const openvdb::Vec3R &delta)
-    {
-        mPoints.add(grid.indexToWorld(pos + delta));
-        ++mPointCount;
-    }
-    template <typename GridT>
-    inline void addPoint(const GridT &grid, const openvdb::Vec3R &dmin)
-    {
-        this->addPoint(grid, dmin, openvdb::Vec3R(getRand(),getRand(),getRand()));
-    }
-    template <typename GridT>
-    inline void addPoint(const GridT &grid, const openvdb::Vec3R &dmin, const openvdb::Coord &size)
-    {
-        const openvdb::Vec3R d(size.x()*getRand(),size.y()*getRand(),size.z()*getRand());
-        this->addPoint(grid, dmin, d);
-    }
-
 public:
     UniformPointScatter(PointAccessorType& points,
                         int pointCount,
@@ -147,7 +117,7 @@ public:
     void operator()(const GridT& grid)
     {
         mVoxelCount = grid.activeVoxelCount();
-        if (mVoxelCount == 0) throw std::runtime_error("No voxels in which to scatter points!");
+        if (mVoxelCount == 0) return;//throw std::runtime_error("No voxels in which to scatter points!");
         const openvdb::Index64 voxelId = mVoxelCount - 1;
         const openvdb::Vec3d dim = grid.voxelSize();
         if (mPointsPerVolume>0) {
@@ -157,15 +127,16 @@ public:
             if (mInterrupter) mInterrupter->start("Uniform scattering with fixed point count");
             mPointsPerVolume = mPointCount/float(dim[0]*dim[1]*dim[2] * mVoxelCount);
         } else {
-            throw std::runtime_error("Invalid point count and point density");
+            return;
+            //throw std::runtime_error("Invalid point count and point density");
         }
         openvdb::CoordBBox bbox;
         /// build sorted multi-map of random voxel-ids to contain a point
         std::multiset<openvdb::Index64> mVoxelSet;
         for (int i=0, chunks=100000; i<mPointCount; i += chunks) {
-            if (mInterrupter && mInterrupter->wasInterrupted()) {
-                throw std::runtime_error("processing was interrupted");
-            }
+            if (util::wasInterrupted(mInterrupter)) return;
+            //    throw std::runtime_error("processing was interrupted");
+            //}
             /// @todo Multi-thread the generation of mVoxelSet
             for (int j=i, end=std::min(i+chunks, mPointCount); j<end; ++j) {
                 mVoxelSet.insert(openvdb::Index64(voxelId*getRand()));
@@ -175,10 +146,10 @@ public:
             mVoxelSet.begin(), voxelEnd = mVoxelSet.end();
         typename GridT::ValueOnCIter valueIter = grid.cbeginValueOn();
         mPointCount = 0;
+        size_t interruptCount = 0;
         for (openvdb::Index64 i=valueIter.getVoxelCount(); voxelIter != voxelEnd; ++voxelIter) {
-            if (mInterrupter && mInterrupter->wasInterrupted()) {
-                throw std::runtime_error("processing was interrupted");
-            }
+            //only check interrupter for every 32'th particle
+            if (!(interruptCount++ & (1<<5)-1) && util::wasInterrupted(mInterrupter)) return;
             while ( i <= *voxelIter ) {
                 ++valueIter;
                 i += valueIter.getVoxelCount();
@@ -211,23 +182,8 @@ public:
     int getPointCount() const { return mPointCount; }
     float getPointsPerVolume() const { return mPointsPerVolume; }
     openvdb::Index64 getVoxelCount() const { return mVoxelCount; }
-}; // class UniformPointScatter
 
-
-/// @brief Non-uniform scatters of point in the active voxels.
-/// The local point count is implicitly defined as a product of
-/// of a global density and the local voxel (or tile) value.
-///
-/// @note This scattering technique can be significantly slower
-/// than a uniform scattering since its computational complexity
-/// is proportional to the active voxel (and tile) count.
-template<
-    typename PointAccessorType,
-    typename RandomGenerator,
-    typename InterruptType = openvdb::util::NullInterrupter>
-class NonUniformPointScatter
-{
-protected:
+private:
     PointAccessorType&        mPoints;
     InterruptType*            mInterrupter;
     int                       mPointCount;
@@ -255,7 +211,21 @@ protected:
         const openvdb::Vec3R d(size.x()*getRand(),size.y()*getRand(),size.z()*getRand());
         this->addPoint(grid, dmin, d);
     }
+}; // class UniformPointScatter
 
+
+/// @brief Non-uniform scatters of point in the active voxels.
+/// The local point count is implicitly defined as a product of
+/// of a global density and the local voxel (or tile) value.
+///
+/// @note This scattering technique can be significantly slower
+/// than a uniform scattering since its computational complexity
+/// is proportional to the active voxel (and tile) count.
+template<typename PointAccessorType,
+         typename RandomGenerator,
+         typename InterruptType = openvdb::util::NullInterrupter>
+class NonUniformPointScatter
+{
 public:
     NonUniformPointScatter(PointAccessorType& points,
                            float pointsPerVolume,
@@ -281,10 +251,10 @@ public:
         const double volumePerVoxel = dim[0]*dim[1]*dim[2],
                      pointsPerVoxel = mPointsPerVolume * volumePerVoxel;
         openvdb::CoordBBox bbox;
+        size_t interruptCount = 0;
         for (typename GridT::ValueOnCIter iter = grid.cbeginValueOn(); iter; ++iter) {
-            if (mInterrupter && mInterrupter->wasInterrupted()) {
-                throw std::runtime_error("processing was interrupted");
-            }
+            //only check interrupter for every 32'th active value
+            if (!(interruptCount++ & (1<<5)-1) && util::wasInterrupted(mInterrupter)) return;
             const double d = (*iter) * pointsPerVoxel * iter.getVoxelCount();
             const int n = int(d);
             if (iter.isVoxelValue()) { // a majorty is expected to be voxels
@@ -301,7 +271,7 @@ public:
                 for (int i = 0; i < n; ++i) this->addPoint(grid, dmin, size);
                 if (getRand() < (d - n)) this->addPoint(grid, dmin, size);
             }
-        }
+        }//loop over the active values
         if (mInterrupter) mInterrupter->end();
     }
 
@@ -315,6 +285,36 @@ public:
 
     int   getPointCount() const { return mPointCount; }
     openvdb::Index64  getVoxelCount() const { return mVoxelCount; }
+    
+private:
+    PointAccessorType&        mPoints;
+    InterruptType*            mInterrupter;
+    int                       mPointCount;
+    float                     mPointsPerVolume;
+    openvdb::Index64          mVoxelCount;
+    RandomGenerator&          mRandomGen;
+    boost::uniform_01<double> mRandom;
+
+    double getRand() { return mRandom(mRandomGen); }
+
+    template <typename GridT>
+    inline void addPoint(const GridT &grid, const openvdb::Vec3R &pos, const openvdb::Vec3R &delta)
+    {
+        mPoints.add(grid.indexToWorld(pos + delta));
+        ++mPointCount;
+    }
+    template <typename GridT>
+    inline void addPoint(const GridT &grid, const openvdb::Vec3R &dmin)
+    {
+        this->addPoint(grid, dmin, openvdb::Vec3R(getRand(),getRand(),getRand()));
+    }
+    template <typename GridT>
+    inline void addPoint(const GridT &grid, const openvdb::Vec3R &dmin, const openvdb::Coord &size)
+    {
+        const openvdb::Vec3R d(size.x()*getRand(),size.y()*getRand(),size.z()*getRand());
+        this->addPoint(grid, dmin, d);
+    }
+    
 }; // class NonUniformPointScatter
 
 } // namespace tools
@@ -323,6 +323,6 @@ public:
 
 #endif // OPENVDB_TOOLS_POINT_SCATTER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
