@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -141,8 +141,6 @@ inline void foreach(const IterT& iter, const XformOp& op,
 /// tools::transformValues(inGrid.cbeginValueOn(), outGrid, Local::op);
 /// @endcode
 ///
-/// @warning The @a shareOp argument is currently ignored.  The functor is always shared.
-///
 /// @note For more complex operations that require finer control over threading,
 /// consider using @c tbb::parallel_for() or @c tbb::parallel_reduce() in conjunction
 /// with a tree::IteratorRange that wraps a grid or tree iterator.
@@ -162,7 +160,6 @@ inline void transformValues(const InIterT& inIter, OutGridT& outGrid,
 
 namespace valxform {
 
-
 template<typename IterT, typename OpT>
 class SharedOpApplier
 {
@@ -217,7 +214,7 @@ public:
 private:
     IterT mIter;
     OpT mOp; // copy of original functor
-    const OpT* mOrigOp; // pointer to original functor
+    OpT const * const mOrigOp; // pointer to original functor
 };
 
 } // namespace valxform
@@ -330,7 +327,92 @@ private:
     const InTreeT* mInputTree;
     OutTreeT* mOutputTree;
     OpT& mOp;
-};
+}; // class SharedOpTransformer
+
+
+template<typename InIterT, typename OutTreeT, typename OpT>
+class CopyableOpTransformer
+{
+public:
+    typedef typename InIterT::TreeT InTreeT;
+    typedef typename tree::IteratorRange<InIterT> IterRange;
+    typedef typename OutTreeT::ValueType OutValueT;
+
+    CopyableOpTransformer(const InIterT& inIter, OutTreeT& outTree, const OpT& op):
+        mIsRoot(true),
+        mInputIter(inIter),
+        mInputTree(inIter.getTree()),
+        mOutputTree(&outTree),
+        mOp(op),
+        mOrigOp(&op)
+    {
+        if (static_cast<const void*>(mInputTree) == static_cast<void*>(mOutputTree)) {
+            OPENVDB_LOG_INFO("use tools::foreach(), not transformValues(),"
+                " to transform a grid in place");
+        }
+    }
+
+    // When splitting this task, give the subtask a copy of the original functor,
+    // not of this task's functor, which might have been modified arbitrarily.
+    CopyableOpTransformer(CopyableOpTransformer& other, tbb::split):
+        mIsRoot(false),
+        mInputIter(other.mInputIter),
+        mInputTree(other.mInputTree),
+        mOutputTree(new OutTreeT(zeroVal<OutValueT>())),
+        mOp(*other.mOrigOp),
+        mOrigOp(other.mOrigOp)
+        {}
+
+    ~CopyableOpTransformer()
+    {
+        // Delete the output tree only if it was allocated locally
+        // (the top-level output tree was supplied by the caller).
+        if (!mIsRoot) {
+            delete mOutputTree;
+            mOutputTree = NULL;
+        }
+    }
+
+    void process(bool threaded = true)
+    {
+        if (!mInputTree || !mOutputTree) return;
+
+        IterRange range(mInputIter);
+
+        // Independently transform elements in the iterator range,
+        // either in parallel or serially.
+        if (threaded) {
+            tbb::parallel_reduce(range, *this);
+        } else {
+            (*this)(range);
+        }
+    }
+
+    /// Transform each element in the given range.
+    void operator()(IterRange& range)
+    {
+        if (!mOutputTree) return;
+        typename tree::ValueAccessor<OutTreeT> outAccessor(*mOutputTree);
+        for ( ; range; ++range) {
+            mOp(range.iterator(), outAccessor);
+        }
+    }
+
+    void join(const CopyableOpTransformer& other)
+    {
+        if (mOutputTree && other.mOutputTree) {
+            mOutputTree->merge(*other.mOutputTree);
+        }
+    }
+
+private:
+    bool mIsRoot;
+    InIterT mInputIter;
+    const InTreeT* mInputTree;
+    OutTreeT* mOutputTree;
+    OpT mOp; // copy of original functor
+    OpT const * const mOrigOp; // pointer to original functor
+}; // class CopyableOpTransformer
 
 } // namespace valxform
 
@@ -341,25 +423,30 @@ private:
 template<typename InIterT, typename OutGridT, typename XformOp>
 inline void
 transformValues(const InIterT& inIter, OutGridT& outGrid, XformOp& op,
-    bool threaded, bool /*shareOp*/)
+    bool threaded, bool shared)
 {
-    /// @todo Implement shareOp = false functionality.
-
     typedef TreeAdapter<OutGridT> Adapter;
     typedef typename Adapter::TreeType OutTreeT;
-    typedef typename valxform::SharedOpTransformer<InIterT, OutTreeT, XformOp> Processor;
-    Processor proc(inIter, Adapter::tree(outGrid), op);
-    proc.process(threaded);
+    if (shared) {
+        typedef typename valxform::SharedOpTransformer<InIterT, OutTreeT, XformOp> Processor;
+        Processor proc(inIter, Adapter::tree(outGrid), op);
+        proc.process(threaded);
+    } else {
+        typedef typename valxform::CopyableOpTransformer<InIterT, OutTreeT, XformOp> Processor;
+        Processor proc(inIter, Adapter::tree(outGrid), op);
+        proc.process(threaded);
+    }
 }
 
 #ifndef _MSC_VER
 template<typename InIterT, typename OutGridT, typename XformOp>
 inline void
 transformValues(const InIterT& inIter, OutGridT& outGrid, const XformOp& op,
-    bool threaded, bool /*shareOp*/)
+    bool threaded, bool /*share*/)
 {
     typedef TreeAdapter<OutGridT> Adapter;
     typedef typename Adapter::TreeType OutTreeT;
+    // Const ops are shared across threads, not copied.
     typedef typename valxform::SharedOpTransformer<InIterT, OutTreeT, const XformOp> Processor;
     Processor proc(inIter, Adapter::tree(outGrid), op);
     proc.process(threaded);
@@ -372,6 +459,6 @@ transformValues(const InIterT& inIter, OutGridT& outGrid, const XformOp& op,
 
 #endif // OPENVDB_TOOLS_VALUETRANSFORMER_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
