@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,7 +31,6 @@
 #ifndef OPENVDB_TOOLS_VOLUME_TO_MESH_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_VOLUME_TO_MESH_HAS_BEEN_INCLUDED
 
-#include <openvdb/openvdb.h>
 #include <openvdb/tree/ValueAccessor.h>
 #include <openvdb/tools/ValueTransformer.h>
 #include <openvdb/util/Util.h>
@@ -49,7 +48,6 @@
 
 #include <boost/scoped_ptr.hpp>
 
-
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
@@ -187,6 +185,10 @@ public:
     ///                         reference surface.)
     void setRefGrid(const GridBase::ConstPtr& grid, double secAdaptivity = 0);
 
+    // Deprecated in versions 1.1
+    template<typename GridT>
+    OPENVDB_DEPRECATED void accumulateAuxiliaryData(const GridT&) { return; }
+
 private:
 
     PointList mPoints;
@@ -213,7 +215,6 @@ namespace internal {
 // Bit-flags
 enum { INSIDE = 0x1, XEDGE = 0x2, YEDGE = 0x4, ZEDGE = 0x8 };
 
-
 const bool sAmbiguous[256] =
    {0,0,0,0,0,1,0,0,
     0,0,1,0,0,0,0,0,
@@ -248,29 +249,6 @@ const bool sAmbiguous[256] =
     0,0,0,0,0,1,0,0,
     0,0,1,0,0,0,0,0};
 
-// Thomas Wang's 32-bit integer hashing method, uses 7 shifts.
-inline Uint hash(Uint a)
-{
-    a -= (a<<6);
-    a ^= (a>>17);
-    a -= (a<<9);
-    a ^= (a<<4);
-    a -= (a<<3);
-    a ^= (a<<10);
-    a ^= (a>>15);
-    return a;
-}
-
-// Integer hash based pseudo random numbers in the [0, 1] range.
-inline double twist(Uint x)
-{
-    return double(hash(x)) * 2.3283064e-10;
-}
-
-inline double twist(Uint x, Uint y, Uint z)
-{
-    return  twist(hash(x) + hash(y) * 101 + hash(z) * 53);
-}
 
 template<class AccessorT>
 inline bool isAmbiguous(const AccessorT& accessor, const Coord& ijk,
@@ -474,7 +452,7 @@ inline bool
 isMergable(LeafType& leaf, const Coord& start, int dim,
     typename LeafType::ValueType::value_type adaptivity)
 {
-    if (adaptivity < 1e-5) return false;
+    if (adaptivity < 1e-6) return false;
 
     typedef typename LeafType::ValueType VecT;
     Coord ijk, end = start;
@@ -908,19 +886,17 @@ Merge<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
            }
        }
 
-       while(tmpLeaf.onVoxelCount() > 0) {
+       while (tmpLeaf.onVoxelCount() > 0) {
            ++numVoxels;
            IntIterT it = tmpLeaf.beginValueOn();
-           const int regionId = it.getValue();
+           regionId = it.getValue();
            for (; it; ++it) {
-               if(it.getValue() == regionId) it.setValueOff();
+               if (it.getValue() == regionId) it.setValueOff();
            }
        }
 
-        mLeafRegionCount[n] = numVoxels;
-
+       mLeafRegionCount[n] = numVoxels;
     }
-
 }
 
 
@@ -963,7 +939,6 @@ private:
 
     double root(double v0, double v1) const { return (mIsovalue - v0) / (v1 - v0); }
     int calcAvgPoint(DistTreeAccessorT&, const Coord&, openvdb::Vec3d&) const;
-    int calcEdgeSigns(DistTreeAccessorT&, const Coord&) const;
 };
 
 
@@ -1028,19 +1003,23 @@ void
 PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
 {
     typedef typename DistTreeT::template ValueConverter<bool>::Type BoolTreeT;
+
     typedef tree::ValueAccessor<const BoolTreeT> BoolTreeAccessorT;
+
     typedef typename BoolTreeT::LeafNodeType BoolLeafT;
     typedef typename IntTreeT::LeafNodeType IntLeafT;
 
 
     boost::scoped_ptr<DistTreeAccessorT> refDistAcc;
     boost::scoped_ptr<BoolTreeAccessorT> refMaskAcc;
+
     if (mRefData && mRefData->isValid()) {
         refDistAcc.reset(new DistTreeAccessorT(*mRefData->mDistTree));
         refMaskAcc.reset(new BoolTreeAccessorT(*mRefData->mTopologyMaskTree));
     }
-    const bool hasRefData = refDistAcc && refMaskAcc;
 
+
+    const bool hasRefData = refDistAcc && refMaskAcc;
     typename IntTreeT::LeafNodeType::ValueOnIter auxIter;
     DistTreeAccessorT distAcc(mDistTree);
 
@@ -1066,21 +1045,7 @@ PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
                 ijk = auxIter.getCoord();
 
                 if (hasRefData && maskLeaf && maskLeaf->isValueOn(ijk)) {
-
                     calcAvgPoint(*refDistAcc.get(), ijk, avg);
-
-                    const int signChangeFlags = calcAvgPoint(*refDistAcc.get(), ijk, avg);
-
-                    if (signChangeFlags != (XEDGE | YEDGE | ZEDGE)) {
-                        const int flags = ~signChangeFlags & calcEdgeSigns(distAcc, ijk);
-                        if (flags != 0) {
-                            const double r = twist(ijk[0], ijk[1], ijk[2]);
-                            if (flags & XEDGE) avg[0] = r;
-                            if (flags & YEDGE) avg[1] = r;
-                            if (flags & ZEDGE) avg[2] = r;
-                        }
-                    }
-
                 } else {
                     calcAvgPoint(distAcc, ijk, avg);
                 }
@@ -1149,93 +1114,6 @@ PointGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) const
     }
 }
 
-
-template <class DistTreeT>
-int
-PointGen<DistTreeT>::calcEdgeSigns(DistTreeAccessorT& acc, const Coord& ijk) const
-{
-    bool signMask[8];
-    Coord coord;
-
-    // Sample corner values
-    coord = ijk;
-    signMask[0] = double(acc.getValue(coord)) < mIsovalue; // i, j, k
-
-    coord[0] += 1;
-    signMask[1] = double(acc.getValue(coord)) < mIsovalue; // i+1, j, k
-
-    coord[2] += 1;
-    signMask[2] = double(acc.getValue(coord)) < mIsovalue; // i+i, j, k+1
-
-    coord[0] = ijk[0];
-    signMask[3] = double(acc.getValue(coord)) < mIsovalue; // i, j, k+1
-
-    coord[1] += 1; coord[2] = ijk[2];
-    signMask[4] = double(acc.getValue(coord)) < mIsovalue; // i, j+1, k
-
-    coord[0] += 1;
-    signMask[5] = double(acc.getValue(coord)) < mIsovalue; // i+1, j+1, k
-
-    coord[2] += 1;
-    signMask[6] = double(acc.getValue(coord)) < mIsovalue; // i+1, j+1, k+1
-
-    coord[0] = ijk[0];
-    signMask[7] = double(acc.getValue(coord)) < mIsovalue; // i, j+1, k+1
-
-    int edgeFlags = 0;
-
-    if (signMask[0] != signMask[1]) { // Edged: 0 - 1
-        edgeFlags |= XEDGE;
-    }
-
-    if (signMask[1] != signMask[2]) { // Edged: 1 - 2
-        edgeFlags |= ZEDGE;
-    }
-
-    if (signMask[3] != signMask[2]) { // Edged: 3 - 2
-        edgeFlags |= XEDGE;
-    }
-
-    if (signMask[0] != signMask[3]) { // Edged: 0 - 3
-        edgeFlags |= ZEDGE;
-    }
-
-    if (signMask[4] != signMask[5]) { // Edged: 4 - 5
-        edgeFlags |= XEDGE;
-    }
-
-    if (signMask[5] != signMask[6]) { // Edged: 5 - 6
-        edgeFlags |= ZEDGE;
-    }
-
-    if (signMask[7] != signMask[6]) { // Edged: 7 - 6
-        edgeFlags |= XEDGE;
-    }
-
-    if (signMask[4] != signMask[7]) { // Edged: 4 - 7
-        edgeFlags |= ZEDGE;
-    }
-
-    if (signMask[0] != signMask[4]) { // Edged: 0 - 4
-        edgeFlags |= YEDGE;
-    }
-
-    if (signMask[1] != signMask[5]) { // Edged: 1 - 5
-        edgeFlags |= YEDGE;
-    }
-
-    if (signMask[2] != signMask[6]) { // Edged: 2 - 6
-        edgeFlags |= YEDGE;
-    }
-
-    if (signMask[3] != signMask[7]) { // Edged: 3 - 7
-        edgeFlags |= YEDGE;
-    }
-
-    return edgeFlags;
-}
-
-
 template <class DistTreeT>
 int
 PointGen<DistTreeT>::calcAvgPoint(DistTreeAccessorT& acc,
@@ -1659,6 +1537,8 @@ MeshGen<DistTreeT, MeshingOp>::operator()(
             ijk = iter.getCoord();
             const char& edgeFlags = iter.getValue();
 
+            const bool isInside = edgeFlags & INSIDE;
+
             refEdgeFlags = 0;
             isSemLinePoly = 0;
             if (hasRefData) {
@@ -1668,7 +1548,7 @@ MeshGen<DistTreeT, MeshingOp>::operator()(
                 }
             }
 
-            const bool isInside = edgeFlags & INSIDE;
+
             int v0 = auxAcc.getValue(ijk);
 
             if (edgeFlags & XEDGE) {
@@ -2030,8 +1910,8 @@ SeamMaskGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) cons
 {
     typedef typename BoolTreeT::LeafNodeType::ValueOnIter ValueOnIterT;
     Coord ijk, n_ijk;
-    for (size_t n = range.begin(); n != range.end(); ++n) {
-        ValueOnIterT it = mSeamMaskLeafs[n]->beginValueOn();
+    for (size_t leafIdx = range.begin(); leafIdx != range.end(); ++leafIdx) {
+        ValueOnIterT it = mSeamMaskLeafs[leafIdx]->beginValueOn();
         for (; it; ++it) {
             ijk = it.getCoord();
             if (!mTopologyMaskAcc.isValueOn(ijk)) {
@@ -2052,6 +1932,243 @@ SeamMaskGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) cons
 }
 
 
+////////////////////////////////////////
+
+
+template<class CharAccessorT, typename AuxAccessorT>
+class AuxDataGenerator
+{
+public:
+    AuxDataGenerator(CharAccessorT& edgeAcc, AuxAccessorT& auxAcc)
+        : mEdgeAcc(edgeAcc), mAuxAcc(auxAcc) {}
+
+
+    void setXEdge(char edgeFlags, const Coord& ijk)
+    {
+        mEdgeAcc.setValue(ijk, edgeFlags | XEDGE);
+
+        mAuxAcc.setActiveState(ijk, true);
+
+        Coord coord = ijk;
+        coord[1] = ijk[1]-1;
+        mAuxAcc.setActiveState(coord, true);
+
+        coord[0] = ijk[0]; coord[1] = ijk[1]-1; coord[2] = ijk[2]-1;
+        mAuxAcc.setActiveState(coord, true);
+
+        coord[0] = ijk[0]; coord[1] = ijk[1]; coord[2] = ijk[2]-1;
+        mAuxAcc.setActiveState(coord, true);
+    }
+
+    void setYEdge(char edgeFlags, const Coord& ijk)
+    {
+        mEdgeAcc.setValue(ijk, edgeFlags | YEDGE);
+
+        mAuxAcc.setActiveState(ijk, true);
+
+        Coord coord = ijk;
+        coord[2] = ijk[2]-1;
+        mAuxAcc.setActiveState(coord, true);
+
+        coord[0] = ijk[0]-1; coord[1] = ijk[1]; coord[2] = ijk[2];
+        mAuxAcc.setActiveState(coord, true);
+
+        coord[0] = ijk[0]-1; coord[1] = ijk[1]; coord[2] = ijk[2]-1;
+        mAuxAcc.setActiveState(coord, true);
+    }
+
+    void setZEdge(char edgeFlags, const Coord& ijk)
+    {
+        mEdgeAcc.setValue(ijk, edgeFlags | ZEDGE);
+
+        mAuxAcc.setActiveState(ijk, true);
+
+        Coord coord = ijk;
+        coord[1] = ijk[1]-1;
+        mAuxAcc.setActiveState(coord, true);
+
+        coord[0] = ijk[0]-1; coord[1] = ijk[1]; coord[2] = ijk[2];
+        mAuxAcc.setActiveState(coord, true);
+
+        coord[0] = ijk[0]-1; coord[1] = ijk[1]-1; coord[2] = ijk[2];
+        mAuxAcc.setActiveState(coord, true);
+    }
+
+private:
+    CharAccessorT& mEdgeAcc;
+    AuxAccessorT& mAuxAcc;
+};
+
+
+////////////////////////////////////////
+
+
+template<class DistTreeT, class AuxTreeT, class CharTreeT>
+inline void
+tileAuxiliaryData(
+    const DistTreeT& distTree, CharTreeT& edgeTree, AuxTreeT& auxTree,
+    double iso)
+{
+    typedef tree::ValueAccessor<const DistTreeT> DistAccessorT;
+    typedef tree::ValueAccessor<AuxTreeT> AuxTreeAccessorT;
+    typedef tree::ValueAccessor<CharTreeT> CharTreeAccessorT;
+
+    typename DistTreeT::ValueType isoValue = typename DistTreeT::ValueType(iso);
+
+    DistAccessorT distAcc(distTree);
+    CharTreeAccessorT edgeAcc(edgeTree);
+    AuxTreeAccessorT auxAcc(auxTree);
+
+    AuxDataGenerator<CharTreeAccessorT, AuxTreeAccessorT> auxData(edgeAcc, auxAcc);
+
+    Coord ijk, nijk;
+    typename DistTreeT::ValueType value;
+    CoordBBox bbox;
+    bool processTileFace;
+
+    typename DistTreeT::ValueOnCIter tileIter(distTree);
+    tileIter.setMaxDepth(DistTreeT::ValueOnCIter::LEAF_DEPTH - 1);
+
+    for ( ; tileIter; ++tileIter) {
+        tileIter.getBoundingBox(bbox);
+
+        const bool thisInside = (distAcc.getValue(bbox.min()) < isoValue);
+        const int thisDepth = distAcc.getValueDepth(bbox.min());
+
+
+        // Eval x-edges
+
+        ijk = bbox.max();
+        nijk = ijk;
+        ++nijk[0];
+
+        processTileFace = true;
+        if (thisDepth >= distAcc.getValueDepth(nijk)) {
+            processTileFace = thisInside != (distAcc.getValue(nijk) < isoValue);
+        }
+
+        if (processTileFace) {
+            for (ijk[1] = bbox.min()[1]; ijk[1] <= bbox.max()[1]; ++ijk[1]) {
+                nijk[1] = ijk[1];
+                for (ijk[2] = bbox.min()[2]; ijk[2] <= bbox.max()[2]; ++ijk[2]) {
+                    nijk[2] = ijk[2];
+                    if ((distAcc.getValue(nijk) < isoValue) != thisInside) {
+                        auxData.setXEdge(edgeAcc.getValue(ijk) | char(thisInside), ijk);
+                    }
+                }
+            }
+        }
+
+        ijk = bbox.min();
+        --ijk[0];
+
+        processTileFace = true;
+        if (thisDepth >= distAcc.getValueDepth(ijk)) {
+            processTileFace = !distAcc.probeValue(ijk, value) && thisInside != (value < isoValue);
+        }
+
+        if (processTileFace) {
+            for (ijk[1] = bbox.min()[1]; ijk[1] <= bbox.max()[1]; ++ijk[1]) {
+                for (ijk[2] = bbox.min()[2]; ijk[2] <= bbox.max()[2]; ++ijk[2]) {
+                    if (!distAcc.probeValue(ijk, value) && (value < isoValue) != thisInside) {
+                         auxData.setXEdge(edgeAcc.getValue(ijk) | char(!thisInside), ijk);
+                    }
+                }
+            }
+        }
+
+
+        // Eval y-edges
+
+        ijk = bbox.max();
+        nijk = ijk;
+        ++nijk[1];
+
+        processTileFace = true;
+        if (thisDepth >= distAcc.getValueDepth(nijk)) {
+            processTileFace = thisInside != (distAcc.getValue(nijk) < isoValue);
+        }
+
+        if (processTileFace) {
+            for (ijk[0] = bbox.min()[0]; ijk[0] <= bbox.max()[0]; ++ijk[0]) {
+                nijk[0] = ijk[0];
+                for (ijk[2] = bbox.min()[2]; ijk[2] <= bbox.max()[2]; ++ijk[2]) {
+                    nijk[2] = ijk[2];
+
+                    if ((distAcc.getValue(nijk) < isoValue) != thisInside) {
+                        auxData.setYEdge(edgeAcc.getValue(ijk) | char(thisInside), ijk);
+                    }
+                }
+            }
+        }
+
+
+        ijk = bbox.min();
+        --ijk[1];
+
+        processTileFace = true;
+        if (thisDepth >= distAcc.getValueDepth(ijk)) {
+            processTileFace = !distAcc.probeValue(ijk, value) && thisInside != (value < isoValue);
+        }
+
+        if (processTileFace) {
+            for (ijk[0] = bbox.min()[0]; ijk[0] <= bbox.max()[0]; ++ijk[0]) {
+                for (ijk[2] = bbox.min()[2]; ijk[2] <= bbox.max()[2]; ++ijk[2]) {
+
+                    if (!distAcc.probeValue(ijk, value) && (value < isoValue) != thisInside) {
+                        auxData.setYEdge(edgeAcc.getValue(ijk) | char(!thisInside), ijk);
+                    }
+                }
+            }
+        }
+
+
+        // Eval z-edges
+
+        ijk = bbox.max();
+        nijk = ijk;
+        ++nijk[2];
+
+        processTileFace = true;
+        if (thisDepth >= distAcc.getValueDepth(nijk)) {
+            processTileFace = thisInside != (distAcc.getValue(nijk) < isoValue);
+        }
+
+        if (processTileFace) {
+            for (ijk[0] = bbox.min()[0]; ijk[0] <= bbox.max()[0]; ++ijk[0]) {
+                nijk[0] = ijk[0];
+                for (ijk[1] = bbox.min()[1]; ijk[1] <= bbox.max()[1]; ++ijk[1]) {
+                    nijk[1] = ijk[1];
+
+                    if ((distAcc.getValue(nijk) < isoValue) != thisInside) {
+                        auxData.setZEdge(edgeAcc.getValue(ijk) | char(thisInside), ijk);
+                    }
+                }
+            }
+        }
+
+        ijk = bbox.min();
+        --ijk[2];
+
+        processTileFace = true;
+        if (thisDepth >= distAcc.getValueDepth(ijk)) {
+            processTileFace = !distAcc.probeValue(ijk, value) && thisInside != (value < isoValue);
+        }
+
+        if (processTileFace) {
+            for (ijk[0] = bbox.min()[0]; ijk[0] <= bbox.max()[0]; ++ijk[0]) {
+                for (ijk[1] = bbox.min()[1]; ijk[1] <= bbox.max()[1]; ++ijk[1]) {
+
+                    if (!distAcc.probeValue(ijk, value) && (value < isoValue) != thisInside) {
+                        auxData.setZEdge(edgeAcc.getValue(ijk) | char(!thisInside), ijk);
+                    }
+                }
+            }
+        }
+    }
+}
+
+
 } // namespace internal
 
 
@@ -2061,7 +2178,7 @@ SeamMaskGen<DistTreeT>::operator()(const tbb::blocked_range<size_t>& range) cons
 inline VolumeToMesh::VolumeToMesh(double isovalue, double adaptivity)
     : mPointListSize(0)
     , mPolygonPoolListSize(0)
-    , mIsovalue(1e-4 + isovalue)
+    , mIsovalue(isovalue)
     , mPrimAdaptivity(adaptivity)
     , mSecAdaptivity(0.0)
     , mRefGrid(GridBase::ConstPtr())
@@ -2133,16 +2250,24 @@ VolumeToMesh::operator()(const GridT& distGrid)
     typename CharTreeT::Ptr edgeTree; // edge flags
     typename IntTreeT::Ptr auxTree; // auxiliary data
 
-    const bool extraSignCheck = std::abs(mIsovalue - double(distGrid.background()))
-        < 2.0 * double(transform.voxelSize()[0]);
+    const bool nonLevelSetGrid = distGrid.getGridClass() != GRID_LEVEL_SET;
+
+    const bool extraSignCheck = nonLevelSetGrid ||
+        (std::abs(mIsovalue - double(distGrid.background())) < (1.5 * transform.voxelSize()[0]));
 
     // Collect auxiliary data
     {
+
         internal::LeafCPtrList<DistTreeT> sourceLeafs(distTree);
         internal::AuxiliaryData<DistTreeT> op(distTree, sourceLeafs, mIsovalue, extraSignCheck);
         op.runParallel();
         edgeTree = op.edgeTree();
         auxTree = op.auxTree();
+
+        // Collect auxiliary data from active tiles
+        if (nonLevelSetGrid) {
+            internal::tileAuxiliaryData(distTree, *edgeTree, *auxTree, mIsovalue);
+        }
     }
 
     // Optionally collect auxiliary data from a reference surface.
@@ -2246,6 +2371,6 @@ VolumeToMesh::operator()(const GridT& distGrid)
 
 #endif // OPENVDB_TOOLS_VOLUME_TO_MESH_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
