@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -127,7 +127,7 @@ protected:
 
         // Note: setItem() can't be called on const iterators.
         void setItem(Index pos, const ChildT& c) const { this->parent().setChildNode(pos, &c); }
-    };
+    };// ChildIter
 
     template<typename NodeT, typename ValueT, typename MaskIterT, typename TagT>
     struct ValueIter: public SparseIteratorBase<
@@ -141,7 +141,7 @@ protected:
 
         // Note: setItem() can't be called on const iterators.
         void setItem(Index pos, const ValueT& v) const { this->parent().mNodes[pos].setValue(v); }
-    };
+    };// ValueIter
 
     template<typename NodeT, typename ChildT, typename ValueT, typename TagT>
     struct DenseIter: public DenseIteratorBase<
@@ -172,7 +172,7 @@ protected:
         {
             this->parent().unsetChildNode(pos, value);
         }
-    };
+    };// DenseIter
 
 public:
     // Iterators (see Iterator.h for usage)
@@ -348,20 +348,6 @@ public:
     template<typename AccessorT>
     bool probeValueAndCache(const Coord& xyz, ValueType& value, AccessorT&) const;
 
-    /// Return the value of the voxel at the given coordinates and, if necessary, update
-    /// the accessor with pointers to the nodes along the path from the root node to
-    /// the node containing the voxel.  Return the state and level of the voxel
-    /// in @a on and @a level, respectively.
-    /// @note Used internally by ValueAccessor.
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& on, int& level, AccessorT&) const;
-
-    template<typename ProbeType, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const;
-
-    template<bool State, bool Level, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const;
-
     /// @brief Return the level of the tree (0 = leaf) at which the value
     /// at the given coordinates resides.
     ///
@@ -374,12 +360,6 @@ public:
     /// Mark all values (both tiles and voxels) as active.
     void setValuesOn();
 
-    /// @brief Update the given accessor with pointers to the nodes along the path
-    /// from the root node to the node containing voxel (x, y, z).
-    /// @note Used internally by ValueAccessor.
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED void updateCache(const Coord& xyz, AccessorT&) const;
-
     //
     // I/O
     //
@@ -388,11 +368,28 @@ public:
     void writeBuffers(std::ostream&, bool toHalf = false) const;
     void readBuffers(std::istream&, bool fromHalf = false);
 
-
-    // @brief Flood fill inactive values based on the signs of the active values,
-    // setting outside values to +background and inside values to -background.
+    /// @brief Overwrites the inactive values with a new value whos
+    /// magnitude is equal to the specified background value and sign
+    /// is consistant with the lexicographically closest active value.
+    /// The net effect is a propagation of signs from the active to the
+    /// inactive values. Note this flood-filling is also performed on
+    /// any child nodes.
+    ///
+    /// @note This method is primarily useful for propagating the sign
+    /// from the (active) voxels in a narrow-band level set to the
+    /// inactive values outside the narrow band.
     void signedFloodFill(const ValueType& background);
 
+    /// @brief Sets the inactive values to either the outside or inside
+    /// value, depending on the sign of the closest corresponding
+    /// active value. More specefically, an inactive value is set to
+    /// the outside value if the closest active value in the
+    /// lexicographic direction is positive, else it is set to the
+    /// inside value. Note this operation is also performed on any child nodes.
+    void signedFloodFill(const ValueType& outside, const ValueType& inside);
+
+    /// Change the sign of all the values represented in this node and
+    /// its child nodes.
     void negate();
 
     /// Replace active tiles with dense voxels, i.e., with active leaf nodes.
@@ -404,14 +401,18 @@ public:
     /// node and the other tree is cannibalized in the process!
     void merge(InternalNode& other, const ValueType& background, const ValueType& otherBackground);
 
-    /// @brief Union this branch's set of active values with the other
-    /// branch's active values.  The value type of the other branch can be
-    /// different.
+    /// @brief Union this branch's set of active values with the other branch's
+    /// active values.  The value type of the other branch can be different.
+    /// @details The resulting state of a value is active if the corresponding value
+    /// was already active OR if it is active in the other tree.  Also, a resulting
+    /// value maps to a voxel if the corresponding value already mapped to a voxel
+    /// OR if it is a voxel in the other tree.  Thus, a resulting value can only
+    /// map to a tile if the corresponding value already mapped to a tile
+    /// AND if it is a tile value in other tree.
     ///
-    /// @note Values in this branch are not modified, only their state. That
-    /// is, active values in this branch are untouched, and values that
-    /// were inactive in this branch but active in the other branch
-    /// are marked as active and set to their original value in this branch.
+    /// Specifically, active tiles and voxels in this branch are not changed, and
+    /// tiles or voxels that were inactive in this branch but active in the other branch
+    /// are marked as active in this branch but left with their original values.
     template<typename OtherChildNodeType>
     void topologyUnion(const InternalNode<OtherChildNodeType, Log2Dim>& other);
 
@@ -498,6 +499,8 @@ public:
     template<typename AccessorT>
     const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT&) const;
 
+    /// @brief Change inactive tiles or voxels with value oldBackground to newBackground
+    /// or -oldBackground to -newBackground. Active values are unchanged.
     void resetBackground(const ValueType& oldBackground, const ValueType& newBackground);
 
     /// @brief Return @c true if the given tree branch has the same node and active value
@@ -1031,54 +1034,6 @@ InternalNode<ChildT, Log2Dim>::probeValueAndCache(const Coord& xyz,
 }
 
 
-template<typename ChildT, Index Log2Dim>
-template<typename AccessorT>
-inline const typename ChildT::ValueType&
-InternalNode<ChildT, Log2Dim>::getValue(const Coord& xyz, bool& state, int& level,
-    AccessorT& acc) const
-{
-    const Index n = this->coord2offset(xyz);
-    if (this->isChildMaskOn(n)) {
-        acc.insert(xyz, mNodes[n].getChild());
-        return mNodes[n].getChild()->getValue(xyz, state, level, acc);
-    }
-    state = this->isValueMaskOn(n);
-    level = LEVEL;
-    return mNodes[n].getValue();
-}
-
-
-template<typename ChildT, Index Log2Dim>
-template<typename ProbeT, typename AccessorT>
-inline const typename ChildT::ValueType&
-InternalNode<ChildT, Log2Dim>::probe(const Coord& xyz, ProbeT& p, AccessorT& acc) const
-{
-    const Index n = this->coord2offset(xyz);
-    if (this->isChildMaskOn(n)) {
-        acc.insert(xyz, mNodes[n].getChild());
-        return mNodes[n].getChild()->probe(xyz, p, acc);
-    }
-    p.setState(this->isValueMaskOn(n));
-    p.setLevel(LEVEL);
-    return mNodes[n].getValue();
-}
-
-template<typename ChildT, Index Log2Dim>
-template<bool State, bool Level, typename AccessorT>
-inline const typename ChildT::ValueType&
-InternalNode<ChildT, Log2Dim>::probe(const Coord& xyz, bool& state, int& level, AccessorT& acc) const
-{
-    const Index n = this->coord2offset(xyz);
-    if (this->isChildMaskOn(n)) {
-        acc.insert(xyz, mNodes[n].getChild());
-        return mNodes[n].getChild()->template probe<State,Level,AccessorT>(xyz, state, level, acc);
-    }
-    if (State) state = this->isValueMaskOn(n);
-    if (Level) level = LEVEL;
-    return mNodes[n].getValue();
-}
-
-
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::setValueOff(const Coord& xyz)
@@ -1401,19 +1356,6 @@ InternalNode<ChildT, Log2Dim>::setValueOnSumAndCache(const Coord& xyz,
 }
 
 
-template<typename ChildT, Index Log2Dim>
-template<typename AccessorT>
-inline void
-InternalNode<ChildT, Log2Dim>::updateCache(const Coord& xyz, AccessorT& acc) const
-{
-    const Index n = this->coord2offset(xyz);
-    if (this->isChildMaskOn(n)) {
-        acc.insert(xyz, mNodes[n].getChild());
-        mNodes[n].getChild()->updateCache(xyz, acc);
-    }
-}
-
-
 ////////////////////////////////////////
 
 
@@ -1570,47 +1512,51 @@ InternalNode<ChildT, Log2Dim>::getLastValue() const
 
 
 ////////////////////////////////////////
-
-/// @brief Propagates the sign from a narrow-band LEVEL SET into the
-/// inactive voxels and tiles. Note this method should only be use on
-/// narrow-band level sets!!!
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::signedFloodFill(const ValueType& background)
 {
-    const ValueType zero = zeroVal<ValueType>();
-    assert(!math::isExactlyEqual(background, zero)); // background must differ from -background
+    this->signedFloodFill(background, negative(background));
+}
 
+template<typename ChildT, Index Log2Dim>
+inline void
+InternalNode<ChildT, Log2Dim>::signedFloodFill(const ValueType& outsideValue,
+                                               const ValueType& insideValue)
+{
     // First, flood fill all child nodes.
     for (ChildOnIter iter = this->beginChildOn(); iter; ++iter) {
-        iter->signedFloodFill(background);
+        iter->signedFloodFill(outsideValue, insideValue);
     }
     const Index first = mChildMask.findFirstOn();
-    assert(first < NUM_VALUES);
-    bool inside = (mNodes[first].getChild()->getFirstValue() < zero),
-         xInside = inside,
-         yInside = inside;
-    for (Index i = 0; i != (1 << Log2Dim); ++i) {
-        const int i00 = i << (2 * Log2Dim); // offset for block(i, 0, 0)
-        if (isChildMaskOn(i00)) {
-            xInside = (mNodes[i00].getChild()->getLastValue() < zero);
-        }
-        yInside = xInside;
-        for (Index j = 0; j != (1 << Log2Dim); ++j) {
-            const Index ij0 = i00 + (j << Log2Dim); // offset for block(i, j, 0)
-            if (isChildMaskOn(ij0)) {
-                yInside = (mNodes[ij0].getChild()->getLastValue() < zero);
+    if (first < NUM_VALUES) {
+        bool xInside = math::isNegative(mNodes[first].getChild()->getFirstValue()),
+            yInside = xInside, zInside = xInside;
+        for (Index x = 0; x != (1 << Log2Dim); ++x) {
+            const int x00 = x << (2 * Log2Dim); // offset for block(x, 0, 0)
+            if (isChildMaskOn(x00)) {
+                xInside = math::isNegative(mNodes[x00].getChild()->getLastValue());
             }
-            inside = yInside;
-            for (Index k = 0; k != (1 << Log2Dim); ++k) {
-                const Index ijk = ij0 + k; // offset for block(i, j, k)
-                if (isChildMaskOn(ijk)) {
-                    inside = (mNodes[ijk].getChild()->getLastValue() < zero);
-                } else {
-                    mNodes[ijk].setValue(inside ? negative(background) : background);
+            yInside = xInside;
+            for (Index y = 0; y != (1 << Log2Dim); ++y) {
+                const Index xy0 = x00 + (y << Log2Dim); // offset for block(x, y, 0)
+                if (isChildMaskOn(xy0)) {
+                    yInside = math::isNegative(mNodes[xy0].getChild()->getLastValue());
+                }
+                zInside = yInside;
+                for (Index z = 0; z != (1 << Log2Dim); ++z) {
+                    const Index xyz = xy0 + z; // offset for block(x, y, z)
+                    if (isChildMaskOn(xyz)) {
+                        zInside = math::isNegative(mNodes[xyz].getChild()->getLastValue());
+                    } else {
+                        mNodes[xyz].setValue(zInside ? insideValue : outsideValue);
+                    }
                 }
             }
         }
+    } else {//no child nodes exist simply use the sign of the first tile value.
+        const ValueType v =  math::isNegative(mNodes[0].getValue()) ? insideValue : outsideValue;
+        for (Index i = 0; i < NUM_VALUES; ++i) mNodes[i].setValue(v);
     }
 }
 
@@ -1685,11 +1631,16 @@ InternalNode<ChildT, Log2Dim>::topologyUnion(const InternalNode<OtherChildT, Log
 
     for (OtherChildIter iter = other.cbeginChildOn(); iter; ++iter) {
         const Index i = iter.pos();
-        if (mChildMask.isOn(i)) {
+        if (mChildMask.isOn(i)) {//this has a child node
             mNodes[i].getChild()->topologyUnion(*iter);
-        } else if (mValueMask.isOff(i)) { //inactive tile
+        } else {// this is a tile so replace it with a child branch with identical topology
+            ChildNodeType* child = new ChildNodeType(*iter, mNodes[i].getValue(), TopologyCopy());
+            if (mValueMask.isOn(i)) {
+                mValueMask.isOff(i);//we're replacing the active tile with a child branch
+                child->setValuesOn();//activate all values since it was an active tile
+            }
             mChildMask.setOn(i);
-            mNodes[i].setChild(new ChildNodeType(*iter, mNodes[i].getValue(), TopologyCopy()));
+            mNodes[i].setChild(child);
         }
     }
     for (OtherValueIter iter = other.cbeginValueOn(); iter; ++iter) {
@@ -2164,18 +2115,19 @@ InternalNode<ChildT, Log2Dim>::offset2globalCoord(Index n) const
 template<typename ChildT, Index Log2Dim>
 inline void
 InternalNode<ChildT, Log2Dim>::resetBackground(const ValueType& oldBackground,
-    const ValueType& newBackground)
+                                               const ValueType& newBackground)
 {
+    if (math::isExactlyEqual(oldBackground, newBackground)) return;
     for (Index i = 0; i < NUM_VALUES; ++i) {
-        if (this->isChildMaskOn(i)) {
-            mNodes[i].getChild()->resetBackground(oldBackground, newBackground);
-        } else {
-            if (math::isApproxEqual(mNodes[i].getValue(), oldBackground)) {
-                mNodes[i].setValue(newBackground);
-            } else if (math::isApproxEqual(mNodes[i].getValue(), negative(oldBackground))) {
-                mNodes[i].setValue(negative(newBackground));
-            }
-        }
+       if (this->isChildMaskOn(i)) {
+           mNodes[i].getChild()->resetBackground(oldBackground, newBackground);
+       } else if (this->isValueMaskOff(i)) {
+           if (math::isApproxEqual(mNodes[i].getValue(), oldBackground)) {
+               mNodes[i].setValue(newBackground);
+           } else if (math::isApproxEqual(mNodes[i].getValue(), negative(oldBackground))) {
+               mNodes[i].setValue(negative(newBackground));
+           }
+       }
     }
 }
 
@@ -2253,6 +2205,6 @@ InternalNode<ChildT, Log2Dim>::getChildNode(Index n) const
 
 #endif // OPENVDB_TREE_INTERNALNODE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
