@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -55,7 +55,9 @@ namespace OPENVDB_VERSION_NAME {
 namespace tree {
 
 /// @brief Templated block class to hold specific data types and a fixed
-/// number of values determined by Log2Dim
+/// number of values determined by Log2Dim. The actual coordinate
+/// dimension of the block is 2^Log2Dim, i.e. Log2Dim=3 corresponds to
+/// a LeafNode that spans a 8^3 block.
 template<typename T, Index Log2Dim>
 class LeafNode
 {
@@ -80,26 +82,34 @@ public:
     struct ValueConverter {
         typedef LeafNode<OtherValueType, Log2Dim> Type;
     };
-
+    
+    /// @brief Stores the actual values in the LeafNode. Its dimension
+    /// it fixed to 2^(3*Log2Dim)
     class Buffer
     {
     public:
+        /// @brief Empty default constructor
         Buffer(): mData(new ValueType[SIZE]) {}
+        /// @brief Constructs a buffer populated with the specified value
         Buffer(const ValueType& val) :  mData(new ValueType[SIZE]) { this->fill(val); }
+        /// @brief Copy constructor
         Buffer(const Buffer& other)   : mData(new ValueType[SIZE]) { *this = other; }
+        /// @brief Destructor
         ~Buffer() { delete [] mData; }
-
+        /// @brief Populates the buffer with a constant value
         void fill(const ValueType& val)
         {
             ValueType* target = mData;
             Index size = SIZE;
             while (size--) *target++ = val;
         }
-        // Fast access methods
+        /// Return a const reference to the i'th element of the Buffer
         const ValueType& getValue(Index i) const { assert(i < SIZE); return mData[i]; }
-        void setValue(Index i, const ValueType& val) { assert(i < SIZE); mData[i] = val; }
+        /// Return a const reference to the i'th element of the Buffer
         const ValueType& operator[](Index i) const { return this->getValue(i); }
-
+        /// Set the i'th value of the Buffer to the specified value
+        void setValue(Index i, const ValueType& val) { assert(i < SIZE); mData[i] = val; }
+        /// Assigns the values in the other Buffer to this Buffer
         Buffer& operator=(const Buffer& other)
         {
             ValueType* target = mData;
@@ -108,6 +118,8 @@ public:
             while (size--) *target++ = *source++;
             return *this;
         }
+        /// Return true if the values in the other buffer exactly
+        /// equates the values in this buffer
         bool operator==(const Buffer& other) const
         {
             const ValueType* target = mData;
@@ -116,16 +128,19 @@ public:
             while (size--) if(!math::isExactlyEqual(*target++, *source++)) return false;
             return true;
         }
+        /// Return true if any of the values in the other buffer do
+        /// not exactly equate the values in this buffer
         bool operator!=(const Buffer& other) const { return !(other == *this); }
-
+        /// Replace the values in this Buffer with the values in the other Buffer 
         void swap(Buffer& other)
         {
             ValueType* tmp = mData;
             mData = other.mData;
             other.mData = tmp;
         }
-
+        /// Return the memory-footprint of this Buffer in units of bytes
         static Index memUsage() { return sizeof(ValueType*) + SIZE * sizeof(ValueType); }
+        /// Return the number of values represented in this Buffer
         static Index size() { return SIZE; }
 
     private:
@@ -171,17 +186,23 @@ public:
     //
     // Statistics
     //
-    /// Return log2 of the size of the buffer storage.
+    /// Return log2 of the dimension of this LeafNode, e.g. 3 if dimensions are 8^3
     static Index log2dim() { return Log2Dim; }
-    /// Return the number of voxels in each dimension.
+    /// Return the number of voxels in each coordinate dimension.
     static Index dim() { return DIM; }
+    /// Return the total number of voxels represented by this LeafNode
     static Index size() { return SIZE; }
+    /// Return the total number of voxels represented by this LeafNode
     static Index numValues() { return SIZE; }
+    /// Return the level of this node, which by definition is zero for LeafNodes
     static Index getLevel() { return LEVEL; }
+    /// Append the Log2Dim of this LeafNode to the specified vector
     static void getNodeLog2Dims(std::vector<Index>& dims) { dims.push_back(Log2Dim); }
+    /// Return the dimension of child nodes of this LeafNode, which is one for voxels.
     static Index getChildDim() { return 1; }
-
+    /// Return the leaf count for this node, which is one.
     static Index32 leafCount() { return 1; }
+    /// Return the non-leaf count for this node, which is zero.
     static Index32 nonLeafCount() { return 0; }
 
     /// Return the number of voxels marked On.
@@ -364,8 +385,6 @@ public:
     /// @brief Exchange this node's data buffer with the given data buffer
     /// without changing the active states of the values.
     void swap(Buffer& other) { mBuffer.swap(other); }
-    /// @deprecated Use buffer() instead.
-    OPENVDB_DEPRECATED const Buffer& getBuffer() const { return mBuffer; }
     const Buffer& buffer() const { return mBuffer; }
     Buffer& buffer() { return mBuffer; }
 
@@ -373,18 +392,22 @@ public:
     // I/O methods
     //
     /// @brief Read in just the topology.
+    /// @param is        the stream from which to read
     /// @param fromHalf  if true, floating-point input values are assumed to be 16-bit
-    void readTopology(std::istream&, bool fromHalf = false);
+    void readTopology(std::istream& is, bool fromHalf = false);
     /// @brief Write out just the topology.
+    /// @param os      the stream to which to write
     /// @param toHalf  if true, output floating-point values as 16-bit half floats
-    void writeTopology(std::ostream&, bool toHalf = false) const;
+    void writeTopology(std::ostream& os, bool toHalf = false) const;
 
     /// @brief Read buffers from a stream.
+    /// @param is        the stream from which to read
     /// @param fromHalf  if true, floating-point input values are assumed to be 16-bit
-    void readBuffers(std::istream&, bool fromHalf = false);
+    void readBuffers(std::istream& is, bool fromHalf = false);
     /// @brief Write buffers to a stream.
+    /// @param os      the stream to which to write
     /// @param toHalf  if true, output floating-point values as 16-bit half floats
-    void writeBuffers(std::ostream&, bool toHalf = false) const;
+    void writeBuffers(std::ostream& os, bool toHalf = false) const;
 
     size_t streamingSize(bool toHalf = false) const;
 
@@ -472,10 +495,7 @@ public:
         mBuffer[offset] += val;
         mValueMask.setOn(offset);
     }
-    /// @deprecated Use setValueOnly() instead.
-    OPENVDB_DEPRECATED void resetValue(const Coord& xyz, const ValueType& val) {
-        this->resetValue(LeafNode::coord2offset(xyz), val);
-    }
+
     /// @brief Change the value of the voxel at the given coordinates without altering
     /// the voxel's active state.
     void setValueOnly(const Coord& xyz, const ValueType& val) {
@@ -488,23 +508,12 @@ public:
     void setValueOnly(Index offset, const ValueType& val) {
         assert(offset<SIZE); mBuffer[offset] = val;
     }
-    /// @deprecated Use setValueOnly() instead.
-    OPENVDB_DEPRECATED void resetValue(Index offset, const ValueType& val) {
-        mBuffer[offset] = val;
-    }
 
     /// Add the given value to all active voxels, leaving inactive voxels unchanged.
     void addValue(const ValueType& val);
     /// Multiply all active voxels by the given value, leaving inactive voxels unchanged.
     void scaleValue(const ValueType& scale);
 
-    /// @brief Mark all voxels as active, but don't change their values.
-    /// @deprecated Use setValuesOn() instead.
-    OPENVDB_DEPRECATED void setValueMaskOn() { mValueMask.setOn(); }
-    /// @brief Mark all voxels as inactive, but don't change their values.
-    /// @deprecated Use setValuesOff() instead.
-    OPENVDB_DEPRECATED void setValueMaskOff() { mValueMask.setOff(); }
-
     /// Mark all voxels as active, but don't change their values.
     void setValuesOn() { mValueMask.setOn(); }
     /// Mark all voxels as inactive, but don't change their values.
@@ -605,32 +614,12 @@ public:
         level = LEVEL;
         return mBuffer[offset];
     }
-    template<typename ProbeType, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const
-    {
-        const Index offset = this->coord2offset(xyz);
-        p.setState(mValueMask.isOn(offset));
-        p.setLevel(LEVEL);
-        return mBuffer[offset];
-    }
-    template<bool State, bool Level, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const
-    {
-        const Index offset = this->coord2offset(xyz);
-        if (State) state =  mValueMask.isOn(offset);
-        if (Level) level = LEVEL;
-        return mBuffer[offset];
-    }
 
     /// @brief Return the LEVEL (=0) at which leaf node values reside.
     /// @note Used internally by ValueAccessor (note last argument is a dummy).
     template<typename AccessorT>
     static Index getValueLevelAndCache(const Coord&, AccessorT&) { return LEVEL; }
 
-    /// @note Used internally by ValueAccessor.
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED void updateCache(const Coord&, AccessorT&) const {}
-
     /// @brief Return a const reference to the first value in the buffer.
     /// @note Though it is potentially risky you can convert this
     /// to a non-const pointer by means of const_case<ValueType*>&.
@@ -641,9 +630,26 @@ public:
     /// @brief Replace inactive occurrences of @a oldBackground with @a newBackground,
     /// and inactive occurrences of @a -oldBackground with @a -newBackground.
     void resetBackground(const ValueType& oldBackground, const ValueType& newBackground);
-
+    
+    /// @brief Overwrites the inactive voxels with a new value whos
+    /// magnitude is equal to the specified background value and sign
+    /// is consistant with the lexicographically closest active voxel.
+    /// The net effect is a propagation of signs from the active
+    /// voxels to the inactive voxels. 
+    ///
+    /// @note This method is primarily useful for propagating the sign
+    /// from the (active) voxels in a narrow-band level set to the
+    /// inactive values outside the narrow band.
     void signedFloodFill(const ValueType& background);
 
+    /// @brief Sets the inactive voxels to either the outside or inside
+    /// value, depending on the sign of the closest corresponding
+    /// active voxels. More specefically, an inactive voxel is set to
+    /// the outside value if the closest active voxel in the
+    /// lexicographic direction is positive, else it is set to the
+    /// inside value.
+    void signedFloodFill(const ValueType& outside, const ValueType& inside);
+
     void negate();
 
     void merge(const LeafNode&);
@@ -768,12 +774,13 @@ public:
     bool isValueMaskOff() const { return mValueMask.isOff(); }
     const NodeMaskType& getValueMask() const { return mValueMask; }
     NodeMaskType& getValueMask() { return mValueMask; }
+    void setValueMask(const NodeMaskType& mask) { mValueMask = mask; }
     bool isChildMaskOn(Index) const { return false; } // leaf nodes have no children
     bool isChildMaskOff(Index) const { return true; }
     bool isChildMaskOff() const { return true; }
 protected:
     void setValueMask(Index n, bool on) { mValueMask.set(n, on); }
-    void setValueMaskOn(Index n) { setValueMask(n, true); }
+    void setValueMaskOn(Index n)  { mValueMask.setOn(n); }
     void setValueMaskOff(Index n) { mValueMask.setOff(n); }
 
     /// Compute the origin of the leaf node that contains the voxel with the given coordinates.
@@ -811,7 +818,7 @@ LeafNode<T, Log2Dim>::LeafNode():
 
 template<typename T, Index Log2Dim>
 inline
-LeafNode<T, Log2Dim>::LeafNode(const Coord& xyz, const T& val, bool active):
+LeafNode<T, Log2Dim>::LeafNode(const Coord& xyz, const ValueType& val, bool active):
     mBuffer(val),
     mValueMask(active),
     mOrigin(xyz & (~(DIM - 1)))
@@ -968,7 +975,7 @@ LeafNode<T, Log2Dim>::setActiveState(const Coord& xyz, bool on)
 
 template<typename T, Index Log2Dim>
 inline void
-LeafNode<T, Log2Dim>::addValue(const T& val)
+LeafNode<T, Log2Dim>::addValue(const ValueType& val)
 {
     for (typename NodeMaskType::OnIterator iter = mValueMask.beginOn(); iter; ++iter) {
         mBuffer[iter.pos()] += val;
@@ -978,7 +985,7 @@ LeafNode<T, Log2Dim>::addValue(const T& val)
 
 template<typename T, Index Log2Dim>
 inline void
-LeafNode<T, Log2Dim>::scaleValue(const T& scale)
+LeafNode<T, Log2Dim>::scaleValue(const ValueType& scale)
 {
     for (typename NodeMaskType::OnIterator iter = mValueMask.beginOn(); iter; ++iter) {
         mBuffer[iter.pos()] *= scale;
@@ -1138,7 +1145,8 @@ LeafNode<T, Log2Dim>::hasSameTopology(const LeafNode<OtherType, OtherLog2Dim>* o
 
 template<typename T, Index Log2Dim>
 inline bool
-LeafNode<T, Log2Dim>::isConstant(T& constValue, bool& state, const T& tolerance) const
+LeafNode<T, Log2Dim>::isConstant(ValueType& constValue,
+    bool& state, const ValueType& tolerance) const
 {
     if (!mValueMask.isOn() && !mValueMask.isOff()) return false;
 
@@ -1154,53 +1162,54 @@ LeafNode<T, Log2Dim>::isConstant(T& constValue, bool& state, const T& tolerance)
     return allEqual;
 }
 
-
-/// @brief Propagates the sign from a narrow-band LEVEL SET into the
-/// inactive voxels. Note this method should only be use on
-/// narrow-band level sets!!!
 template<typename T, Index Log2Dim>
 inline void
-LeafNode<T, Log2Dim>::signedFloodFill(const T& background)
+LeafNode<T, Log2Dim>::signedFloodFill(const ValueType& background)
+{
+    this->signedFloodFill(background, negative(background));
+}
+
+template<typename T, Index Log2Dim>
+inline void
+LeafNode<T, Log2Dim>::signedFloodFill(const ValueType& outsideValue,
+                                      const ValueType& insideValue)
 {
-    assert(!math::isZero(background)); // background must differ from -background
     const Index first = mValueMask.findFirstOn();
-
-    if (first == SIZE) return; // empty
-
-    const T zero = zeroVal<T>();
-    bool inside = (mBuffer[first] < zero), xInside = inside, yInside = inside;
-    for (Index i = 0; i != (1 << Log2Dim); ++i) {
-        const Index x = i << (2 * Log2Dim);
-        if (mValueMask.isOn(x)) {
-            xInside = (mBuffer[x] < zero); // element(i, 0, 0)
-        }
-        yInside = xInside;
-        for (Index j = 0; j != (1 << Log2Dim); ++j) {
-            const Index xy = x + (j << Log2Dim);
-            if (mValueMask.isOn(xy)) {
-                yInside = (mBuffer[xy] < zero); // element(i, j, 0)
+    if (first < SIZE) {
+        bool xInside = math::isNegative(mBuffer[first]), yInside = xInside, zInside = xInside;
+        for (Index x = 0; x != (1 << Log2Dim); ++x) {
+            const Index x00 = x << (2 * Log2Dim);
+            if (mValueMask.isOn(x00)) {
+                xInside = math::isNegative(mBuffer[x00]); // element(x, 0, 0)
             }
-            inside = yInside;
-            for (Index k = 0; k != (1 << Log2Dim); ++k) {
-                const Index xyz = xy + k; // element(i, j, k)
-                if (mValueMask.isOn(xyz)) {
-                    inside = (mBuffer[xyz] < zero);
-                } else if (inside) {
-                    mBuffer[xyz] = negative(background);
-                } else { // outside
-                    mBuffer[xyz] = background;
+            yInside = xInside;
+            for (Index y = 0; y != (1 << Log2Dim); ++y) {
+                const Index xy0 = x00 + (y << Log2Dim);
+                if (mValueMask.isOn(xy0)) {
+                    yInside = math::isNegative(mBuffer[xy0]); // element(x, y, 0)
+                }
+                zInside = yInside;
+                for (Index z = 0; z != (1 << Log2Dim); ++z) {
+                    const Index xyz = xy0 + z; // element(x, y, z)
+                    if (mValueMask.isOn(xyz)) {
+                        zInside = math::isNegative(mBuffer[xyz]);
+                    } else {
+                        mBuffer[xyz] = zInside ? insideValue : outsideValue;
+                    }
                 }
             }
         }
+    } else {// if no active voxels exist simply use the sign of the first value
+        mBuffer.fill(math::isNegative(mBuffer[0]) ? insideValue : outsideValue);
     }
 }
 
 
 template<typename T, Index Log2Dim>
 inline void
-LeafNode<T, Log2Dim>::resetBackground(const T& oldBackground, const T& newBackground)
+LeafNode<T, Log2Dim>::resetBackground(const ValueType& oldBackground,
+                                      const ValueType& newBackground)
 {
-    if (math::isExactlyEqual(oldBackground, newBackground)) return;
     typename NodeMaskType::OffIterator iter;
     // For all inactive values...
     for (iter = this->mValueMask.beginOff(); iter; ++iter) {
@@ -1518,6 +1527,6 @@ operator<<(std::ostream& os, const typename LeafNode<T, Log2Dim>::Buffer& buf)
 
 #endif // OPENVDB_TREE_LEAFNODE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
