@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -208,7 +208,6 @@ public:
     /// @brief Exchange this node's data buffer with the given data buffer
     /// without changing the active states of the values.
     void swap(Buffer& other) { mBuffer.swap(other); }
-    OPENVDB_DEPRECATED const Buffer& getBuffer() const { return mBuffer; }
     const Buffer& buffer() const { return mBuffer; }
     Buffer& buffer() { return mBuffer; }
 
@@ -275,26 +274,15 @@ public:
     void setValueOnly(const Coord& xyz, bool val) {
         this->setValueOnly(LeafNode::coord2offset(xyz), val);
     }
-    OPENVDB_DEPRECATED void resetValue(const Coord& xyz, bool val) {
-        mBuffer.mData.set(this->coord2offset(xyz), val);
-    }
     /// @brief Change the value of the voxel at the given offset without altering
     /// the voxel's active state.
     void setValueOnly(Index offset, bool val) { assert(offset<SIZE); mBuffer.setValue(offset,val); }
-    OPENVDB_DEPRECATED void resetValue(Index offset, bool val) { mBuffer.setValue(offset, val); }
 
     /// Add the given value to all active voxels, leaving inactive voxels unchanged.
     void addValue(bool val);
     /// Multiply all active voxels by the given value, leaving inactive voxels unchanged.
     void scaleValue(bool scale);
 
-    /// @brief Mark all voxels as active, but don't change their values.
-    /// @deprecated Use setValuesOn() instead.
-    OPENVDB_DEPRECATED void setValueMaskOn() { mValueMask.setOn(); }
-    /// @brief Mark all voxels as inactive, but don't change their values.
-    /// @deprecated Use setValuesOff() instead.
-    OPENVDB_DEPRECATED void setValueMaskOff() { mValueMask.setOff(); }
-
     /// Mark all voxels as active, but don't change their values.
     void setValuesOn() { mValueMask.setOn(); }
     /// Mark all voxels as inactive, but don't change their values.
@@ -365,45 +353,11 @@ public:
         return this->probeValue(xyz, val);
     }
 
-    /// @brief Return the value of the voxel at the given coordinates and return
-    /// its active state and level (i.e., 0) in @a state and @a level.
-    /// @note Used internally by ValueAccessor (note last argument is a dummy).
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& state, int& level, AccessorT&) const
-    {
-        const Index offset = this->coord2offset(xyz);
-        state = mValueMask.isOn(offset);
-        level = LEVEL;
-        // This *CANNOT* use operator ? for Windows
-        if (mBuffer.mData.isOn(offset)) return sOn; else return sOff;
-    }
-    template<typename ProbeType, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const
-    {
-        const Index offset = this->coord2offset(xyz);
-        p.setState(mValueMask.isOn(offset));
-        p.setLevel(LEVEL);
-        // This *CANNOT* use operator ? for Windows
-        if (mBuffer.mData.isOn(offset)) return sOn; else return sOff;
-    }
-    template<bool State, bool Level, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const
-    {
-        const Index offset = this->coord2offset(xyz);
-        if (State) state =  mValueMask.isOn(offset);
-        if (Level) level = LEVEL;
-        // This *CANNOT* use operator ? for Windows
-        if (mBuffer.mData.isOn(offset)) return sOn; else return sOff;
-    }
     /// @brief Return the LEVEL (=0) at which leaf node values reside.
     /// @note Used internally by ValueAccessor (note last argument is a dummy).
     template<typename AccessorT>
     static Index getValueLevelAndCache(const Coord&, AccessorT&) { return LEVEL; }
 
-    /// @note Used internally by ValueAccessor.
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED void updateCache(const Coord&, AccessorT&) const {}
-
     /// @brief Return a const reference to the first entry in the buffer.
     /// @note Since it's actually a reference to a static data member
     /// it should not be converted to a non-const pointer!
@@ -479,6 +433,8 @@ public:
     //@{
     /// This function exists only to enable template instantiation.
     void signedFloodFill(bool) {}
+    /// This function exists only to enable template instantiation.
+    void signedFloodFill(bool, bool) {}
     template<typename PruneOp> void pruneOp(PruneOp&) {}
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
     void pruneInactive(const ValueType&) {}
@@ -629,12 +585,13 @@ public:
     bool isValueMaskOff() const { return mValueMask.isOff(); }
     const NodeMaskType& getValueMask() const { return mValueMask; }
     NodeMaskType& getValueMask() { return mValueMask; }
+    void setValueMask(const NodeMaskType& mask) { mValueMask = mask; }
     bool isChildMaskOn(Index) const { return false; } // leaf nodes have no children
     bool isChildMaskOff(Index) const { return true; }
     bool isChildMaskOff() const { return true; }
 protected:
     void setValueMask(Index n, bool on) { mValueMask.set(n, on); }
-    void setValueMaskOn(Index n) { setValueMask(n, true); }
+    void setValueMaskOn(Index n)  { mValueMask.setOn(n); }
     void setValueMaskOff(Index n) { mValueMask.setOff(n); }
 
     /// Compute the origin of the leaf node that contains the voxel with the given coordinates.
@@ -1398,6 +1355,6 @@ LeafNode<bool, Log2Dim>::doVisit2(NodeT& self, OtherChildAllIterT& otherIter,
 
 #endif // OPENVDB_TREE_LEAFNODEBOOL_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
