@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -41,6 +41,7 @@
 #include <boost/type_traits/remove_const.hpp>
 #include <openvdb/Exceptions.h>
 #include <openvdb/Types.h>
+#include <openvdb/io/Compression.h> // for truncateRealToHalf()
 #include <openvdb/math/Math.h> // for isZero(), isExactlyEqual(), etc.
 #include <openvdb/math/BBox.h>
 #include <openvdb/util/NodeMasks.h> // for backward compatibility only (see readTopology())
@@ -389,8 +390,14 @@ public:
     /// Return the bounding box of this RootNode, i.e., an infinite bounding box.
     static CoordBBox getNodeBoundingBox() { return CoordBBox::inf(); }
 
+    /// @brief Change inactive tiles or voxels with a value equal to +/- the
+    /// old background to the specified value (with the same sign). Active values
+    /// are unchanged.
     void setBackground(const ValueType& value);
-    ValueType getBackground() const { return mBackground; }
+    /// Return the background value
+    const ValueType& background() const { return mBackground; }
+    /// @deprecated Use background()
+    OPENVDB_DEPRECATED ValueType getBackground() const { return mBackground; }
 
     /// Return @c true if the given tile is inactive and has the background value.
     bool isBackgroundTile(const Tile&) const;
@@ -493,15 +500,6 @@ public:
     void writeBuffers(std::ostream&, bool toHalf = false) const;
     void readBuffers(std::istream&, bool fromHalf = false);
 
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& state, int& level, AccessorT&) const;
-
-    template<typename ProbeType, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ProbeType& p, AccessorT&) const;
-
-    template<bool State, bool Level, typename AccessorT>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level, AccessorT&) const;
-
     /// Return the value of the voxel at the given coordinates and, if necessary, update
     /// the accessor with pointers to the nodes along the path from the root node to
     /// the node containing the voxel.
@@ -566,12 +564,6 @@ public:
     template<typename AccessorT>
     int getValueDepthAndCache(const Coord& xyz, AccessorT&) const;
 
-    /// @brief Update the given accessor with pointers to the nodes along the path
-    /// from the root node to the node containing voxel (x, y, z).
-    /// @note Used internally by ValueAccessor.
-    template<typename AccessorT>
-    OPENVDB_DEPRECATED void updateCache(const Coord& xyz, AccessorT&) const;
-
     /// Call the @c PruneOp functor for each child node and, if the functor
     /// returns @c true, prune the node and replace it with a tile.
     ///
@@ -622,9 +614,20 @@ public:
     template<typename AccessorT>
     const LeafNodeType* probeConstLeafAndCache(const Coord& xyz, AccessorT&) const;
 
-    /// @brief Propagate the sign from a narrow-band level set into inactive voxels and tiles.
-    /// @note This method should only be used on closed narrow-band level sets!!!
-    bool signedFloodFill();
+    /// @brief Set the values of all inactive voxels and tiles of a narrow-band
+    /// level set from the signs of the active voxels, setting outside values to
+    /// +background and inside values to -background.
+    ///
+    /// @note This method should only be used on closed, narrow-band level sets!
+    void signedFloodFill();
+
+    /// @brief Set the values of all inactive voxels and tiles of a narrow-band
+    /// level set from the signs of the active voxels, setting outside values to
+    /// "outside" and inside values to "inside. The background value of the grid
+    /// will be set to the "outside" value.
+    ///
+    /// @note This method should only be used on closed, narrow-band level sets!
+    void signedFloodFill(const ValueType& outside, const ValueType& inside);
 
     /// Move child nodes from the other tree into this tree wherever those nodes
     /// correspond to constant-value tiles in this tree, and replace leaf-level
@@ -636,14 +639,19 @@ public:
     /// Turn active tiles into dense voxels, i.e., into leaf nodes that are entirely active.
     void voxelizeActiveTiles();
 
-    /// @brief Union this tree's set of active values with the other
-    /// tree's active values.  The value type of the other tree can be
-    /// different.
+    /// @brief Union this tree's set of active values with the active values
+    /// of the other tree, whose @c ValueType may be different.
+    /// @details The resulting state of a value is active if the corresponding value
+    /// was already active OR if it is active in the other tree.  Also, a resulting
+    /// value maps to a voxel if the corresponding value already mapped to a voxel
+    /// OR if it is a voxel in the other tree.  Thus, a resulting value can only
+    /// map to a tile if the corresponding value already mapped to a tile
+    /// AND if it is a tile value in other tree.
     ///
-    /// @note Values in this tree are not modified, only their state. That
-    /// is, active values in this tree are untouched, and values that
-    /// were inactive in this tree but active in the other tree
-    /// are marked as active and set to their original value in this tree.
+    /// @note This operation modifies only active states, not values.
+    /// Specifically, active tiles and voxels in this tree are not changed, and
+    /// tiles or voxels that were inactive in this tree but active in the other tree
+    /// are marked as active in this tree but left with their original values.
     template<typename OtherChildType>
     void topologyUnion(const RootNode<OtherChildType>& other);
 
@@ -824,13 +832,15 @@ RootNode<ChildT>::setBackground(const ValueType& background)
     // and -mBackground with -background.
     for (MapIter iter=mTable.begin(); iter!=mTable.end(); ++iter) {
         ChildT *child = iter->second.child;
-        if (child)
+        if (child) {
             child->resetBackground(/*old=*/mBackground, /*new=*/background);
-        else {
-            if (math::isApproxEqual(getTile(iter).value, mBackground)) {
-                getTile(iter).value = background;
-            } else if (math::isApproxEqual(getTile(iter).value, negative(mBackground))) {
-                getTile(iter).value = negative(background);
+        } else {
+            Tile& tile = getTile(iter);
+            if (tile.active) continue;//only change inactive tiles
+            if (math::isApproxEqual(tile.value, mBackground)) {
+                tile.value = background;
+            } else if (math::isApproxEqual(tile.value, negative(mBackground))) {
+                tile.value = negative(background);
             }
         }
     }
@@ -1594,78 +1604,6 @@ RootNode<ChildT>::probeValueAndCache(const Coord& xyz, ValueType& value, Accesso
 }
 
 
-template<typename ChildT>
-template<typename AccessorT>
-inline const typename ChildT::ValueType&
-RootNode<ChildT>::getValue(const Coord& xyz, bool& state, int& level, AccessorT& acc) const
-{
-    MapCIter iter = this->findCoord(xyz);
-    if (iter == mTable.end()) {
-        state = false;
-        level = -1;
-        return mBackground;
-    } else if (isChild(iter)) {
-        acc.insert(xyz, &getChild(iter));
-        return getChild(iter).getValue(xyz, state, level, acc);
-    }
-    state = isTileOn(iter);
-    level = LEVEL;
-    return getTile(iter).value;
-}
-
-
-template<typename ChildT>
-template<typename ProbeT, typename AccessorT>
-inline const typename ChildT::ValueType&
-RootNode<ChildT>::probe(const Coord& xyz, ProbeT& p, AccessorT& acc) const
-{
-    MapCIter iter = this->findCoord(xyz);
-    if (iter == mTable.end()) {
-        p.setState(false);
-        p.setLevel(-1);
-        return mBackground;
-    } else if (isChild(iter)) {
-        acc.insert(xyz, &getChild(iter));
-        return getChild(iter).probe(xyz, p, acc);
-    }
-    p.setState(isTileOn(iter));
-    p.setLevel(LEVEL);
-    return getTile(iter).value;
-}
-
-template<typename ChildT>
-template<bool State, bool Level, typename AccessorT>
-inline const typename ChildT::ValueType&
-RootNode<ChildT>::probe(const Coord& xyz, bool& state, int& level, AccessorT& acc) const
-{
-    MapCIter iter = this->findCoord(xyz);
-    if (iter == mTable.end()) {
-        if (State) state = false;
-        if (Level) level = -1;
-        return mBackground;
-    } else if (isChild(iter)) {
-        acc.insert(xyz, &getChild(iter));
-        return getChild(iter).template probe<State,Level,AccessorT>(xyz, state, level, acc);
-    }
-    if (State) state = isTileOn(iter);
-    if (Level) level = LEVEL;
-    return getTile(iter).value;
-}
-
-
-template<typename ChildT>
-template<typename AccessorT>
-inline void
-RootNode<ChildT>::updateCache(const Coord& xyz, AccessorT& acc) const
-{
-    MapCIter iter = this->findCoord(xyz);
-    if (iter != mTable.end() && isChild(iter)) {
-        acc.insert(xyz, &getChild(iter));
-        return getChild(iter).updateCache(xyz, acc);
-    }
-}
-
-
 ////////////////////////////////////////
 
 
@@ -1732,7 +1670,12 @@ template<typename ChildT>
 inline bool
 RootNode<ChildT>::writeTopology(std::ostream& os, bool toHalf) const
 {
-    os.write(reinterpret_cast<const char*>(&mBackground), sizeof(ValueType));
+    if (!toHalf) {
+        os.write(reinterpret_cast<const char*>(&mBackground), sizeof(ValueType));
+    } else {
+        ValueType truncatedVal = io::truncateRealToHalf(mBackground);
+        os.write(reinterpret_cast<const char*>(&truncatedVal), sizeof(ValueType));
+    }
     io::setGridBackgroundValuePtr(os, &mBackground);
 
     const Index numTiles = this->getTileCount(), numChildren = this->getChildCount();
@@ -1927,6 +1870,9 @@ RootNode<ChildT>::pruneInactive()
 }
 
 
+////////////////////////////////////////
+
+
 template<typename ChildT>
 inline typename ChildT::LeafNodeType*
 RootNode<ChildT>::touchLeaf(const Coord& xyz)
@@ -1945,6 +1891,7 @@ RootNode<ChildT>::touchLeaf(const Coord& xyz)
     return child->touchLeaf(xyz);
 }
 
+
 template<typename ChildT>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -1966,6 +1913,9 @@ RootNode<ChildT>::touchLeafAndCache(const Coord& xyz, AccessorT& acc)
 }
 
 
+////////////////////////////////////////
+
+
 template<typename ChildT>
 inline typename ChildT::LeafNodeType*
 RootNode<ChildT>::probeLeaf(const Coord& xyz)
@@ -1975,6 +1925,7 @@ RootNode<ChildT>::probeLeaf(const Coord& xyz)
     return getChild(iter).probeLeaf(xyz);
 }
 
+
 template<typename ChildT>
 inline const typename ChildT::LeafNodeType*
 RootNode<ChildT>::probeConstLeaf(const Coord& xyz) const
@@ -1984,6 +1935,7 @@ RootNode<ChildT>::probeConstLeaf(const Coord& xyz) const
     return getChild(iter).probeConstLeaf(xyz);
 }
 
+
 template<typename ChildT>
 template<typename AccessorT>
 inline typename ChildT::LeafNodeType*
@@ -1996,6 +1948,7 @@ RootNode<ChildT>::probeLeafAndCache(const Coord& xyz, AccessorT& acc)
     return child->probeLeafAndCache(xyz, acc);
 }
 
+
 template<typename ChildT>
 template<typename AccessorT>
 inline const typename ChildT::LeafNodeType*
@@ -2013,27 +1966,35 @@ RootNode<ChildT>::probeConstLeafAndCache(const Coord& xyz, AccessorT& acc) const
 
 
 template<typename ChildT>
-inline bool
+inline void
 RootNode<ChildT>::signedFloodFill()
+{
+    this->signedFloodFill(mBackground, negative(mBackground));
+}
+
+
+template<typename ChildT>
+inline void
+RootNode<ChildT>::signedFloodFill(const ValueType& outside, const ValueType& inside)
 {
     const ValueType zero = zeroVal<ValueType>();
 
-    if (!(mBackground > zero)) return false;//bkgd must be positive
+    mBackground = outside;
 
     // First, flood fill all child nodes and put child-keys into a sorted set
     CoordSet nodeKeys;
     for (MapIter i = mTable.begin(), e = mTable.end(); i != e; ++i) {
         if (this->isTile(i)) continue;
-        getChild(i).signedFloodFill(mBackground);
+        getChild(i).signedFloodFill(outside, inside);
         nodeKeys.insert(i->first);//only add inactive tiles!
     }
 
     // We employ a simple z-scanline algorithm that inserts inactive
-    // tiles with a negative background value if they are sandwiched
+    // tiles with the inside value if they are sandwiched
     // between inside child nodes only!
-    const Tile inside(negative(mBackground), /*on=*/false);
+    const Tile insideTile(inside, /*on=*/false);
     CoordSetCIter b = nodeKeys.begin(), e = nodeKeys.end();
-    if ( b == e ) return false;
+    if ( b == e ) return;
     for (CoordSetCIter a = b++; b != e; ++a, ++b) {
         Coord d = *b - *a; // delta of neighboring keys
         if (d[0]!=0 || d[1]!=0 || d[2]==Int32(ChildT::DIM)) continue;//not z-scanline or neighbors
@@ -2041,10 +2002,9 @@ RootNode<ChildT>::signedFloodFill()
         const ValueType fill[] = { getChild(i).getLastValue(), getChild(j).getFirstValue() };
         if (!(fill[0] < zero) || !(fill[1] < zero)) continue; // scanline isn't inside
         for (Coord c = *a + Coord(0u,0u,ChildT::DIM); c[2] != (*b)[2]; c[2] += ChildT::DIM) {
-            mTable[c] = inside;
+            mTable[c] = insideTile;
         }
     }
-    return true;
 }
 
 
@@ -2094,6 +2054,10 @@ RootNode<ChildT>::merge(RootNode& other)
     other.clear();
 }
 
+
+////////////////////////////////////////
+
+
 template<typename ChildT>
 template<typename OtherChildType>
 inline void
@@ -2112,9 +2076,11 @@ RootNode<ChildT>::topologyUnion(const RootNode<OtherChildType>& other)
                     *(new ChildT(other.getChild(i), mBackground, TopologyCopy())));
             } else if (this->isChild(j)) { // union with child branch
                 this->getChild(j).topologyUnion(other.getChild(i));
-            } else if (this->isTileOff(j)) { // replace tile with other child branch
-                this->setChild(j,
-                    *(new ChildT(other.getChild(i), this->getTile(j).value, TopologyCopy())));
+            } else {// this is a tile so replace it with a child branch with identical topology
+                ChildT* child = new ChildT(
+                    other.getChild(i), this->getTile(j).value, TopologyCopy());
+                if (this->isTileOn(j)) child->setValuesOn();//this is an active tile
+                this->setChild(j, *child);
             }
         } else if (other.isTileOn(i)) { // other is an active tile
             if (j == mTable.end()) { // insert an active tile
@@ -2403,6 +2369,6 @@ RootNode<ChildT>::doVisit2(RootNodeT& self, OtherRootNodeT& other, VisitorOp& op
 
 #endif // OPENVDB_TREE_ROOTNODE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
