@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -417,9 +417,9 @@ public:
     ///
     /// @note This method is faster then tolerance based prune and
     /// useful for narrow-band level set applications where inactive
-    /// values are limited to either an inside or outside value. 
+    /// values are limited to either an inside or outside value.
     void pruneLevelSet();
-    
+
     /// @brief @return the leaf node that contains voxel (x, y, z) and
     /// if it doesn't exist, create it, but preserve the values and
     /// active states of all voxels.
@@ -427,7 +427,7 @@ public:
     /// Use this method to preallocate a static tree topology over which to
     /// safely perform multithreaded processing.
     LeafNodeType* touchLeaf(const Coord& xyz);
-    
+
     /// @brief @return a pointer to the leaf node that contains
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
     LeafNodeType* probeLeaf(const Coord& xyz);
@@ -435,9 +435,6 @@ public:
     /// voxel (x, y, z) and if it doesn't exist, return NULL.
     const LeafNodeType* probeConstLeaf(const Coord& xyz) const;
 
-    /// Expand this tree's index range to include the given coordinates.
-    OPENVDB_DEPRECATED bool expand(const Coord& xyz) { return mRoot.expand(xyz); }
-
 
     //
     // Aux methods
@@ -469,21 +466,32 @@ public:
     virtual Metadata::Ptr getBackgroundValue() const;
 
     /// Return this tree's background value.
-    ValueType getBackground() const { return mRoot.getBackground(); }
+    const ValueType& background() const { return mRoot.background(); }
+    /// @deprecated Use background()
+    OPENVDB_DEPRECATED ValueType getBackground() const { return mRoot.background(); }
     /// Replace this tree's background value.
     void setBackground(const ValueType& background) { mRoot.setBackground(background); }
 
-    OPENVDB_DEPRECATED Index getWidth()  const { return mRoot.getWidth(); }
-    OPENVDB_DEPRECATED Index getHeight() const { return mRoot.getHeight(); }
-    OPENVDB_DEPRECATED Index getDepth()  const { return mRoot.getDepth(); }
-
     /// Min and max are both inclusive.
     virtual void getIndexRange(CoordBBox& bbox) const { mRoot.getIndexRange(bbox); }
 
-    /// @brief Propagate the sign from a narrow-band level set into inactive voxels and tiles.
+    /// @brief Set the values of all inactive voxels and tiles of a narrow-band
+    /// level set from the signs of the active voxels, setting outside values to
+    /// +background and inside values to -background.
+    ///
     /// @note This method should only be used on closed, narrow-band level sets!
     void signedFloodFill() { mRoot.signedFloodFill(); }
 
+    /// @brief Set the values of all inactive voxels and tiles of a narrow-band
+    /// level set from the signs of the active voxels, setting outside values to
+    /// "outside" and inside values to "inside. The background value of the grid
+    /// will be set to the "outside" value.
+    ///
+    /// @note This method should only be used on closed, narrow-band level sets!
+    void signedFloodFill(const ValueType& outside, const ValueType& inside)
+    {
+        mRoot.signedFloodFill(outside, inside);
+    }
 
     /// Move child nodes from the other tree into this tree wherever those nodes
     /// correspond to constant-value tiles in this tree, and replace leaf-level
@@ -507,6 +515,12 @@ public:
 
     /// @brief Union this tree's set of active values with the active values
     /// of the other tree, whose @c ValueType may be different.
+    /// @details The resulting state of a value is active if the corresponding value
+    /// was already active OR if it is active in the other tree.  Also, a resulting
+    /// value maps to a voxel if the corresponding value already mapped to a voxel
+    /// OR if it is a voxel in the other tree.  Thus, a resulting value can only
+    /// map to a tile if the corresponding value already mapped to a tile
+    /// AND if it is a tile value in other tree.
     ///
     /// @note This operation modifies only active states, not values.
     /// Specifically, active tiles and voxels in this tree are not changed, and
@@ -1410,14 +1424,14 @@ template<typename RootNodeType>
 inline void
 Tree<RootNodeType>::pruneInactive()
 {
-    this->pruneInactive(this->getBackground());
+    this->pruneInactive(this->background());
 }
 
 template<typename RootNodeType>
 inline void
 Tree<RootNodeType>::pruneLevelSet()
 {
-    LevelSetPrune<ValueType> op;
+    LevelSetPrune<ValueType> op(this->background());
     this->pruneOp(op);
 }
 
@@ -1463,7 +1477,7 @@ Tree<RootNodeType>::getBackgroundValue() const
         typedef TypedMetadata<ValueType> MetadataT;
         result = Metadata::createMetadata(valueType());
         if (MetadataT* m = dynamic_cast<MetadataT*>(result.get())) {
-            m->value() = mRoot.getBackground();
+            m->value() = mRoot.background();
         }
     }
     return result;
@@ -1856,7 +1870,7 @@ Tree<RootNodeType>::print(std::ostream& os, int verboseLevel) const
             os << " x " << (1 << *dims.rbegin()) << "^3)\n";
         }
     }
-    os << "  Background value: " << mRoot.getBackground() << "\n";
+    os << "  Background value: " << mRoot.background() << "\n";
 
     if (verboseLevel == 1) return;
 
@@ -1896,7 +1910,7 @@ Tree<RootNodeType>::print(std::ostream& os, int verboseLevel) const
 
         const double activeRatio = (100.0 * numActiveVoxels) / totalVoxels;
         os << "  Percentage of active voxels:   " << std::setprecision(3) << activeRatio << "%\n";
-        
+
         if (leafCount>0) {
             const double fillRatio =
                 (100.0 * numActiveLeafVoxels) / (leafCount * LeafNodeType::NUM_VOXELS);
@@ -1936,6 +1950,6 @@ Tree<RootNodeType>::print(std::ostream& os, int verboseLevel) const
 
 #endif // OPENVDB_TREE_TREE_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
