@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -264,7 +264,7 @@ public:
     typedef typename PrevItemT::IterT PrevIterT;
     /// The type of node (non-const) whose iterator is stored in this list item
     typedef typename boost::mpl::front<NodeVecT>::type _NodeT;
-    /// The type of iterator stored in this list item (e.g., InternalNode<...>::ValueOnCIter)
+    /// The type of iterator stored in this list item (e.g., InternalNode::ValueOnCIter)
     typedef typename IterTraits<typename PrevIterT::NonConstNodeType, PrevIterT>::template
         NodeConverter<_NodeT>::Type IterT;
 
@@ -420,7 +420,7 @@ public:
     typedef typename PrevItemT::IterT PrevIterT;
     /// The type of node (non-const) whose iterator is stored in this list item
     typedef typename boost::mpl::front<NodeVecT>::type _NodeT;
-    /// The type of iterator stored in this list item (e.g., InternalNode<...>::ValueOnCIter)
+    /// The type of iterator stored in this list item (e.g., InternalNode::ValueOnCIter)
     typedef typename IterTraits<typename PrevIterT::NonConstNodeType, PrevIterT>::template
         NodeConverter<_NodeT>::Type IterT;
 
@@ -536,7 +536,7 @@ public:
     typedef typename boost::mpl::front<NodeVecT>::type _NodeT;
     /// The type of iterator stored in the previous list item
     typedef typename PrevItemT::IterT PrevIterT;
-    /// The type of iterator stored in this list item (e.g., RootNode<...>::ValueOnCIter)
+    /// The type of iterator stored in this list item (e.g., RootNode::ValueOnCIter)
     typedef typename IterTraits<typename PrevIterT::NonConstNodeType, PrevIterT>::template
         NodeConverter<_NodeT>::Type IterT;
 
@@ -718,13 +718,9 @@ public:
     /// @brief Change the tile or voxel value to which this iterator is currently pointing
     /// and mark it as active.
     void setValue(const ValueT& val) const { mValueIterList.setValue(mLevel, val); }
-    //@{
     /// @brief Change the active/inactive state of the tile or voxel value to which
     /// this iterator is currently pointing.
     void setActiveState(bool on) const { mValueIterList.setValueOn(mLevel, on); }
-    /// @deprecated Use setActiveState() instead.
-    OPENVDB_DEPRECATED void setValueOn(bool on=true) const {mValueIterList.setValueOn(mLevel,on);}
-    //@}
     /// Mark the tile or voxel value to which this iterator is currently pointing as inactive.
     void setValueOff() const { mValueIterList.setValueOff(mLevel); }
 
@@ -1359,6 +1355,6 @@ private:
 
 #endif // OPENVDB_TREE_TREEITERATOR_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
