@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -593,55 +593,6 @@ public:
         return mNext.probeConstLeaf(xyz);
     }
 
-    // ====================> DEPRECATED METHODS <========================
-    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& state, int& level)
-    {
-        if (this->isHashed(xyz)) {
-            assert(mNode);
-            return mNode->getValue(xyz, state, level, *mParent);
-        }
-        return mNext.getValue(xyz, state, level);
-    }
-    template <typename ValueProbeType>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ValueProbeType &p)
-    {
-        if (this->isHashed(xyz)) {
-            assert(mNode);
-            return mNode->probe(xyz, p, *mParent);
-        }
-        return mNext.probe(xyz, p);
-    }
-    template <bool State, bool Level>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level)
-    {
-        if (this->isHashed(xyz)) {
-            assert(mNode);
-            return mNode->probe<State, Level>(xyz, state, level, *mParent);
-        }
-        return mNext.probe<State, Level>(xyz, state, level);
-    }
-    OPENVDB_DEPRECATED const ValueType& newGetValue(const Coord& xyz)
-    {
-        if (this->isHashed(xyz)) {
-            assert(mNode);
-            const ValueType& val = mNode->getValue(xyz);
-            mNode->updateCache(xyz, *mParent);
-            return val;
-        }
-        return mNext.newGetValue(xyz);
-    }
-    OPENVDB_DEPRECATED void newSetValue(const Coord& xyz, const ValueType& value)
-    {
-        if (this->isHashed(xyz)) {
-            assert(mNode);
-            BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
-            const_cast<NodeType*>(mNode)->setValue(xyz, value);
-            mNode->updateCache(xyz, *mParent);
-        } else {
-            mNext.setValue(xyz, value);
-        }
-    }
-    /////////////////////////////////////////////////////////////////////////
     /// Return the active state of the voxel at the given coordinates.
     bool isValueOn(const Coord& xyz)
     {
@@ -847,39 +798,7 @@ public:
         assert(mRoot);
         return mRoot->getValueAndCache(xyz, *mParent);
     }
-    // ====================> DEPRECATED METHODS <========================
-    OPENVDB_DEPRECATED const ValueType& newGetValue(const Coord& xyz)
-    {
-        assert(mRoot);
-        const ValueType& val = mRoot->getValue(xyz);
-        mRoot->updateCache(xyz, *mParent);
-        return val;
-    }
-    OPENVDB_DEPRECATED const ValueType& getValue(const Coord& xyz, bool& state, int& level)
-    {
-        assert(mRoot);
-        return mRoot->getValue(xyz, state, level, *mParent);
-    }
-    template <bool State, bool Level>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, bool& state, int& level)
-    {
-        assert(mRoot);
-        return mRoot->probe<State, Level>(xyz, state, level, *mParent);
-    }
-    template <typename ValueProbeType>
-    OPENVDB_DEPRECATED const ValueType& probe(const Coord& xyz, ValueProbeType& p)
-    {
-        assert(mRoot);
-        return mRoot->probe(xyz, p, *mParent);
-    }
-    OPENVDB_DEPRECATED void newSetValue(const Coord& xyz, const ValueType& value)
-    {
-        assert(mRoot);
-        BOOST_STATIC_ASSERT(!TreeCacheT::IsConstTree);
-        const_cast<RootNodeType*>(mRoot)->setValue(xyz, value);
-        mRoot->updateCache(xyz, *mParent);
-    }
-    ///////////////////////////////////////////////////////////////////////////
+
     void setValue(const Coord& xyz, const ValueType& value)
     {
         assert(mRoot);
@@ -1357,7 +1276,7 @@ private:
     template<typename, Index> friend class LeafNode;
     // Allow trees to deregister themselves.
     template<typename> friend class Tree;
-    
+
     // This private method is merely for convenience.
     inline ValueAccessor1& self() const { return const_cast<ValueAccessor1&>(*this); }
 
@@ -1717,7 +1636,7 @@ private:
     template<typename, Index> friend class LeafNode;
     // Allow trees to deregister themselves.
     template<typename> friend class Tree;
-    
+
     // This private method is merely for convenience.
     inline ValueAccessor2& self() const { return const_cast<ValueAccessor2&>(*this); }
 
@@ -2142,7 +2061,7 @@ private:
     template<typename, Index> friend class LeafNode;
     // Allow trees to deregister themselves.
     template<typename> friend class Tree;
-    
+
     // This private method is merely for convenience.
     inline ValueAccessor3& self() const { return const_cast<ValueAccessor3&>(*this); }
 
@@ -2238,6 +2157,6 @@ private:
 
 #endif // OPENVDB_TREE_VALUEACCESSOR_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
