@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -37,7 +37,7 @@
     CPPUNIT_ASSERT_DOUBLES_EQUAL((expected), (actual), /*tolerance=*/0.0);
 
 namespace {
-const int GRID_DIM = 20;
+const int GRID_DIM = 10;
 }
 
 
@@ -454,7 +454,7 @@ TestDivergence::testWSDivergence()
         VectorGrid::Ptr inGrid = VectorGrid::create();
         math::MapBase::Ptr base_map( new math::ScaleMap(voxel_sizes));
         // apply rotation
-        math::MapBase::Ptr rotated_map = base_map->rotate(1.5, math::X_AXIS);
+        math::MapBase::Ptr rotated_map = base_map->preRotate(1.5, math::X_AXIS);
         inGrid->setTransform(math::Transform::Ptr(new math::Transform(rotated_map)));
 
         VectorTree& inTree = inGrid->tree();
@@ -600,7 +600,7 @@ TestDivergence::testWSDivergenceStencil()
         VectorGrid::Ptr inGrid = VectorGrid::create();
         math::MapBase::Ptr base_map( new math::ScaleMap(voxel_sizes));
         // apply rotation
-        math::MapBase::Ptr rotated_map = base_map->rotate(1.5, math::X_AXIS);
+        math::MapBase::Ptr rotated_map = base_map->preRotate(1.5, math::X_AXIS);
         inGrid->setTransform(math::Transform::Ptr(new math::Transform(rotated_map)));
 
         VectorTree& inTree = inGrid->tree();
@@ -651,6 +651,6 @@ TestDivergence::testWSDivergenceStencil()
     }
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
