@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -74,7 +74,6 @@ CPPUNIT_TEST_SUITE_REGISTRATION(TestGradient);
 void
 TestGradient::testISGradient()
 {
-
     using namespace openvdb;
 
     typedef FloatGrid::ConstAccessor  AccessorType;
@@ -132,7 +131,6 @@ TestGradient::testISGradient()
 void
 TestGradient::testISGradientStencil()
 {
-
     using namespace openvdb;
 
     typedef FloatGrid::ConstAccessor  AccessorType;
@@ -192,11 +190,9 @@ TestGradient::testISGradientStencil()
 }
 
 
-
 void
 TestGradient::testWSGradient()
 {
-
     using namespace openvdb;
 
     typedef FloatGrid::ConstAccessor  AccessorType;
@@ -285,7 +281,7 @@ TestGradient::testWSGradient()
         Vec3d voxel_sizes(0.25, 0.45, 0.75);
         math::MapBase::Ptr base_map( new math::ScaleMap(voxel_sizes));
         // apply rotation
-        rotated_map = base_map->rotate(1.5, math::X_AXIS);
+        rotated_map = base_map->preRotate(1.5, math::X_AXIS);
         grid->setTransform(math::Transform::Ptr(new math::Transform(rotated_map)));
         // remake the sphere
         unittest_util::makeSphere<FloatGrid>(
@@ -314,7 +310,6 @@ TestGradient::testWSGradient()
         result = math::Gradient<math::ScaleMap, math::CD_2ND>::result(*scale_map, inAccessor, xyz);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
     }
-
 }
 
 
@@ -417,7 +412,7 @@ TestGradient::testWSGradientStencil()
         Vec3d voxel_sizes(0.25, 0.45, 0.75);
         math::MapBase::Ptr base_map( new math::ScaleMap(voxel_sizes));
         // apply rotation
-        rotated_map = base_map->rotate(1.5, math::X_AXIS);
+        rotated_map = base_map->preRotate(1.5, math::X_AXIS);
         grid->setTransform(math::Transform::Ptr(new math::Transform(rotated_map)));
         // remake the sphere
         unittest_util::makeSphere<FloatGrid>(
@@ -444,13 +439,12 @@ TestGradient::testWSGradientStencil()
         result = math::Gradient<math::ScaleMap, math::CD_2ND>::result(map, dense_2ndOrder);
         CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, result.length(), /*tolerance=*/0.01);
     }
-
 }
 
+
 void
 TestGradient::testWSGradientNormSqr()
 {
-
     using namespace openvdb;
 
     typedef FloatGrid::ConstAccessor  AccessorType;
@@ -496,7 +490,6 @@ TestGradient::testWSGradientNormSqr()
 void
 TestGradient::testWSGradientNormSqrStencil()
 {
-
     using namespace openvdb;
 
     typedef FloatGrid::ConstAccessor  AccessorType;
@@ -549,10 +542,10 @@ TestGradient::testWSGradientNormSqrStencil()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, normsqrd, /*tolerance=*/0.05);
 }
 
+
 void
 TestGradient::testGradientTool()
 {
-
     using namespace openvdb;
 
     typedef FloatGrid::ConstAccessor  AccessorType;
@@ -575,10 +568,12 @@ TestGradient::testGradientTool()
         /*tolerance=*/0.01);
 }
 
+
 void
 TestGradient::testIntersectsIsoValue()
 {
     using namespace openvdb;
+
     {// test zero crossing in -x
         FloatGrid  grid(/*backgroundValue=*/5.0);
         FloatTree& tree = grid.tree();
@@ -668,6 +663,7 @@ TestGradient::testIntersectsIsoValue()
     }
 }
 
+
 void
 TestGradient::testOldStyleStencils()
 {
@@ -699,9 +695,8 @@ TestGradient::testOldStyleStencils()
     math::CurvatureStencil<FloatGrid> cs(*grid);
     cs.moveTo(xyz);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, cs.gradient().length(), /*tolerance=*/0.01);
-
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
