@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -82,6 +82,7 @@ public:
     typedef boost::shared_ptr<const ProxyTree> ConstPtr;
 
     static const openvdb::Index DEPTH = 0;
+    static const ValueType backg = 0;
 
     ProxyTree() {}
     ProxyTree(const ValueType&) {}
@@ -90,7 +91,7 @@ public:
     static const openvdb::Name& treeType() { static const openvdb::Name s("proxy"); return s; }
     virtual const openvdb::Name& type() const { return treeType(); }
     virtual openvdb::Name valueType() const { return "proxy"; }
-    ValueType getBackground() const { return 0; }
+    const ValueType& background() const { return backg; }
 
     virtual TreeBasePtr copy() const { return TreeBasePtr(new ProxyTree(*this)); }
 
@@ -266,6 +267,6 @@ TestGrid::testCopyGrid()
     ASSERT_DOUBLES_EXACTLY_EQUAL(1.0f, tree2.getValue(changeCoord));
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
