@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,7 +31,6 @@
 #include <cppunit/extensions/HelperMacros.h>
 #include <openvdb/Exceptions.h>
 #include <openvdb/math/Maps.h>
-#include <openvdb/math/Frustum.h>
 #include <openvdb/util/MapsUtil.h>
 
 
@@ -295,13 +294,13 @@ TestMaps::testFrustum()
     Vec3d trans(2,2,2);
     NonlinearFrustumMap::Ptr map =
         boost::static_pointer_cast<NonlinearFrustumMap, MapBase>(
-            frustum.preScale(Vec3d(10,10,10))->translate(trans));
+            frustum.preScale(Vec3d(10,10,10))->postTranslate(trans));
 
     CPPUNIT_ASSERT(!map->hasUniformScale());
 
     Vec3d result;
     result = map->voxelSize();
-    
+
     CPPUNIT_ASSERT( isApproxEqual(result.x(), 0.1));
     CPPUNIT_ASSERT( isApproxEqual(result.y(), 0.1));
     CPPUNIT_ASSERT( isApproxEqual(result.z(), 0.5, 0.0001));
@@ -398,15 +397,6 @@ TestMaps::testFrustum()
     CPPUNIT_ASSERT( isApproxEqual( side * (up.length()), up));
     // check that the linear map inside the frustum is a simple affine map (i.e. has no shear)
     CPPUNIT_ASSERT(frustumMap_from_camera.hasSimpleAffine());
-
-    //-- The Frustum Class has been deprecated
-    // // construct a frustum from the frustum map
-//     Frustum<double> frustum_from_map(frustumMap_from_camera);
-//     // construct the same frustum directly
-//     Vec3d left = up.cross(direction);
-//     left.normalize();
-//     left *= aspect * up.length();
-//     Frustum<double> frustum_from_camera(position, direction, up, left, z_near, z_near+depth);
 }
 
 
@@ -500,6 +490,6 @@ TestMaps::testCalcBoundingBox()
      }
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
