@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -304,7 +304,7 @@ TestMeanCurvature::testWSMeanCurvature()
         FloatGrid::Ptr grid = FloatGrid::create();
         math::MapBase::Ptr base_map( new math::ScaleMap(voxel_sizes));
         // apply rotation
-        math::MapBase::Ptr rotated_map = base_map->rotate(1.5, math::X_AXIS);
+        math::MapBase::Ptr rotated_map = base_map->preRotate(1.5, math::X_AXIS);
         grid->setTransform(math::Transform::Ptr(new math::Transform(rotated_map)));
         CPPUNIT_ASSERT(grid->empty());
 
@@ -470,7 +470,7 @@ TestMeanCurvature::testWSMeanCurvatureStencil()
         FloatGrid::Ptr grid = FloatGrid::create();
         math::MapBase::Ptr base_map( new math::ScaleMap(voxel_sizes));
         // apply rotation
-        math::MapBase::Ptr rotated_map = base_map->rotate(1.5, math::X_AXIS);
+        math::MapBase::Ptr rotated_map = base_map->preRotate(1.5, math::X_AXIS);
         grid->setTransform(math::Transform::Ptr(new math::Transform(rotated_map)));
         CPPUNIT_ASSERT(grid->empty());
 
@@ -572,6 +572,6 @@ TestMeanCurvature::testOldStyleStencils()
     }
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
