@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -32,13 +32,13 @@
 #include <openvdb/Exceptions.h>
 #include <openvdb/math/Math.h>
 #include <openvdb/math/Quat.h>
+#include <openvdb/math/Mat4.h>
 
 using namespace openvdb::math;
 
 class TestQuat: public CppUnit::TestCase
 {
 public:
-
     CPPUNIT_TEST_SUITE( TestQuat );
     CPPUNIT_TEST( testConstructor );
     CPPUNIT_TEST( testAxisAngle );
@@ -56,13 +56,12 @@ public:
     void testOpMultiply();
     void testInvert();
     void testEulerAngles();
-
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestQuat);
 
 
-void 
+void
 TestQuat::testConstructor()
 {
     {
@@ -72,7 +71,7 @@ TestQuat::testConstructor()
         CPPUNIT_ASSERT( isExactlyEqual(qq.z(), 3.45f) );
         CPPUNIT_ASSERT( isExactlyEqual(qq.w(), 4.56f) );
     }
-    
+
     {
         float a[] = {1.23, 2.34, 3.45, 4.56};
         Quat<float> qq(a);
@@ -81,41 +80,39 @@ TestQuat::testConstructor()
         CPPUNIT_ASSERT( isExactlyEqual(qq.z(), 3.45f) );
         CPPUNIT_ASSERT( isExactlyEqual(qq.w(), 4.56f) );
     }
-    
 }
 
-void 
+
+void
 TestQuat::testAxisAngle()
 {
     float TOL = 1e-6;
 
-    
     Quat<float> q1(1  , 2  , 3  , 4  );
     Quat<float> q2(1.2, 2.3, 3.4, 4.5);
 
     Vec3f v(1, 2, 3);
     v.normalize();
     float a = M_PI/4;
-    
+
     Quat<float> q(v,a);
     float b = q.angle();
     Vec3f vv = q.axis();
- 
+
     CPPUNIT_ASSERT( isApproxEqual(a, b, TOL) );
-    CPPUNIT_ASSERT( v.eq(vv) );
-    
+    CPPUNIT_ASSERT( v.eq(vv, TOL) );
+
     q1.setAxisAngle(v,a);
     b = q1.angle();
     vv = q1.axis();
     CPPUNIT_ASSERT( isApproxEqual(a, b, TOL) );
-    CPPUNIT_ASSERT( v.eq(vv) );
+    CPPUNIT_ASSERT( v.eq(vv, TOL) );
 }
 
-void 
+
+void
 TestQuat::testOpPlus()
 {
-
-    
     Quat<float> q1(1  , 2  , 3  , 4  );
     Quat<float> q2(1.2, 2.3, 3.4, 4.5);
 
@@ -127,14 +124,14 @@ TestQuat::testOpPlus()
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), y) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), z) );
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), w) );
-    
+
     q = q1;
     q += q2;
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), x) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), y) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), z) );
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), w) );
-    
+
     q.add(q1,q2);
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), x) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), y) );
@@ -142,30 +139,29 @@ TestQuat::testOpPlus()
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), w) );
 }
 
-void 
+
+void
 TestQuat::testOpMinus()
 {
-    
     Quat<float> q1(1  , 2  , 3  , 4  );
     Quat<float> q2(1.2, 2.3, 3.4, 4.5);
-    
 
     Quat<float> q = q1 - q2;
-    
+
     float
         x=q1.x()-q2.x(), y=q1.y()-q2.y(), z=q1.z()-q2.z(), w=q1.w()-q2.w();
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), x) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), y) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), z) );
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), w) );
-    
+
     q = q1;
     q -= q2;
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), x) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), y) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), z) );
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), w) );
-    
+
     q.sub(q1,q2);
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), x) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), y) );
@@ -173,16 +169,15 @@ TestQuat::testOpMinus()
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), w) );
 }
 
-void 
+
+void
 TestQuat::testOpMultiply()
 {
-
-    
     Quat<float> q1(1  , 2  , 3  , 4  );
     Quat<float> q2(1.2, 2.3, 3.4, 4.5);
 
     Quat<float> q = q1 * 1.5f;
-    
+
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), float(1.5f)*q1.x()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), float(1.5f)*q1.y()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), float(1.5f)*q1.z()) );
@@ -194,47 +189,125 @@ TestQuat::testOpMultiply()
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), float(1.5f)*q1.y()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), float(1.5f)*q1.z()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), float(1.5f)*q1.w()) );
-    
+
     q.scale(1.5f, q1);
     CPPUNIT_ASSERT( isExactlyEqual(q.x(), float(1.5f)*q1.x()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.y(), float(1.5f)*q1.y()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.z(), float(1.5f)*q1.z()) );
     CPPUNIT_ASSERT( isExactlyEqual(q.w(), float(1.5f)*q1.w()) );
-    
 }
 
 
-void 
+void
 TestQuat::testInvert()
 {
     float TOL = 1e-6;
-    
+
     Quat<float> q1(1  , 2  , 3  , 4  );
     Quat<float> q2(1.2, 2.3, 3.4, 4.5);
 
 
     q1 = q2;
     q2 = q2.inverse();
-    
+
     Quat<float> q = q1*q2;
- 
+
     CPPUNIT_ASSERT( q.eq( Quat<float>(0,0,0,1), TOL ) );
-    
+
     q1.normalize();
     q2 = q1.conjugate();
     q = q1*q2;
     CPPUNIT_ASSERT( q.eq( Quat<float>(0,0,0,1), TOL ) );
 }
 
-void 
+
+void
 TestQuat::testEulerAngles()
 {
-    const Quat<float> rot(X_AXIS, 1.0);
-    Vec3f result = rot.eulerAngles(XZY_ROTATION);
-    CPPUNIT_ASSERT_EQUAL(result, Vec3f(1,0,0));
+    
+    {
+        double TOL = 1e-7;
+
+        Mat4d rx, ry, rz;
+        const double angle1 = 20. * M_PI / 180.;
+        const double angle2 = 64. * M_PI / 180.; 
+        const double angle3 = 125. *M_PI / 180.; 
+        rx.setToRotation(Vec3d(1,0,0), angle1);
+        ry.setToRotation(Vec3d(0,1,0), angle2);
+        rz.setToRotation(Vec3d(0,0,1), angle3);
+        
+        Mat4d r = rx * ry * rz;
+        
+        const Quat<double> rot(r.getMat3());
+        Vec3d result = rot.eulerAngles(ZYX_ROTATION);
+       
+        rx.setToRotation(Vec3d(1,0,0), result[0]);
+        ry.setToRotation(Vec3d(0,1,0), result[1]);
+        rz.setToRotation(Vec3d(0,0,1), result[2]);
+        
+        Mat4d rtest = rx * ry * rz;
+
+        CPPUNIT_ASSERT(r.eq(rtest, TOL));
+    }
+
+    {
+        double TOL = 1e-7;
+
+        Mat4d rx, ry, rz;
+        const double angle1 = 20. * M_PI / 180.;
+        const double angle2 = 64. * M_PI / 180.; 
+        const double angle3 = 125. *M_PI / 180.; 
+        rx.setToRotation(Vec3d(1,0,0), angle1);
+        ry.setToRotation(Vec3d(0,1,0), angle2);
+        rz.setToRotation(Vec3d(0,0,1), angle3);
+        
+        Mat4d r = rz * ry * rx;
+        
+        const Quat<double> rot(r.getMat3());
+        Vec3d result = rot.eulerAngles(XYZ_ROTATION);
+       
+        rx.setToRotation(Vec3d(1,0,0), result[0]);
+        ry.setToRotation(Vec3d(0,1,0), result[1]);
+        rz.setToRotation(Vec3d(0,0,1), result[2]);
+        
+        Mat4d rtest = rz * ry * rx;
+
+        CPPUNIT_ASSERT(r.eq(rtest, TOL));
+    }
+
+    {
+        double TOL = 1e-7;
+
+        Mat4d rx, ry, rz;
+        const double angle1 = 20. * M_PI / 180.;
+        const double angle2 = 64. * M_PI / 180.; 
+        const double angle3 = 125. *M_PI / 180.; 
+        rx.setToRotation(Vec3d(1,0,0), angle1);
+        ry.setToRotation(Vec3d(0,1,0), angle2);
+        rz.setToRotation(Vec3d(0,0,1), angle3);
+        
+        Mat4d r = rz * rx * ry;
+        
+        const Quat<double> rot(r.getMat3());
+        Vec3d result = rot.eulerAngles(YXZ_ROTATION);
+       
+        rx.setToRotation(Vec3d(1,0,0), result[0]);
+        ry.setToRotation(Vec3d(0,1,0), result[1]);
+        rz.setToRotation(Vec3d(0,0,1), result[2]);
+        
+        Mat4d rtest = rz * rx * ry;
+
+        CPPUNIT_ASSERT(r.eq(rtest, TOL));
+    }
+    
+    {
+        const Quat<float> rot(X_AXIS, 1.0);
+        Vec3f result = rot.eulerAngles(XZY_ROTATION);
+        CPPUNIT_ASSERT_EQUAL(result, Vec3f(1,0,0));
+    }
+    
 }
 
-
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
