@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,9 +31,9 @@
 #include <sstream>
 #include <cppunit/extensions/HelperMacros.h>
 #include <boost/random/mersenne_twister.hpp>
+#include <tbb/atomic.h>
 #include <openvdb/Types.h>
 #include <openvdb/openvdb.h>
-#include <openvdb/tools/Cpt.h>
 #include <openvdb/tools/GridOperators.h>
 #include <openvdb/tools/Filter.h>
 #include <openvdb/tools/LevelSetUtil.h>
@@ -57,6 +57,7 @@ public:
 
     CPPUNIT_TEST_SUITE(TestTools);
     CPPUNIT_TEST(testDilateVoxels);
+    CPPUNIT_TEST(testErodeVoxels);
     CPPUNIT_TEST(testFilter);
     CPPUNIT_TEST(testFloatApply);
     CPPUNIT_TEST(testLevelSetSphere);
@@ -71,6 +72,7 @@ public:
     CPPUNIT_TEST_SUITE_END();
 
     void testDilateVoxels();
+    void testErodeVoxels();
     void testFilter();
     void testFloatApply();
     void testLevelSetSphere();
@@ -102,9 +104,10 @@ TestTools::testDilateVoxels()
 
     const openvdb::Index leafDim = Tree543f::LeafNodeType::DIM;
     CPPUNIT_ASSERT_EQUAL(1 << 3, int(leafDim));
-    
+
     {
         // Set and dilate a single voxel at the center of a leaf node.
+        tree->clear();
         tree->setValue(Coord(leafDim >> 1), 1.0);
         CPPUNIT_ASSERT_EQUAL(Index64(1), tree->activeVoxelCount());
         openvdb::tools::dilateVoxels(*tree);
@@ -112,15 +115,19 @@ TestTools::testDilateVoxels()
     }
     {
         // Create an active, leaf node-sized tile.
+        tree->clear();
         tree->fill(CoordBBox(Coord(0), Coord(leafDim - 1)), 1.0);
         CPPUNIT_ASSERT_EQUAL(Index32(0), tree->leafCount());
         CPPUNIT_ASSERT_EQUAL(Index64(leafDim * leafDim * leafDim), tree->activeVoxelCount());
 
         tree->setValue(Coord(leafDim, leafDim - 1, leafDim - 1), 1.0);
+        CPPUNIT_ASSERT_EQUAL(Index64(leafDim * leafDim * leafDim + 1),
+                             tree->activeVoxelCount());
+
         openvdb::tools::dilateVoxels(*tree);
 
         CPPUNIT_ASSERT_EQUAL(Index64(leafDim * leafDim * leafDim + 1 + 5),
-            tree->activeVoxelCount());
+                             tree->activeVoxelCount());
     }
     {
         // Set and dilate a single voxel at each of the eight corners of a leaf node.
@@ -174,10 +181,10 @@ TestTools::testDilateVoxels()
             openvdb::tools::dilateVoxels(*tree);
         }
     }
-    
+
     {// dialte a narrow band of a sphere
         typedef openvdb::Grid<Tree543f> GridType;
-        GridType grid(tree->getBackground());
+        GridType grid(tree->background());
         unittest_util::makeSphere<GridType>(/*dim=*/openvdb::Coord(64, 64, 64),
                                             /*center=*/openvdb::Vec3f(0, 0, 0),
                                             /*radius=*/20, grid, /*dx=*/1.0f,
@@ -186,10 +193,10 @@ TestTools::testDilateVoxels()
         openvdb::tools::dilateVoxels(grid.tree());
         CPPUNIT_ASSERT(grid.tree().activeVoxelCount() > count);
     }
-    
+
     {// dilate a fog volume of a sphere
         typedef openvdb::Grid<Tree543f> GridType;
-        GridType grid(tree->getBackground());
+        GridType grid(tree->background());
         unittest_util::makeSphere<GridType>(/*dim=*/openvdb::Coord(64, 64, 64),
                                             /*center=*/openvdb::Vec3f(0, 0, 0),
                                             /*radius=*/20, grid, /*dx=*/1.0f,
@@ -218,6 +225,174 @@ TestTools::testDilateVoxels()
 //     }
 }
 
+void
+TestTools::testErodeVoxels()
+{
+    using openvdb::CoordBBox;
+    using openvdb::Coord;
+    using openvdb::Index32;
+    using openvdb::Index64;
+
+    typedef openvdb::tree::Tree4<float, 5, 4, 3>::Type TreeType;
+
+    TreeType::Ptr tree(new TreeType);
+    tree->setBackground(/*background=*/5.0);
+    CPPUNIT_ASSERT(tree->empty());
+
+    const int leafDim = TreeType::LeafNodeType::DIM;
+    CPPUNIT_ASSERT_EQUAL(1 << 3, leafDim);
+
+    {
+        // Set, dilate and erode a single voxel at the center of a leaf node.
+        tree->clear();
+        CPPUNIT_ASSERT_EQUAL(0, int(tree->activeVoxelCount()));
+
+        tree->setValue(Coord(leafDim >> 1), 1.0);
+        CPPUNIT_ASSERT_EQUAL(1, int(tree->activeVoxelCount()));
+
+        openvdb::tools::dilateVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(7, int(tree->activeVoxelCount()));
+
+        openvdb::tools::erodeVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(1, int(tree->activeVoxelCount()));
+
+        openvdb::tools::erodeVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(0, int(tree->activeVoxelCount()));
+    }
+    {
+        // Create an active, leaf node-sized tile.
+        tree->clear();
+        tree->fill(CoordBBox(Coord(0), Coord(leafDim - 1)), 1.0);
+        CPPUNIT_ASSERT_EQUAL(0, int(tree->leafCount()));
+        CPPUNIT_ASSERT_EQUAL(leafDim * leafDim * leafDim, int(tree->activeVoxelCount()));
+
+        tree->setValue(Coord(leafDim, leafDim - 1, leafDim - 1), 1.0);
+        CPPUNIT_ASSERT_EQUAL(1, int(tree->leafCount()));
+        CPPUNIT_ASSERT_EQUAL(leafDim * leafDim * leafDim + 1,int(tree->activeVoxelCount()));
+
+        openvdb::tools::dilateVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(3, int(tree->leafCount()));
+        CPPUNIT_ASSERT_EQUAL(leafDim * leafDim * leafDim + 1 + 5,int(tree->activeVoxelCount()));
+
+        openvdb::tools::erodeVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(1, int(tree->leafCount()));
+        CPPUNIT_ASSERT_EQUAL(leafDim * leafDim * leafDim + 1, int(tree->activeVoxelCount()));
+    }
+    {
+        // Set and dilate a single voxel at each of the eight corners of a leaf node.
+        for (int i = 0; i < 8; ++i) {
+            tree->clear();
+
+            openvdb::Coord xyz(
+                i & 1 ? leafDim - 1 : 0,
+                i & 2 ? leafDim - 1 : 0,
+                i & 4 ? leafDim - 1 : 0);
+            tree->setValue(xyz, 1.0);
+            CPPUNIT_ASSERT_EQUAL(1, int(tree->activeVoxelCount()));
+
+            openvdb::tools::dilateVoxels(*tree);
+            CPPUNIT_ASSERT_EQUAL(7, int(tree->activeVoxelCount()));
+
+            openvdb::tools::erodeVoxels(*tree);
+            CPPUNIT_ASSERT_EQUAL(1, int(tree->activeVoxelCount()));
+        }
+    }
+    {
+        // Set three active voxels and dilate and erode
+        tree->clear();
+        tree->setValue(Coord(0), 1.0);
+        tree->setValue(Coord( 1, 0, 0), 1.0);
+        tree->setValue(Coord(-1, 0, 0), 1.0);
+        CPPUNIT_ASSERT_EQUAL(3, int(tree->activeVoxelCount()));
+
+        openvdb::tools::dilateVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(17, int(tree->activeVoxelCount()));
+
+        openvdb::tools::erodeVoxels(*tree);
+        CPPUNIT_ASSERT_EQUAL(3, int(tree->activeVoxelCount()));
+    }
+    {
+        struct Info {
+            void test(TreeType::Ptr tree) {
+                CPPUNIT_ASSERT_EQUAL(activeVoxelCount, int(tree->activeVoxelCount()));
+                CPPUNIT_ASSERT_EQUAL(leafCount,        int(tree->leafCount()));
+                CPPUNIT_ASSERT_EQUAL(nonLeafCount,     int(tree->nonLeafCount()));
+            }
+            int activeVoxelCount, leafCount, nonLeafCount;
+        };
+        Info iterInfo[12] = {
+            { 0,     0,  1 },//an empty tree only contains a root node
+            { 1,     1,  3 },
+            { 7,     1,  3 },
+            { 25,    1,  3 },
+            { 63,    1,  3 },
+            { 129,   4,  3 },
+            { 231,   7,  9 },
+            { 377,   7,  9 },
+            { 575,   7,  9 },
+            { 833,  10,  9 },
+            { 1159, 16,  9 },
+            { 1561, 19, 15 },
+        };
+
+        // Perform repeated dilations, starting with a single voxel.
+        tree->clear();
+        iterInfo[0].test(tree);
+
+        tree->setValue(Coord(leafDim >> 1), 1.0);
+        iterInfo[1].test(tree);
+
+        for (int i = 2; i < 12; ++i) {
+            openvdb::tools::dilateVoxels(*tree);
+            iterInfo[i].test(tree);
+        }
+        for (int i = 10; i >= 0; --i) {
+            openvdb::tools::erodeVoxels(*tree);
+            iterInfo[i].test(tree);
+        }
+
+        // Now try it using the resursive calls
+        for (int i = 2; i < 12; ++i) {
+            tree->clear();
+            tree->setValue(Coord(leafDim >> 1), 1.0);
+            openvdb::tools::dilateVoxels(*tree, i-1);
+            iterInfo[i].test(tree);
+        }
+        for (int i = 10; i >= 0; --i) {
+            tree->clear();
+            tree->setValue(Coord(leafDim >> 1), 1.0);
+            openvdb::tools::dilateVoxels(*tree, 10);
+            openvdb::tools::erodeVoxels(*tree, 11-i);
+            iterInfo[i].test(tree);
+        }
+    }
+
+    {// erode a narrow band of a sphere
+        typedef openvdb::Grid<TreeType> GridType;
+        GridType grid(tree->background());
+        unittest_util::makeSphere<GridType>(/*dim=*/openvdb::Coord(64, 64, 64),
+                                            /*center=*/openvdb::Vec3f(0, 0, 0),
+                                            /*radius=*/20, grid, /*dx=*/1.0f,
+                                            unittest_util::SPHERE_DENSE_NARROW_BAND);
+        const openvdb::Index64 count = grid.tree().activeVoxelCount();
+        openvdb::tools::erodeVoxels(grid.tree());
+        CPPUNIT_ASSERT(grid.tree().activeVoxelCount() < count);
+    }
+
+    {// erode a fog volume of a sphere
+        typedef openvdb::Grid<TreeType> GridType;
+        GridType grid(tree->background());
+        unittest_util::makeSphere<GridType>(/*dim=*/openvdb::Coord(64, 64, 64),
+                                            /*center=*/openvdb::Vec3f(0, 0, 0),
+                                            /*radius=*/20, grid, /*dx=*/1.0f,
+                                            unittest_util::SPHERE_DENSE_NARROW_BAND);
+        openvdb::tools::sdfToFogVolume(grid);
+        const openvdb::Index64 count = grid.tree().activeVoxelCount();
+        openvdb::tools::erodeVoxels(grid.tree());
+        CPPUNIT_ASSERT(grid.tree().activeVoxelCount() < count);
+    }
+}
+
 void
 TestTools::testFilter()
 {
@@ -238,7 +413,7 @@ TestTools::testFilter()
         openvdb::FloatTree& tree = grid->tree();
         openvdb::tools::Filter<openvdb::FloatGrid> filter(*grid);
         const float offset = 2.34f;
-        filter.offset(offset,true/*=serial*/);
+        filter.offset(offset, true/*=serial*/);
         for (int x=0; x<dim[0]; ++x) {
             xyz[0]=x;
             for (int y=0; y<dim[1]; ++y) {
@@ -272,7 +447,7 @@ TestTools::testFilter()
         const int width = 2;
         openvdb::math::DenseStencil<openvdb::FloatGrid> stencil(*referenceGrid, width);
         openvdb::tools::Filter<openvdb::FloatGrid> filter(*filteredGrid);
-        filter.median(width, false/*=serial*/);
+        filter.median(width, /*interations=*/1, false/*=serial*/);
         std::vector<float> tmp;
         for (int x=0; x<dim[0]; ++x) {
             xyz[0]=x;
@@ -308,7 +483,7 @@ TestTools::testFilter()
         const int width = 2;
         openvdb::math::DenseStencil<openvdb::FloatGrid> stencil(*referenceGrid, width);
         openvdb::tools::Filter<openvdb::FloatGrid> filter(*filteredGrid);
-        filter.mean(width, false/*=serial*/);
+        filter.mean(width,  /*interations=*/1, false/*=serial*/);
         for (int x=0; x<dim[0]; ++x) {
             xyz[0]=x;
             for (int y=0; y<dim[1]; ++y) {
@@ -388,7 +563,7 @@ TestTools::testLevelSetAdvect()
     const int dim = 64;//256
     const openvdb::Vec3f center(0.35f, 0.35f, 0.35f);
     const float radius = 0.15f, voxelSize = 1.0f/(dim-1);
-    
+
     typedef openvdb::FloatGrid GridT;
     typedef openvdb::Vec3fGrid VectT;
 
@@ -434,14 +609,14 @@ TestTools::testLevelSetAdvect()
         GridT::Ptr grid = openvdb::tools::createLevelSetSphere<GridT>(radius, center, voxelSize);
         typedef openvdb::tools::EnrightField<float> FieldT;
         FieldT field;
-        
+
         typedef openvdb::tools::LevelSetAdvection<GridT, FieldT>  AdvectT;
         AdvectT advect(*grid, field);
         advect.setSpatialScheme(openvdb::math::HJWENO5_BIAS);
         advect.setTemporalScheme(openvdb::math::TVD_RK2);
         advect.setTrackerSpatialScheme(openvdb::math::HJWENO5_BIAS);
         advect.setTrackerTemporalScheme(openvdb::math::TVD_RK1);
-        
+
         FrameWriter fw(dim, grid); fw("Enright",0, 0);
         //for (float t = 0, dt = 0.005f; !grid->empty() && t < 3.0f; t += dt) {
         //    fw("Enright", t + dt, advect.advect(t, t + dt));
@@ -461,7 +636,7 @@ TestTools::testLevelSetAdvect()
         AdvectT advect(*grid, field);
         advect.setSpatialScheme(openvdb::math::HJWENO5_BIAS);
         advect.setTemporalScheme(openvdb::math::TVD_RK2);
-        
+
         FrameWriter fw(dim, grid); fw("Aligned",0, 0);
         //for (float t = 0, dt = 0.005f; !grid->empty() && t < 3.0f; t += dt) {
         //    fw("Aligned", t + dt, advect.advect(t, t + dt));
@@ -529,9 +704,9 @@ TestTools::testFloatApply()
     openvdb::tools::foreach(tree.begin<ValueIter>(), Local::visit, /*threaded=*/true);
 
     float expected = Local::op(background);
-    //CPPUNIT_ASSERT_DOUBLES_EQUAL(expected, tree.getBackground(), /*tolerance=*/0.0);
+    //CPPUNIT_ASSERT_DOUBLES_EQUAL(expected, tree.background(), /*tolerance=*/0.0);
     //expected = Local::op(-background);
-    //CPPUNIT_ASSERT_DOUBLES_EQUAL(expected, -tree.getBackground(), /*tolerance=*/0.0);
+    //CPPUNIT_ASSERT_DOUBLES_EQUAL(expected, -tree.background(), /*tolerance=*/0.0);
 
     for (openvdb::FloatTree::ValueOnCIter it = tree.cbeginValueOn(); it; ++it) {
         xyz = it.getCoord();
@@ -558,10 +733,10 @@ TestTools::testMagnitude()
     CPPUNIT_ASSERT(!tree.empty());
     CPPUNIT_ASSERT_EQUAL(dim[0]*dim[1]*dim[2], int(tree.activeVoxelCount()));
 
-    openvdb::tools::Gradient<openvdb::FloatGrid> gradient(*grid);
-    openvdb::VectorGrid::Ptr gradGrid = gradient.process();
-    openvdb::tools::Magnitude<openvdb::VectorGrid> magnitude(*gradGrid);
-    openvdb::FloatGrid::Ptr mag = magnitude.process();
+    openvdb::VectorGrid::Ptr gradGrid = openvdb::tools::gradient(*grid);
+    CPPUNIT_ASSERT_EQUAL(int(tree.activeVoxelCount()), int(gradGrid->activeVoxelCount()));
+
+    openvdb::FloatGrid::Ptr mag = openvdb::tools::magnitude(*gradGrid);
     CPPUNIT_ASSERT_EQUAL(int(tree.activeVoxelCount()), int(mag->activeVoxelCount()));
 
     openvdb::FloatGrid::ConstAccessor accessor = mag->getConstAccessor();
@@ -591,8 +766,7 @@ TestTools::testNormalize()
     CPPUNIT_ASSERT_EQUAL(dim[0]*dim[1]*dim[2], int(tree.activeVoxelCount()));
     openvdb::Coord xyz(10, 20, 30);
 
-    openvdb::tools::Gradient<openvdb::FloatGrid> gradient(*grid);
-    openvdb::VectorGrid::Ptr grad = gradient.process();
+    openvdb::VectorGrid::Ptr grad = openvdb::tools::gradient(*grid);
 
     typedef openvdb::VectorGrid::ValueType Vec3Type;
 
@@ -611,8 +785,7 @@ TestTools::testNormalize()
     Vec3Type v = accessor.getValue(xyz);
     //std::cerr << "\nPassed testNormalize(" << xyz << ")=" << v.length() << std::endl;
     CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0,v.length(),0.001);
-    openvdb::tools::Normalize<openvdb::VectorGrid> normalize(*grad);
-    openvdb::VectorGrid::Ptr norm = normalize.process();
+    openvdb::VectorGrid::Ptr norm = openvdb::tools::normalize(*grad);
 
     accessor = norm->getConstAccessor();
     v = accessor.getValue(xyz);
@@ -867,16 +1040,21 @@ struct FloatToVec {
     // Transform a scalar value into a vector value.
     static openvdb::Vec3s toVec(const ValueT& v) { return openvdb::Vec3s(v, v*2, v*3); }
 
-    void operator()(const InIterT& it, Accessor& acc) const
+    FloatToVec() { numTiles = 0; }
+
+    void operator()(const InIterT& it, Accessor& acc)
     {
         if (it.isVoxelValue()) { // set a single voxel
             acc.setValue(it.getCoord(), toVec(*it));
         } else { // fill an entire tile
+            numTiles.fetch_and_increment();
             openvdb::CoordBBox bbox;
             it.getBoundingBox(bbox);
             if (acc.getTree()) acc.getTree()->fill(bbox, toVec(*it));
         }
     }
+
+    tbb::atomic<int> numTiles;
 };
 
 }
@@ -912,22 +1090,29 @@ TestTools::testTransformValues()
     ftree.fill(CoordBBox(Coord(2048), Coord(2048 + 32 - 1)), 3 * 2048); // level-2 tile
     ftree.fill(CoordBBox(Coord(3072), Coord(3072 + 256 - 1)), 3 * 3072); // level-3 tile
 
-    FloatToVec<Tree323f::ValueOnCIter, Tree323v> op;
-    Tree323v vtree;
-    openvdb::tools::transformValues(ftree.cbeginValueOn(), vtree, op, /*threaded=*/true);
+    for (int shareOp = 0; shareOp <= 1; ++shareOp) {
+        FloatToVec<Tree323f::ValueOnCIter, Tree323v> op;
+        Tree323v vtree;
+        openvdb::tools::transformValues(ftree.cbeginValueOn(), vtree, op,
+            /*threaded=*/true, shareOp);
 
-    Vec3s expected;
-    for (Tree323v::ValueOnCIter it = vtree.cbeginValueOn(); it; ++it) {
-        xyz = it.getCoord();
-        expected = op.toVec(xyz[0] + xyz[1] + xyz[2]);
-        CPPUNIT_ASSERT_EQUAL(expected, it.getValue());
+        // The tile count is accurate only if the functor is shared.  Otherwise,
+        // it is initialized to zero in the main thread and never changed.
+        CPPUNIT_ASSERT_EQUAL(shareOp ? 3 : 0, int(op.numTiles));
+
+        Vec3s expected;
+        for (Tree323v::ValueOnCIter it = vtree.cbeginValueOn(); it; ++it) {
+            xyz = it.getCoord();
+            expected = op.toVec(xyz[0] + xyz[1] + xyz[2]);
+            CPPUNIT_ASSERT_EQUAL(expected, it.getValue());
+        }
+        // Check values inside the tiles.
+        CPPUNIT_ASSERT_EQUAL(op.toVec(3 * 1024), vtree.getValue(Coord(1024 + 4)));
+        CPPUNIT_ASSERT_EQUAL(op.toVec(3 * 2048), vtree.getValue(Coord(2048 + 16)));
+        CPPUNIT_ASSERT_EQUAL(op.toVec(3 * 3072), vtree.getValue(Coord(3072 + 128)));
     }
-    // Check values inside the tiles.
-    CPPUNIT_ASSERT_EQUAL(op.toVec(3 * 1024), vtree.getValue(Coord(1024 + 4)));
-    CPPUNIT_ASSERT_EQUAL(op.toVec(3 * 2048), vtree.getValue(Coord(2048 + 16)));
-    CPPUNIT_ASSERT_EQUAL(op.toVec(3 * 3072), vtree.getValue(Coord(3072 + 128)));
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
