@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -1461,9 +1461,8 @@ TestTree::testPruneLevelSet()
     //std::cerr << "Leaf count=" << tree.leafCount() << std::endl;
     CPPUNIT_ASSERT_EQUAL(tree.activeVoxelCount(), count-removed);
     CPPUNIT_ASSERT_EQUAL(tree.activeLeafVoxelCount(), count-removed);
-
-    openvdb::tree::LevelSetPrune<float> op;
-    tree.pruneOp(op);
+    
+    tree.pruneLevelSet();
 
     CPPUNIT_ASSERT(tree.leafCount() < leafCount);
     //std::cerr << "Leaf count=" << tree.leafCount() << std::endl;
@@ -1669,6 +1668,7 @@ TestTree::testLeafManager()
     unittest_util::makeSphere<FloatGrid>(
         Coord(dim), center, radius, *grid, unittest_util::SPHERE_SPARSE_NARROW_BAND);
     const size_t leafCount = tree.leafCount();
+
     //grid->print(std::cout, 3);
     {// test with no aux buffers
         openvdb::tree::LeafManager<FloatTree> r(tree);
@@ -1746,8 +1746,38 @@ TestTree::testLeafManager()
             CPPUNIT_ASSERT(r.getBuffer(n,1) == r.getBuffer(n,2));
         }
     }
+    {// test with const tree (buffers are not swappable)
+        openvdb::tree::LeafManager<const FloatTree> r(tree);
+
+        for (size_t numAuxBuffers = 0; numAuxBuffers <= 2; ++numAuxBuffers += 2) {
+            r.rebuildAuxBuffers(numAuxBuffers);
+
+            CPPUNIT_ASSERT_EQUAL(leafCount, r.leafCount());
+            CPPUNIT_ASSERT_EQUAL(int(numAuxBuffers * leafCount), int(r.auxBufferCount()));
+            CPPUNIT_ASSERT_EQUAL(numAuxBuffers, r.auxBuffersPerLeaf());
+
+            size_t n = 0;
+            for (FloatTree::LeafCIter iter = tree.cbeginLeaf(); iter; ++iter, ++n) {
+                CPPUNIT_ASSERT(r.leaf(n) == *iter);
+                // Verify that each aux buffer was initialized with a copy of the leaf buffer.
+                for (size_t bufIdx = 0; bufIdx < numAuxBuffers; ++bufIdx) {
+                    CPPUNIT_ASSERT(r.getBuffer(n, bufIdx) == iter->buffer());
+                }
+            }
+            CPPUNIT_ASSERT_EQUAL(r.leafCount(), n);
+
+            for (size_t i = 0; i < numAuxBuffers; ++i) {
+                for (size_t j = 0; j < numAuxBuffers; ++j) {
+                    // Verify that swapping buffers with themselves and swapping
+                    // leaf buffers with aux buffers have no effect.
+                    const bool canSwap = (i != j && i != 0 && j != 0);
+                    CPPUNIT_ASSERT_EQUAL(canSwap, r.swapBuffer(i, j));
+                }
+            }
+        }
+    }
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
