@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -71,7 +71,7 @@ TestTreeGetSetValues::testGetBackground()
     const float background = 256.0f;
     Tree323f tree(background);
 
-    ASSERT_DOUBLES_EXACTLY_EQUAL(background, tree.getBackground());
+    ASSERT_DOUBLES_EXACTLY_EQUAL(background, tree.background());
 }
 
 
@@ -459,6 +459,6 @@ TestTreeGetSetValues::testHasActiveTiles()
     CPPUNIT_ASSERT(tree.hasActiveTiles());
 }
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
